@@ -4,31 +4,37 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{path::PathBuf, sync::Arc};
+use std::{
+    net::{IpAddr, Ipv4Addr},
+    path::PathBuf,
+    sync::Arc,
+};
 
 use arc_swap::ArcSwap;
 use pwhash::sha512_crypt;
 use store::{
-    rand::{distributions::Alphanumeric, thread_rng, Rng},
     Stores,
+    rand::{Rng, distr::Alphanumeric, rng},
 };
-use tokio::sync::{mpsc, Notify};
+use tokio::sync::{Notify, Semaphore, mpsc};
 use utils::{
+    Semver, UnwrapFailure,
     config::{Config, ConfigKey},
-    failed, UnwrapFailure,
+    failed,
 };
 
 use crate::{
-    config::{server::Listeners, telemetry::Telemetry},
-    ipc::{DeliveryEvent, HousekeeperEvent, QueueEvent, ReportingEvent, StateEvent},
-    Core, Data, Inner, Ipc, IPC_CHANNEL_BUFFER,
+    Caches, Core, Data, IPC_CHANNEL_BUFFER, Inner, Ipc,
+    config::{network::AsnGeoLookupConfig, server::Listeners, telemetry::Telemetry},
+    core::BuildServer,
+    ipc::{HousekeeperEvent, QueueEvent, ReportingEvent, StateEvent},
 };
 
 use super::{
+    WEBADMIN_KEY,
     backup::BackupParams,
     config::{ConfigManager, Patterns},
     console::store_console,
-    WEBADMIN_KEY,
 };
 
 pub struct BootManager {
@@ -41,7 +47,6 @@ pub struct BootManager {
 pub struct IpcReceivers {
     pub state_rx: Option<mpsc::Receiver<StateEvent>>,
     pub housekeeper_rx: Option<mpsc::Receiver<HousekeeperEvent>>,
-    pub delivery_rx: Option<mpsc::Receiver<DeliveryEvent>>,
     pub queue_rx: Option<mpsc::Receiver<QueueEvent>>,
     pub report_rx: Option<mpsc::Receiver<ReportingEvent>>,
 }
@@ -190,18 +195,6 @@ impl BootManager {
             StoreOp::None => {
                 // Add hostname lookup if missing
                 let mut insert_keys = Vec::new();
-                if config
-                    .value("lookup.default.hostname")
-                    .filter(|v| !v.is_empty())
-                    .is_none()
-                {
-                    insert_keys.push(ConfigKey::from((
-                        "lookup.default.hostname",
-                        hostname::get()
-                            .map(|v| v.to_string_lossy().into_owned())
-                            .unwrap_or_else(|_| "localhost".to_string()),
-                    )));
-                }
 
                 // Generate an OAuth key if missing
                 if config
@@ -211,7 +204,7 @@ impl BootManager {
                 {
                     insert_keys.push(ConfigKey::from((
                         "oauth.key",
-                        thread_rng()
+                        rng()
                             .sample_iter(Alphanumeric)
                             .take(64)
                             .map(char::from)
@@ -227,7 +220,7 @@ impl BootManager {
                 {
                     insert_keys.push(ConfigKey::from((
                         "cluster.key",
-                        thread_rng()
+                        rng()
                             .sample_iter(Alphanumeric)
                             .take(64)
                             .map(char::from)
@@ -235,47 +228,96 @@ impl BootManager {
                     )));
                 }
 
-                // Download SPAM filters if missing
-                if config
-                    .value("version.spam-filter")
-                    .filter(|v| !v.is_empty())
-                    .is_none()
-                {
-                    match manager.fetch_config_resource("spam-filter").await {
-                        Ok(external_config) => {
-                            trc::event!(
-                                Config(trc::ConfigEvent::ImportExternal),
-                                Version = external_config.version,
-                                Id = "spam-filter"
-                            );
-                            insert_keys.extend(external_config.keys);
-                        }
-                        Err(err) => {
-                            config.new_build_error(
-                                "*",
-                                format!("Failed to fetch spam filter: {err}"),
-                            );
-                        }
+                // Download Spam filter rules if missing
+                // TODO remove this check in 1.0
+                let mut update_webadmin = match config.value("version.spam-filter").and_then(|v| {
+                    if !v.is_empty() {
+                        Some(Semver::try_from(v))
+                    } else {
+                        None
                     }
+                }) {
+                    Some(Err(_)) => {
+                        let _ = manager.clear_prefix("lookup.spam-").await;
+                        let _ = manager
+                            .clear_prefix("sieve.trusted.scripts.spam-filter")
+                            .await;
+                        let _ = manager
+                            .clear_prefix("sieve.trusted.scripts.track-replies")
+                            .await;
+                        let _ = manager.clear_prefix("sieve.trusted.scripts.greylist").await;
+                        let _ = manager.clear_prefix("sieve.trusted.scripts.train").await;
+                        //let _ = manager.clear_prefix("session.data.script").await;
+                        let _ = manager.clear("version.spam-filter").await;
 
-                    // Add default settings
-                    for key in [
-                        ("queue.quota.size.messages", "100000"),
-                        ("queue.quota.size.size", "10737418240"),
-                        ("queue.quota.size.enable", "true"),
-                        ("queue.throttle.rcpt.key", "rcpt_domain"),
-                        ("queue.throttle.rcpt.concurrency", "5"),
-                        ("queue.throttle.rcpt.enable", "true"),
-                        ("session.throttle.ip.key", "remote_ip"),
-                        ("session.throttle.ip.concurrency", "5"),
-                        ("session.throttle.ip.enable", "true"),
-                        ("session.throttle.sender.key.0", "sender_domain"),
-                        ("session.throttle.sender.key.1", "rcpt"),
-                        ("session.throttle.sender.rate", "25/1h"),
-                        ("session.throttle.sender.enable", "true"),
-                        ("report.analysis.addresses", "postmaster@*"),
-                    ] {
-                        insert_keys.push(ConfigKey::from(key));
+                        match manager.fetch_spam_rules().await {
+                            Ok(external_config) => {
+                                trc::event!(
+                                    Config(trc::ConfigEvent::ImportExternal),
+                                    Version = external_config.version.to_string(),
+                                    Id = "spam-filter"
+                                );
+                                insert_keys.extend(external_config.keys);
+                            }
+                            Err(err) => {
+                                config.new_build_error(
+                                    "*",
+                                    format!("Failed to fetch spam filter: {err}"),
+                                );
+                            }
+                        }
+
+                        true
+                    }
+                    Some(Ok(_)) => false,
+                    None => {
+                        match manager.fetch_spam_rules().await {
+                            Ok(external_config) => {
+                                trc::event!(
+                                    Config(trc::ConfigEvent::ImportExternal),
+                                    Version = external_config.version.to_string(),
+                                    Id = "spam-filter"
+                                );
+                                insert_keys.extend(external_config.keys);
+                            }
+                            Err(err) => {
+                                config.new_build_error(
+                                    "*",
+                                    format!("Failed to fetch spam filter: {err}"),
+                                );
+                            }
+                        }
+
+                        // Add default settings
+                        for key in [
+                            ("queue.quota.size.messages", "100000"),
+                            ("queue.quota.size.size", "10737418240"),
+                            ("queue.quota.size.enable", "true"),
+                            ("queue.limiter.inbound.ip.key", "remote_ip"),
+                            ("queue.limiter.inbound.ip.rate", "5/1s"),
+                            ("queue.limiter.inbound.ip.enable", "true"),
+                            ("queue.limiter.inbound.sender.key.0", "sender_domain"),
+                            ("queue.limiter.inbound.sender.key.1", "rcpt"),
+                            ("queue.limiter.inbound.sender.rate", "25/1h"),
+                            ("queue.limiter.inbound.sender.enable", "true"),
+                            ("report.analysis.addresses", "postmaster@*"),
+                        ] {
+                            insert_keys.push(ConfigKey::from(key));
+                        }
+
+                        false
+                    }
+                };
+
+                // TODO remove key migration in 1.0
+                for (old_key, new_key) in [
+                    ("lookup.default.hostname", "server.hostname"),
+                    ("lookup.default.domain", "report.domain"),
+                ] {
+                    if let (Some(old_value), None) = (config.value(old_key), config.value(new_key))
+                    {
+                        insert_keys.push(ConfigKey::from((new_key, old_value)));
+                        update_webadmin = true;
                     }
                 }
 
@@ -319,14 +361,14 @@ impl BootManager {
                         config.keys.insert(item.key.clone(), item.value.clone());
                     }
 
-                    if let Err(err) = manager.set(insert_keys).await {
+                    if let Err(err) = manager.set(insert_keys, true).await {
                         config
                             .new_build_error("*", format!("Failed to update configuration: {err}"));
                     }
                 }
 
-                // Parse lookup stores
-                stores.parse_lookups(&mut config).await;
+                // Parse in-memory stores
+                stores.parse_in_memory(&mut config, false).await;
 
                 // Parse settings
                 let core = Core::parse(&mut config, stores, manager).await;
@@ -334,6 +376,9 @@ impl BootManager {
                 // Parse data
                 let data = Data::parse(&mut config);
 
+                // Parse caches
+                let cache = Caches::parse(&mut config);
+
                 // Enable telemetry
                 #[cfg(feature = "enterprise")]
                 telemetry.enable(core.is_enterprise_edition());
@@ -346,9 +391,10 @@ impl BootManager {
                 );
 
                 // Webadmin auto-update
-                if config
-                    .property_or_default::<bool>("webadmin.auto-update", "false")
-                    .unwrap_or_default()
+                if update_webadmin
+                    || config
+                        .property_or_default::<bool>("webadmin.auto-update", "false")
+                        .unwrap_or_default()
                 {
                     if let Err(err) = data.webadmin.update(&core).await {
                         trc::event!(
@@ -359,14 +405,41 @@ impl BootManager {
                     }
                 }
 
+                // Spam filter auto-update
+                if config
+                    .property_or_default::<bool>("spam-filter.auto-update", "false")
+                    .unwrap_or_default()
+                {
+                    if let Err(err) = core.storage.config.update_spam_rules(false, false).await {
+                        trc::event!(
+                            Resource(trc::ResourceEvent::Error),
+                            Details = "Failed to update spam-filter",
+                            CausedBy = err
+                        );
+                    }
+                }
+
                 // Build shared inner
-                let (ipc, ipc_rxs) = build_ipc();
+                let has_remote_asn = matches!(
+                    core.network.asn_geo_lookup,
+                    AsnGeoLookupConfig::Resource { .. }
+                );
+                let (ipc, ipc_rxs) = build_ipc(&mut config);
                 let inner = Arc::new(Inner {
                     shared_core: ArcSwap::from_pointee(core),
                     data,
                     ipc,
+                    cache,
                 });
 
+                // Fetch ASN database
+                if has_remote_asn {
+                    inner
+                        .build_server()
+                        .lookup_asn_country(IpAddr::V4(Ipv4Addr::new(8, 8, 8, 8)))
+                        .await;
+                }
+
                 // Parse TCP acceptors
                 servers.parse_tcp_acceptors(&mut config, inner.clone());
 
@@ -408,9 +481,8 @@ impl BootManager {
     }
 }
 
-pub fn build_ipc() -> (Ipc, IpcReceivers) {
+pub fn build_ipc(config: &mut Config) -> (Ipc, IpcReceivers) {
     // Build ipc receivers
-    let (delivery_tx, delivery_rx) = mpsc::channel(IPC_CHANNEL_BUFFER);
     let (state_tx, state_rx) = mpsc::channel(IPC_CHANNEL_BUFFER);
     let (housekeeper_tx, housekeeper_rx) = mpsc::channel(IPC_CHANNEL_BUFFER);
     let (queue_tx, queue_rx) = mpsc::channel(IPC_CHANNEL_BUFFER);
@@ -419,15 +491,19 @@ pub fn build_ipc() -> (Ipc, IpcReceivers) {
         Ipc {
             state_tx,
             housekeeper_tx,
-            delivery_tx,
             queue_tx,
             report_tx,
             index_tx: Arc::new(Notify::new()),
+            local_delivery_sm: Arc::new(Semaphore::new(
+                config
+                    .property_or_default::<usize>("queue.threads.local", "10")
+                    .unwrap_or(10)
+                    .max(1),
+            )),
         },
         IpcReceivers {
             state_rx: Some(state_rx),
             housekeeper_rx: Some(housekeeper_rx),
-            delivery_rx: Some(delivery_rx),
             queue_rx: Some(queue_rx),
             report_rx: Some(report_rx),
         },
@@ -449,7 +525,7 @@ fn quickstart(path: impl Into<PathBuf>) {
     }
 
     let admin_pass = std::env::var("STALWART_ADMIN_PASSWORD").unwrap_or_else(|_| {
-        thread_rng()
+        rng()
             .sample_iter(Alphanumeric)
             .take(10)
             .map(char::from)
