@@ -7,15 +7,15 @@
 use std::{collections::hash_map::Entry, fmt, str::FromStr};
 
 use serde::{
+    Deserializer, Serializer,
     de::{self, IgnoredAny, Visitor},
     ser::SerializeMap,
-    Deserializer, Serializer,
 };
 use store::U64_LEN;
 
 use crate::{
+    Permission, Principal, ROLE_ADMIN, Type,
     backend::internal::{PrincipalField, PrincipalUpdate, PrincipalValue},
-    Permission, Principal, Type, ROLE_ADMIN,
 };
 
 impl Principal {
@@ -291,7 +291,7 @@ impl Principal {
     }
 
     pub fn has_str_value(&self, key: PrincipalField, value: &str) -> bool {
-        self.fields.get(&key).map_or(false, |v| match v {
+        self.fields.get(&key).is_some_and(|v| match v {
             PrincipalValue::String(v) => v == value,
             PrincipalValue::StringList(l) => l.iter().any(|v| v == value),
             PrincipalValue::Integer(_) | PrincipalValue::IntegerList(_) => false,
@@ -299,7 +299,7 @@ impl Principal {
     }
 
     pub fn has_int_value(&self, key: PrincipalField, value: u64) -> bool {
-        self.fields.get(&key).map_or(false, |v| match v {
+        self.fields.get(&key).is_some_and(|v| match v {
             PrincipalValue::Integer(v) => *v == value,
             PrincipalValue::IntegerList(l) => l.iter().any(|v| *v == value),
             PrincipalValue::String(_) | PrincipalValue::StringList(_) => false,
@@ -369,7 +369,10 @@ impl Principal {
 
     pub fn update_external(&mut self, mut external: Principal) -> Vec<PrincipalUpdate> {
         let mut updates = Vec::new();
-        if let Some(name) = external.take_str(PrincipalField::Description) {
+        if let Some(name) = external
+            .take_str(PrincipalField::Description)
+            .filter(|s| !s.is_empty())
+        {
             if self.get_str(PrincipalField::Description) != Some(name.as_str()) {
                 updates.push(PrincipalUpdate::set(
                     PrincipalField::Description,
@@ -1088,6 +1091,8 @@ impl Permission {
                 | Permission::SieveRenameScript
                 | Permission::SieveCheckScript
                 | Permission::SieveHaveSpace
+                | Permission::SpamFilterClassify
+                | Permission::SpamFilterTrain
         )
     }
 
