@@ -7,8 +7,8 @@
 use std::{iter::Peekable, sync::Arc, vec::IntoIter};
 
 use common::{
-    listener::{limiter::ConcurrencyLimiter, SessionResult, SessionStream},
-    ConcurrencyLimiters,
+    listener::{SessionResult, SessionStream},
+    KV_RATE_LIMIT_IMAP,
 };
 use imap_proto::{
     receiver::{self, Request},
@@ -111,7 +111,7 @@ impl<T: SessionStream> Session<T> {
                     .await
                     .map(|_| SessionResult::Continue),
                 Command::Status => self
-                    .handle_status(request)
+                    .handle_status(group_requests(&mut requests, vec![request]))
                     .await
                     .map(|_| SessionResult::Continue),
                 Command::Append => self
@@ -134,8 +134,8 @@ impl<T: SessionStream> Session<T> {
                     .handle_search(request, false, is_uid)
                     .await
                     .map(|_| SessionResult::Continue),
-                Command::Fetch(is_uid) => self
-                    .handle_fetch(request, is_uid)
+                Command::Fetch(_) => self
+                    .handle_fetch(group_requests(&mut requests, vec![request]))
                     .await
                     .map(|_| SessionResult::Continue),
                 Command::Store(is_uid) => self
@@ -230,6 +230,14 @@ impl<T: SessionStream> Session<T> {
                     .handle_my_rights(request)
                     .await
                     .map(|_| SessionResult::Continue),
+                Command::GetQuota => self
+                    .handle_get_quota(request)
+                    .await
+                    .map(|_| SessionResult::Continue),
+                Command::GetQuotaRoot => self
+                    .handle_get_quota_root(request)
+                    .await
+                    .map(|_| SessionResult::Continue),
                 Command::Unauthenticate => self
                     .handle_unauthenticate(request)
                     .await
@@ -292,7 +300,12 @@ impl<T: SessionStream> Session<T> {
                     .core
                     .storage
                     .lookup
-                    .is_rate_allowed(format!("ireq:{}", data.account_id).as_bytes(), rate, true)
+                    .is_rate_allowed(
+                        KV_RATE_LIMIT_IMAP,
+                        &data.account_id.to_be_bytes(),
+                        rate,
+                        true,
+                    )
                     .await?
                     .is_some()
                 {
@@ -366,7 +379,9 @@ impl<T: SessionStream> Session<T> {
             | Command::GetAcl
             | Command::ListRights
             | Command::MyRights
-            | Command::Unauthenticate => {
+            | Command::Unauthenticate
+            | Command::GetQuota
+            | Command::GetQuotaRoot => {
                 if let State::Authenticated { .. } | State::Selected { .. } = state {
                     Ok(request)
                 } else {
@@ -414,29 +429,6 @@ impl<T: SessionStream> Session<T> {
             },
         }
     }
-
-    pub fn get_concurrency_limiter(&self, account_id: u32) -> Option<Arc<ConcurrencyLimiters>> {
-        let rate = self.server.core.imap.rate_concurrent?;
-        self.server
-            .inner
-            .data
-            .imap_limiter
-            .get(&account_id)
-            .map(|limiter| limiter.clone())
-            .unwrap_or_else(|| {
-                let limiter = Arc::new(ConcurrencyLimiters {
-                    concurrent_requests: ConcurrencyLimiter::new(rate),
-                    concurrent_uploads: ConcurrencyLimiter::new(rate),
-                });
-                self.server
-                    .inner
-                    .data
-                    .imap_limiter
-                    .insert(account_id, limiter.clone());
-                limiter
-            })
-            .into()
-    }
 }
 
 impl<T: SessionStream> State<T> {
