@@ -11,12 +11,11 @@ use common::{
     AccountId, Mailbox,
 };
 use directory::{backend::internal::PrincipalField, QueryBy};
+use email::mailbox::{MailboxFnc, INBOX_ID};
 use imap_proto::protocol::list::Attribute;
 use jmap::{
     auth::acl::{AclMethods, EffectiveAcl},
     changes::get::ChangesLookup,
-    mailbox::{get::MailboxGet, set::MailboxSet, INBOX_ID},
-    JmapMethods,
 };
 use jmap_proto::{
     object::Object,
@@ -25,7 +24,6 @@ use jmap_proto::{
 use parking_lot::Mutex;
 use store::query::log::{Change, Query};
 use trc::AddContext;
-use utils::lru_cache::LruCached;
 
 use super::{Account, MailboxId, MailboxSync, Session, SessionData};
 
@@ -115,8 +113,8 @@ impl<T: SessionStream> SessionData<T> {
         if let Some(cached_account) = self
             .server
             .inner
-            .data
-            .account_cache
+            .cache
+            .account
             .get(&cached_account_id)
             .and_then(|cached_account| {
                 if cached_account.state_mailbox == state_mailbox
@@ -212,7 +210,18 @@ impl<T: SessionStream> SessionData<T> {
             mailbox_state: AHashMap::with_capacity(mailboxes.len()),
             state_mailbox,
             state_email,
+            obj_size: 0,
         };
+        account.obj_size = (std::mem::size_of::<Account>()
+            + account.prefix.as_ref().map_or(0, |p| p.len())
+            + account
+                .mailbox_names
+                .keys()
+                .map(|k| k.len() + std::mem::size_of::<u32>())
+                .sum::<usize>()
+            + (account.mailbox_state.len()
+                * (std::mem::size_of::<Mailbox>() + std::mem::size_of::<u32>())))
+            as u64;
 
         loop {
             while let Some((mailbox_id, mailbox_parent_id, mailbox)) = iter.next() {
@@ -262,7 +271,7 @@ impl<T: SessionStream> SessionData<T> {
                                 )
                                 .await
                                 .caused_by(trc::location!())?
-                                .map(|v| v.len() as u32)
+                                .map(|v| v.len())
                                 .unwrap_or(0)
                                 .into(),
                             total_unseen: self
@@ -270,7 +279,7 @@ impl<T: SessionStream> SessionData<T> {
                                 .mailbox_unread_tags(account_id, *mailbox_id, &message_ids)
                                 .await
                                 .caused_by(trc::location!())?
-                                .map(|v| v.len() as u32)
+                                .map(|v| v.len())
                                 .unwrap_or(0)
                                 .into(),
                             ..Default::default()
@@ -322,8 +331,8 @@ impl<T: SessionStream> SessionData<T> {
         // Update cache
         self.server
             .inner
-            .data
-            .account_cache
+            .cache
+            .account
             .insert(cached_account_id, Arc::new(account.clone()));
 
         Ok(account)
@@ -342,7 +351,7 @@ impl<T: SessionStream> SessionData<T> {
         // Obtain access token
         let access_token = self
             .server
-            .get_cached_access_token(self.account_id)
+            .get_access_token(self.account_id)
             .await
             .caused_by(trc::location!())?;
         let state = access_token.state();
@@ -469,17 +478,11 @@ impl<T: SessionStream> SessionData<T> {
                     }
 
                     // Update cache
-                    if let Some(cached_account_) = self
-                        .server
-                        .inner
-                        .data
-                        .account_cache
-                        .lock()
-                        .get_mut(&AccountId {
-                            account_id,
-                            primary_id: access_token.primary_id(),
-                        })
-                    {
+                    let ac_id = AccountId {
+                        account_id,
+                        primary_id: access_token.primary_id(),
+                    };
+                    if let Some(cached_account_) = self.server.inner.cache.account.get(&ac_id) {
                         if cached_account_.state_mailbox != state_mailbox
                             || cached_account_.state_email != state_email
                         {
@@ -493,7 +496,11 @@ impl<T: SessionStream> SessionData<T> {
                             });
                             cached_account.state_mailbox = state_mailbox;
                             cached_account.state_email = state_email;
-                            *cached_account_ = Arc::new(cached_account);
+                            self.server
+                                .inner
+                                .cache
+                                .account
+                                .insert(ac_id, Arc::new(cached_account));
                         }
                     }
                 } else {
@@ -599,7 +606,7 @@ impl<T: SessionStream> SessionData<T> {
             if account
                 .prefix
                 .as_ref()
-                .map_or(true, |p| mailbox_name.starts_with(p))
+                .is_none_or( |p| mailbox_name.starts_with(p))
             {
                 for (mailbox_name_, mailbox_id_) in account.mailbox_names.iter() {
                     if (!is_inbox && mailbox_name_ == mailbox_name)
