@@ -7,7 +7,11 @@
 use std::{sync::Arc, time::Instant};
 
 use common::{auth::AccessToken, listener::SessionStream, MailboxId};
-use directory::{backend::internal::PrincipalField, Permission, QueryBy};
+use directory::{
+    backend::internal::{manage::ChangedPrincipals, PrincipalField},
+    Permission, QueryBy, Type,
+};
+use email::mailbox::SCHEMA;
 use imap_proto::{
     protocol::acl::{
         Arguments, GetAclResponse, ListRightsResponse, ModRightsOp, MyRightsResponse, Rights,
@@ -16,10 +20,7 @@ use imap_proto::{
     Command, ResponseCode, StatusResponse,
 };
 
-use jmap::{
-    auth::acl::EffectiveAcl, changes::write::ChangeLog, mailbox::set::SCHEMA,
-    services::state::StateManager, JmapMethods,
-};
+use jmap::auth::acl::EffectiveAcl;
 use jmap_proto::{
     object::{index::ObjectIndexBuilder, Object},
     types::{
@@ -349,7 +350,8 @@ impl<T: SessionStream> Session<T> {
                 );
             if !batch.is_empty() {
                 data.server
-                    .write_batch(batch)
+                    .store()
+                    .write(batch)
                     .await
                     .imap_ctx(&arguments.tag, trc::location!())?;
                 let mut changes = ChangeLogBuilder::new();
@@ -368,7 +370,13 @@ impl<T: SessionStream> Session<T> {
             }
 
             // Invalidate ACLs
-            data.server.inner.data.access_tokens.remove(&acl_account_id);
+            data.server
+                .increment_token_revision(ChangedPrincipals::from_change(
+                    acl_account_id,
+                    Type::Individual,
+                    PrincipalField::EnabledPermissions,
+                ))
+                .await;
 
             trc::event!(
                 Imap(trc::ImapEvent::SetAcl),
