@@ -13,6 +13,7 @@ use crate::{
 use ahash::AHashMap;
 use common::listener::SessionStream;
 use directory::Permission;
+use email::metadata::MessageMetadata;
 use imap_proto::{
     parser::PushUnique,
     protocol::{
@@ -26,13 +27,7 @@ use imap_proto::{
     receiver::Request,
     Command, ResponseCode, ResponseType, StatusResponse,
 };
-use jmap::{
-    blob::download::BlobDownload,
-    changes::{get::ChangesLookup, write::ChangeLog},
-    email::metadata::MessageMetadata,
-    services::state::StateManager,
-    JmapMethods,
-};
+use jmap::{blob::download::BlobDownload, changes::get::ChangesLookup};
 use jmap_proto::types::{
     acl::Acl, collection::Collection, id::Id, keyword::Keyword, property::Property,
     state::StateChange, type_state::DataType,
@@ -42,47 +37,66 @@ use store::{
     query::log::{Change, Query},
     write::{assert::HashedValue, BatchBuilder, Bincode, F_BITMAP, F_VALUE},
 };
+use trc::AddContext;
 
 use super::{FromModSeq, ImapContext};
 
 impl<T: SessionStream> Session<T> {
-    pub async fn handle_fetch(
-        &mut self,
-        request: Request<Command>,
-        is_uid: bool,
-    ) -> trc::Result<()> {
+    pub async fn handle_fetch(&mut self, requests: Vec<Request<Command>>) -> trc::Result<()> {
         // Validate access
         self.assert_has_permission(Permission::ImapFetch)?;
 
-        let op_start = Instant::now();
-        let arguments = request.parse_fetch()?;
-
         let (data, mailbox) = self.state.select_data();
         let is_qresync = self.is_qresync;
         let is_rev2 = self.version.is_rev2();
 
-        let enabled_condstore = if !self.is_condstore && arguments.changed_since.is_some()
-            || arguments.attributes.contains(&Attribute::ModSeq)
-        {
-            self.is_condstore = true;
-            true
-        } else {
-            false
-        };
+        let mut ops = Vec::with_capacity(requests.len());
+
+        for request in requests {
+            let is_uid = matches!(request.command, Command::Fetch(true));
+            match request.parse_fetch() {
+                Ok(arguments) => {
+                    let enabled_condstore = if !self.is_condstore
+                        && arguments.changed_since.is_some()
+                        || arguments.attributes.contains(&Attribute::ModSeq)
+                    {
+                        self.is_condstore = true;
+                        true
+                    } else {
+                        false
+                    };
+
+                    ops.push(Ok((is_uid, enabled_condstore, arguments)));
+                }
+                Err(err) => {
+                    ops.push(Err(err));
+                }
+            }
+        }
 
         spawn_op!(data, {
-            let response = data
-                .fetch(
-                    arguments,
-                    mailbox,
-                    is_uid,
-                    is_qresync,
-                    is_rev2,
-                    enabled_condstore,
-                    op_start,
-                )
-                .await?;
-            data.write_bytes(response.into_bytes()).await
+            for op in ops {
+                match op {
+                    Ok((is_uid, enabled_condstore, arguments)) => {
+                        let response = data
+                            .fetch(
+                                arguments,
+                                mailbox.clone(),
+                                is_uid,
+                                is_qresync,
+                                is_rev2,
+                                enabled_condstore,
+                                Instant::now(),
+                            )
+                            .await?;
+
+                        data.write_bytes(response.into_bytes()).await?;
+                    }
+                    Err(err) => data.write_error(err).await?,
+                }
+            }
+
+            Ok(())
         })
     }
 }
@@ -217,7 +231,7 @@ impl<T: SessionStream> SessionData<T> {
         for attribute in &arguments.attributes {
             match attribute {
                 Attribute::BodySection { sections, .. }
-                    if sections.first().map_or(false, |s| {
+                    if sections.first().is_some_and(|s| {
                         matches!(s, Section::Header | Section::HeaderFields { .. })
                     }) => {}
                 Attribute::Body | Attribute::BodyStructure | Attribute::BinarySize { .. } => {
@@ -532,7 +546,6 @@ impl<T: SessionStream> SessionData<T> {
             let mut changelog = self
                 .server
                 .begin_changes(account_id)
-                .await
                 .imap_ctx(&arguments.tag, trc::location!())?;
             for (id, mut keywords) in set_seen_ids {
                 keywords.inner.push(Keyword::Seen);
@@ -545,7 +558,13 @@ impl<T: SessionStream> SessionData<T> {
                     .value(Property::Keywords, keywords.inner, F_VALUE)
                     .value(Property::Keywords, Keyword::Seen, F_BITMAP)
                     .value(Property::Cid, changelog.change_id, F_VALUE);
-                match self.server.write_batch(batch).await {
+                match self
+                    .server
+                    .store()
+                    .write(batch)
+                    .await
+                    .caused_by(trc::location!())
+                {
                     Ok(_) => {
                         changelog.log_update(Collection::Email, id);
                     }
@@ -764,7 +783,7 @@ impl<'x> AsImapDataItem<'x> for Message<'x> {
                 .header_value(&HeaderName::ContentLanguage)
                 .and_then(|hv| {
                     hv.as_text_list()
-                        .map(|list| list.into_iter().map(|item| item.into()).collect())
+                        .map(|list| list.iter().map(|item| item.as_ref().into()).collect())
                 });
 
             extension.body_location = part
