@@ -11,71 +11,81 @@ use crate::{
     op::ImapContext,
     spawn_op,
 };
-use common::{listener::SessionStream, Mailbox};
+use common::{Mailbox, listener::SessionStream};
 use directory::Permission;
 use imap_proto::{
+    Command, ResponseCode, StatusResponse,
     parser::PushUnique,
     protocol::status::{Status, StatusItem, StatusItemType},
     receiver::Request,
-    Command, ResponseCode, StatusResponse,
 };
-use jmap::JmapMethods;
 use jmap_proto::{
     object::Object,
     types::{collection::Collection, id::Id, keyword::Keyword, property::Property, value::Value},
 };
-use store::{
-    roaring::RoaringBitmap,
-    write::{key::DeserializeBigEndian, ValueClass},
-    IndexKeyPrefix, IterateParams, ValueKey,
-};
 use store::{Deserialize, U32_LEN};
+use store::{
+    IndexKeyPrefix, IterateParams, roaring::RoaringBitmap, write::key::DeserializeBigEndian,
+};
 use trc::AddContext;
 
 use super::ToModSeq;
 
 impl<T: SessionStream> Session<T> {
-    pub async fn handle_status(&mut self, request: Request<Command>) -> trc::Result<()> {
+    pub async fn handle_status(&mut self, requests: Vec<Request<Command>>) -> trc::Result<()> {
         // Validate access
         self.assert_has_permission(Permission::ImapStatus)?;
 
-        let op_start = Instant::now();
-        let arguments = request.parse_status(self.version)?;
         let version = self.version;
         let data = self.state.session_data();
 
         spawn_op!(data, {
-            // Refresh mailboxes
-            data.synchronize_mailboxes(false)
-                .await
-                .imap_ctx(&arguments.tag, trc::location!())?;
+            let mut did_sync = false;
 
-            // Fetch status
-            let status = data
-                .status(arguments.mailbox_name, &arguments.items)
-                .await
-                .imap_ctx(&arguments.tag, trc::location!())?;
+            for request in requests.into_iter() {
+                match request.parse_status(version) {
+                    Ok(arguments) => {
+                        let op_start = Instant::now();
+                        if !did_sync {
+                            // Refresh mailboxes
+                            data.synchronize_mailboxes(false)
+                                .await
+                                .imap_ctx(&arguments.tag, trc::location!())?;
+                            did_sync = true;
+                        }
 
-            trc::event!(
-                Imap(trc::ImapEvent::Status),
-                SpanId = data.session_id,
-                MailboxName = status.mailbox_name.clone(),
-                Details = arguments
-                    .items
-                    .iter()
-                    .map(|c| trc::Value::from(format!("{c:?}")))
-                    .collect::<Vec<_>>(),
-                Elapsed = op_start.elapsed()
-            );
+                        // Fetch status
+                        let status = data
+                            .status(arguments.mailbox_name, &arguments.items)
+                            .await
+                            .imap_ctx(&arguments.tag, trc::location!())?;
 
-            let mut buf = Vec::with_capacity(32);
-            status.serialize(&mut buf, version.is_rev2());
-            data.write_bytes(
-                StatusResponse::completed(Command::Status)
-                    .with_tag(arguments.tag)
-                    .serialize(buf),
-            )
-            .await
+                        trc::event!(
+                            Imap(trc::ImapEvent::Status),
+                            SpanId = data.session_id,
+                            MailboxName = status.mailbox_name.clone(),
+                            Details = arguments
+                                .items
+                                .iter()
+                                .map(|c| trc::Value::from(format!("{c:?}")))
+                                .collect::<Vec<_>>(),
+                            Elapsed = op_start.elapsed()
+                        );
+
+                        let mut buf = Vec::with_capacity(32);
+                        status.serialize(&mut buf, version.is_rev2());
+                        data.write_bytes(
+                            StatusResponse::completed(Command::Status)
+                                .with_tag(arguments.tag)
+                                .serialize(buf),
+                        )
+                        .await?;
+                    }
+                    Err(err) => data.write_error(err).await?,
+                }
+            }
+
+            Ok(())
         })
     }
 }
@@ -90,7 +100,7 @@ impl<T: SessionStream> SessionData<T> {
             return if mailbox_name == self.server.core.jmap.shared_folder
                 || mailbox_name
                     .split_once('/')
-                    .map_or(false, |(base_name, path)| {
+                    .is_some_and(|(base_name, path)| {
                         base_name == self.server.core.jmap.shared_folder && !path.contains('/')
                     })
             {
@@ -107,7 +117,8 @@ impl<T: SessionStream> SessionData<T> {
                                     | Status::Unseen
                                     | Status::Recent
                                     | Status::Deleted
-                                    | Status::HighestModSeq => StatusItemType::Number(0),
+                                    | Status::HighestModSeq
+                                    | Status::DeletedStorage => StatusItemType::Number(0),
                                     Status::UidNext | Status::UidValidity => {
                                         StatusItemType::Number(1)
                                     }
@@ -140,42 +151,49 @@ impl<T: SessionStream> SessionData<T> {
                     match item {
                         Status::Messages => {
                             if let Some(value) = mailbox_state.total_messages {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
                         }
                         Status::UidNext => {
                             if let Some(value) = mailbox_state.uid_next {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
                         }
                         Status::UidValidity => {
                             if let Some(value) = mailbox_state.uid_validity {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
                         }
                         Status::Unseen => {
                             if let Some(value) = mailbox_state.total_unseen {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
                         }
                         Status::Deleted => {
                             if let Some(value) = mailbox_state.total_deleted {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
+                            } else {
+                                items_update.push_unique(*item);
+                            }
+                        }
+                        Status::DeletedStorage => {
+                            if let Some(value) = mailbox_state.total_deleted_storage {
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
                         }
                         Status::Size => {
                             if let Some(value) = mailbox_state.size {
-                                items_response.push((*item, StatusItemType::Number(value as u64)));
+                                items_response.push((*item, StatusItemType::Number(value)));
                             } else {
                                 items_update.push_unique(*item);
                             }
@@ -231,22 +249,10 @@ impl<T: SessionStream> SessionData<T> {
             for item in items_update {
                 let result = match item {
                     Status::Messages => mailbox_message_ids.as_ref().map(|v| v.len()).unwrap_or(0),
-                    Status::UidNext => {
-                        (self
-                            .server
-                            .core
-                            .storage
-                            .data
-                            .get_counter(ValueKey {
-                                account_id: mailbox.account_id,
-                                collection: Collection::Mailbox.into(),
-                                document_id: mailbox.mailbox_id,
-                                class: ValueClass::Property(Property::EmailIds.into()),
-                            })
-                            .await
-                            .caused_by(trc::location!())?
-                            + 1) as u64
-                    }
+                    Status::UidNext => self
+                        .get_uid_next(&mailbox)
+                        .await
+                        .caused_by(trc::location!())? as u64,
                     Status::UidValidity => self
                         .server
                         .get_property::<Object<Value>>(
@@ -310,11 +316,32 @@ impl<T: SessionStream> SessionData<T> {
                             0
                         }
                     }
+                    Status::DeletedStorage => {
+                        if let (Some(mailbox_message_ids), Some(mut deleted)) = (
+                            &mailbox_message_ids,
+                            self.server
+                                .get_tag(
+                                    mailbox.account_id,
+                                    Collection::Email,
+                                    Property::Keywords,
+                                    Keyword::Deleted,
+                                )
+                                .await
+                                .caused_by(trc::location!())?,
+                        ) {
+                            deleted &= mailbox_message_ids.as_ref();
+                            self.calculate_mailbox_size(mailbox.account_id, &deleted)
+                                .await
+                                .caused_by(trc::location!())?
+                        } else {
+                            0
+                        }
+                    }
                     Status::Size => {
                         if let Some(mailbox_message_ids) = &mailbox_message_ids {
                             self.calculate_mailbox_size(mailbox.account_id, mailbox_message_ids)
                                 .await
-                                .caused_by(trc::location!())? as u64
+                                .caused_by(trc::location!())?
                         } else {
                             0
                         }
@@ -329,7 +356,7 @@ impl<T: SessionStream> SessionData<T> {
                 };
 
                 items_response.push((item, StatusItemType::Number(result)));
-                values_update.push((item, result as u32));
+                values_update.push((item, result));
             }
 
             // Update cache
@@ -347,6 +374,9 @@ impl<T: SessionStream> SessionData<T> {
                             Status::UidValidity => mailbox_state.uid_validity = value.into(),
                             Status::Unseen => mailbox_state.total_unseen = value.into(),
                             Status::Deleted => mailbox_state.total_deleted = value.into(),
+                            Status::DeletedStorage => {
+                                mailbox_state.total_deleted_storage = value.into()
+                            }
                             Status::Size => mailbox_state.size = value.into(),
                             Status::Recent => {
                                 items_response
@@ -376,9 +406,9 @@ impl<T: SessionStream> SessionData<T> {
     async fn calculate_mailbox_size(
         &self,
         account_id: u32,
-        message_ids: &Arc<RoaringBitmap>,
-    ) -> trc::Result<u32> {
-        let mut total_size = 0u32;
+        message_ids: &RoaringBitmap,
+    ) -> trc::Result<u64> {
+        let mut total_size = 0u64;
         self.server
             .core
             .storage
@@ -407,7 +437,7 @@ impl<T: SessionStream> SessionData<T> {
                             .ok_or_else(|| trc::Error::corrupted_key(key, None, trc::location!()))
                             .and_then(u32::deserialize)
                             .map(|size| {
-                                total_size += size;
+                                total_size += size as u64;
                             })?;
                     }
                     Ok(true)
