@@ -13,7 +13,7 @@ use common::{
     ipc::StateEvent,
     listener::{ServerInstance, SessionData, SessionManager, SessionStream},
     manager::webadmin::Resource,
-    Inner, Server,
+    Inner, Server, KV_ACME,
 };
 use directory::Permission;
 use http_body_util::{BodyExt, Full};
@@ -32,6 +32,7 @@ use jmap_proto::{
     types::{blob::BlobId, id::Id},
 };
 use std::future::Future;
+use store::dispatch::lookup::KeyValue;
 use trc::SecurityEvent;
 use utils::url_params::UrlParams;
 
@@ -231,13 +232,15 @@ impl ParseHttp for Server {
                 }
                 ("oauth-authorization-server", &Method::GET) => {
                     // Limit anonymous requests
-                    self.is_anonymous_allowed(&session.remote_ip).await?;
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
 
                     return self.handle_oauth_metadata(req, session).await;
                 }
                 ("openid-configuration", &Method::GET) => {
                     // Limit anonymous requests
-                    self.is_anonymous_allowed(&session.remote_ip).await?;
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
 
                     return self.handle_oidc_metadata(req, session).await;
                 }
@@ -247,7 +250,7 @@ impl ParseHttp for Server {
                             .core
                             .storage
                             .lookup
-                            .key_get::<String>(format!("acme:{token}").into_bytes())
+                            .key_get::<String>(KeyValue::<()>::build_key(KV_ACME, token))
                             .await?
                         {
                             Some(proof) => Ok(Resource::new("text/plain", proof.into_bytes())
@@ -257,6 +260,10 @@ impl ParseHttp for Server {
                     }
                 }
                 ("mta-sts.txt", &Method::GET) => {
+                    // Limit anonymous requests
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
+
                     return if let Some(policy) = self.build_mta_sts_policy() {
                         Ok(Resource::new("text/plain", policy.to_string().into_bytes())
                             .into_http_response())
@@ -265,12 +272,20 @@ impl ParseHttp for Server {
                     };
                 }
                 ("mail-v1.xml", &Method::GET) => {
+                    // Limit anonymous requests
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
+
                     return self.handle_autoconfig_request(&req).await;
                 }
                 ("autoconfig", &Method::GET) => {
                     if path.next().unwrap_or_default() == "mail"
                         && path.next().unwrap_or_default() == "config-v1.1.xml"
                     {
+                        // Limit anonymous requests
+                        self.is_http_anonymous_request_allowed(&session.remote_ip)
+                            .await?;
+
                         return self.handle_autoconfig_request(&req).await;
                     }
                 }
@@ -281,12 +296,14 @@ impl ParseHttp for Server {
             },
             "auth" => match (path.next().unwrap_or_default(), req.method()) {
                 ("device", &Method::POST) => {
-                    self.is_anonymous_allowed(&session.remote_ip).await?;
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
 
                     return self.handle_device_auth(&mut req, session).await;
                 }
                 ("token", &Method::POST) => {
-                    self.is_anonymous_allowed(&session.remote_ip).await?;
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
 
                     return self.handle_token_request(&mut req, session).await;
                 }
@@ -313,7 +330,8 @@ impl ParseHttp for Server {
                 }
                 ("jwks.json", &Method::GET) => {
                     // Limit anonymous requests
-                    self.is_anonymous_allowed(&session.remote_ip).await?;
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
 
                     return Ok(self.core.oauth.oidc_jwks.clone().into_http_response());
                 }
@@ -407,6 +425,10 @@ impl ParseHttp for Server {
                 if req.method() == Method::GET
                     && path.next().unwrap_or_default() == "config-v1.1.xml"
                 {
+                    // Limit anonymous requests
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
+
                     return self.handle_autoconfig_request(&req).await;
                 }
             }
@@ -414,6 +436,10 @@ impl ParseHttp for Server {
                 if req.method() == Method::POST
                     && path.next().unwrap_or_default() == "autodiscover.xml"
                 {
+                    // Limit anonymous requests
+                    self.is_http_anonymous_request_allowed(&session.remote_ip)
+                        .await?;
+
                     return self
                         .handle_autodiscover_request(
                             fetch_body(&mut req, 8192, session.session_id).await,
@@ -422,34 +448,44 @@ impl ParseHttp for Server {
                 }
             }
             "robots.txt" => {
+                // Limit anonymous requests
+                self.is_http_anonymous_request_allowed(&session.remote_ip)
+                    .await?;
+
                 return Ok(
                     Resource::new("text/plain", b"User-agent: *\nDisallow: /\n".to_vec())
                         .into_http_response(),
                 );
             }
-            "healthz" => match path.next().unwrap_or_default() {
-                "live" => {
-                    return Ok(StatusCode::OK.into_http_response());
-                }
-                "ready" => {
-                    return Ok({
-                        if !self.core.storage.data.is_none() {
-                            StatusCode::OK
-                        } else {
-                            StatusCode::SERVICE_UNAVAILABLE
-                        }
+            "healthz" => {
+                // Limit anonymous requests
+                self.is_http_anonymous_request_allowed(&session.remote_ip)
+                    .await?;
+
+                match path.next().unwrap_or_default() {
+                    "live" => {
+                        return Ok(StatusCode::OK.into_http_response());
                     }
-                    .into_http_response());
+                    "ready" => {
+                        return Ok({
+                            if !self.core.storage.data.is_none() {
+                                StatusCode::OK
+                            } else {
+                                StatusCode::SERVICE_UNAVAILABLE
+                            }
+                        }
+                        .into_http_response());
+                    }
+                    _ => (),
                 }
-                _ => (),
-            },
+            }
             "metrics" => match path.next().unwrap_or_default() {
                 "prometheus" => {
                     if let Some(prometheus) = &self.core.metrics.prometheus {
                         if let Some(auth) = &prometheus.auth {
                             if req
                                 .authorization_basic()
-                                .map_or(true, |secret| secret != auth)
+                                .is_none_or( |secret| secret != auth)
                             {
                                 return Err(trc::AuthEvent::Failed
                                     .into_err()
@@ -507,7 +543,8 @@ impl ParseHttp for Server {
                 if let Some(form) = &self.core.network.contact_form {
                     match *req.method() {
                         Method::POST => {
-                            self.is_anonymous_allowed(&session.remote_ip).await?;
+                            self.is_http_anonymous_request_allowed(&session.remote_ip)
+                                .await?;
 
                             let form_data =
                                 FormData::from_request(&mut req, form.max_size, session.session_id)
@@ -613,6 +650,20 @@ async fn handle_session<T: SessionStream>(inner: Arc<Inner>, session: SessionDat
                                 .and_then(|h| h.parse::<IpAddr>().ok())
                         })
                     {
+                        // Check if the forwarded IP has been blocked
+                        if server.is_ip_blocked(&forwarded_for) {
+                            trc::event!(
+                                Security(trc::SecurityEvent::IpBlocked),
+                                ListenerId = instance.id.clone(),
+                                RemoteIp = forwarded_for,
+                                SpanId = session.session_id,
+                            );
+
+                            return Ok::<_, hyper::Error>(
+                                StatusCode::FORBIDDEN.into_http_response().build(),
+                            );
+                        }
+
                         trc::event!(
                             Http(trc::HttpEvent::RequestUrl),
                             SpanId = session.session_id,
@@ -794,6 +845,10 @@ impl ResolveVariable for HttpContext<'_> {
             _ => Variable::default(),
         }
     }
+
+    fn resolve_global(&self, _: &str) -> Variable<'_> {
+        Variable::Integer(0)
+    }
 }
 
 pub async fn fetch_body(
