@@ -222,7 +222,7 @@ impl TelemetryApi for Server {
                                             if (filter.is_none() && key_filters.is_empty())
                                                 || event
                                                     .span_id()
-                                                    .map_or(false, |span_id| active_span_ids.contains(&span_id))
+                                                    .is_some_and(|span_id| active_span_ids.contains(&span_id))
                                             {
                                                 events.push(event);
                                             } else {
@@ -385,7 +385,13 @@ impl TelemetryApi for Server {
                     .enterprise
                     .as_ref()
                     .and_then(|e| e.metrics_store.as_ref())
-                    .ok_or_else(|| manage::unsupported("No metrics store has been configured"))?
+                    .ok_or_else(|| {
+                        manage::error(
+                            "No metrics store has been defined",
+                            "You need to configure a metrics store in order to use this feature."
+                                .into(),
+                        )
+                    })?
                     .store
                     .query_metrics(after, before)
                     .await?;
