@@ -4,13 +4,13 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::sync::{atomic::Ordering, Arc};
+use std::sync::Arc;
 
-use common::{auth::AccessToken, Server};
+use common::{auth::AccessToken, Server, KV_BAYES_MODEL_USER};
 use directory::{
     backend::internal::{
         lookup::DirectoryStore,
-        manage::{self, not_found, ManageDirectory, UpdatePrincipal},
+        manage::{self, not_found, ChangedPrincipals, ManageDirectory, UpdatePrincipal},
         PrincipalAction, PrincipalField, PrincipalUpdate, PrincipalValue, SpecialSecrets,
     },
     DirectoryInner, Permission, Principal, QueryBy, Type,
@@ -34,7 +34,7 @@ pub enum AccountAuthRequest {
     EnableOtpAuth { url: String },
     DisableOtpAuth { url: Option<String> },
     AddAppPassword { name: String, password: String },
-    RemoveAppPassword { name: String },
+    RemoveAppPassword { name: Option<String> },
 }
 
 #[derive(Debug, serde::Serialize, serde::Deserialize)]
@@ -146,6 +146,21 @@ impl PrincipalManager for Server {
                     }
                 }
 
+                // Set default report domain if missing
+                let report_domain = if principal.typ() == Type::Domain
+                    && self
+                        .core
+                        .storage
+                        .config
+                        .get("report.domain")
+                        .await
+                        .is_ok_and(|v| v.is_none())
+                {
+                    principal.name().to_lowercase().into()
+                } else {
+                    None
+                };
+
                 // Create principal
                 let result = self
                     .core
@@ -154,8 +169,25 @@ impl PrincipalManager for Server {
                     .create_principal(principal, tenant_id, Some(&access_token.permissions))
                     .await?;
 
+                // Set report domain
+                if let Some(report_domain) = report_domain {
+                    if let Err(err) = self
+                        .core
+                        .storage
+                        .config
+                        .set([("report.domain", report_domain)], true)
+                        .await
+                    {
+                        trc::error!(err.details("Failed to set report domain"));
+                    }
+                }
+
+                // Increment revision
+                self.increment_token_revision(result.changed_principals)
+                    .await;
+
                 Ok(JsonResponse::new(json!({
-                    "data": result,
+                    "data": result.id,
                 }))
                 .into_http_response())
             }
@@ -265,6 +297,122 @@ impl PrincipalManager for Server {
                 }))
                 .into_http_response())
             }
+            (None, &Method::DELETE) => {
+                // List principal ids
+                let params = UrlParams::new(req.uri().query());
+                let filter = params.get("filter");
+                let typ = params.parse::<Type>("type").ok_or_else(|| {
+                    trc::EventType::Resource(trc::ResourceEvent::BadParameters)
+                        .into_err()
+                        .details("Invalid type")
+                })?;
+                if params.get("confirm") != Some("true") {
+                    return Err(trc::EventType::Resource(trc::ResourceEvent::BadParameters)
+                        .into_err()
+                        .details("Missing confirmation parameter"));
+                }
+
+                // Validate the access token
+                access_token.assert_has_permission(match typ {
+                    Type::Individual => Permission::IndividualDelete,
+                    Type::Group => Permission::GroupDelete,
+                    Type::List => Permission::MailingListDelete,
+                    Type::Domain => Permission::DomainDelete,
+                    Type::Tenant => Permission::TenantDelete,
+                    Type::Role => Permission::RoleDelete,
+                    Type::ApiKey => Permission::ApiKeyDelete,
+                    Type::OauthClient => Permission::OauthClientDelete,
+                    Type::Resource | Type::Location | Type::Other => Permission::PrincipalDelete,
+                })?;
+
+                let mut tenant = access_token.tenant.map(|t| t.id);
+
+                #[cfg(feature = "enterprise")]
+                if self.core.is_enterprise_edition() {
+                    if tenant.is_none() {
+                        // Limit search to current tenant
+                        if let Some(tenant_name) = params.get("tenant") {
+                            tenant = self
+                                .core
+                                .storage
+                                .data
+                                .get_principal_info(tenant_name)
+                                .await?
+                                .filter(|p| p.typ == Type::Tenant)
+                                .map(|p| p.id);
+                        }
+                    }
+                } else if typ == Type::Tenant {
+                    return Err(manage::enterprise());
+                }
+
+                let principals = self
+                    .core
+                    .storage
+                    .data
+                    .list_principals(filter, tenant, &[typ], &[PrincipalField::Name], 0, 0)
+                    .await?;
+
+                let found = !principals.items.is_empty();
+                if found {
+                    let server = self.clone();
+                    tokio::spawn(async move {
+                        let has_bayes = server
+                            .core
+                            .spam
+                            .bayes
+                            .as_ref()
+                            .is_some_and(|c| c.account_classify);
+                        for principal in principals.items {
+                            // Delete account
+                            match server
+                                .store()
+                                .delete_principal(QueryBy::Id(principal.id()))
+                                .await
+                            {
+                                Ok(changed_principals) => {
+                                    // Increment revision
+                                    server.increment_token_revision(changed_principals).await;
+                                }
+                                Err(err) => {
+                                    trc::error!(err.details("Failed to delete principal"));
+                                    continue;
+                                }
+                            }
+
+                            if matches!(typ, Type::Individual | Type::Group) {
+                                // Remove FTS index
+                                if let Err(err) =
+                                    server.core.storage.fts.remove_all(principal.id()).await
+                                {
+                                    trc::error!(err.details("Failed to delete FTS index"));
+                                }
+
+                                // Delete bayes model
+                                if has_bayes {
+                                    let mut key =
+                                        Vec::with_capacity(std::mem::size_of::<u32>() + 1);
+                                    key.push(KV_BAYES_MODEL_USER);
+                                    key.extend_from_slice(&principal.id().to_be_bytes());
+
+                                    if let Err(err) =
+                                        server.in_memory_store().key_delete_prefix(&key).await
+                                    {
+                                        trc::error!(
+                                            err.details("Failed to delete user bayes model")
+                                        );
+                                    }
+                                }
+                            }
+                        }
+                    });
+                }
+
+                Ok(JsonResponse::new(json!({
+                    "data": found,
+                }))
+                .into_http_response())
+            }
             (Some(name), method) => {
                 // Fetch, update or delete principal
                 let name = decode_path_element(name);
@@ -344,31 +492,37 @@ impl PrincipalManager for Server {
                         })?;
 
                         // Delete account
-                        self.core
-                            .storage
-                            .data
+                        let changed_principals = self
+                            .store()
                             .delete_principal(QueryBy::Id(account_id))
                             .await?;
 
-                        // Remove FTS index
                         if matches!(typ, Type::Individual | Type::Group) {
+                            // Remove FTS index
                             self.core.storage.fts.remove_all(account_id).await?;
+
+                            // Delete bayes model
+                            if self
+                                .core
+                                .spam
+                                .bayes
+                                .as_ref()
+                                .is_some_and(|c| c.account_classify)
+                            {
+                                let mut key = Vec::with_capacity(std::mem::size_of::<u32>() + 1);
+                                key.push(KV_BAYES_MODEL_USER);
+                                key.extend_from_slice(&account_id.to_be_bytes());
+
+                                if let Err(err) =
+                                    self.in_memory_store().key_delete_prefix(&key).await
+                                {
+                                    trc::error!(err.details("Failed to delete user bayes model"));
+                                }
+                            }
                         }
 
-                        // Remove entries from cache
-                        self.inner
-                            .data
-                            .http_auth_cache
-                            .retain(|_, id| id.item != account_id);
-
-                        if matches!(typ, Type::Role | Type::Tenant) {
-                            // Update permissions cache
-                            self.inner.data.permissions.clear();
-                            self.inner
-                                .data
-                                .permissions_version
-                                .fetch_add(1, Ordering::Relaxed);
-                        }
+                        // Increment revision
+                        self.increment_token_revision(changed_principals).await;
 
                         Ok(JsonResponse::new(json!({
                             "data": (),
@@ -401,16 +555,10 @@ impl PrincipalManager for Server {
                         })?;
 
                         // Validate changes
-                        let mut needs_assert = false;
-                        let mut expire_session = false;
-                        let mut expire_token = false;
-                        let mut is_role_change = false;
-
                         for change in &changes {
                             match change.field {
                                 PrincipalField::Secrets => {
-                                    expire_session = true;
-                                    needs_assert = true;
+                                    self.assert_supported_directory()?;
                                 }
                                 PrincipalField::Name
                                 | PrincipalField::Emails
@@ -439,12 +587,6 @@ impl PrincipalManager for Server {
                                 PrincipalField::Roles
                                 | PrincipalField::EnabledPermissions
                                 | PrincipalField::DisabledPermissions => {
-                                    if matches!(typ, Type::Role | Type::Tenant) {
-                                        is_role_change = true;
-                                    } else {
-                                        expire_token = true;
-                                    }
-
                                     if change.field == PrincipalField::Roles
                                         && matches!(
                                             change.action,
@@ -495,12 +637,9 @@ impl PrincipalManager for Server {
                             }
                         }
 
-                        if needs_assert {
-                            self.assert_supported_directory()?;
-                        }
-
                         // Update principal
-                        self.core
+                        let changed_principals = self
+                            .core
                             .storage
                             .data
                             .update_principal(
@@ -511,26 +650,8 @@ impl PrincipalManager for Server {
                             )
                             .await?;
 
-                        if expire_session {
-                            // Remove entries from cache
-                            self.inner
-                                .data
-                                .http_auth_cache
-                                .retain(|_, id| id.item != account_id);
-                        }
-
-                        if is_role_change {
-                            // Update permissions cache
-                            self.inner.data.permissions.clear();
-                            self.inner
-                                .data
-                                .permissions_version
-                                .fetch_add(1, Ordering::Relaxed);
-                        }
-
-                        if expire_token {
-                            self.inner.data.access_tokens.remove(&account_id);
-                        }
+                        // Increment revision
+                        self.increment_token_revision(changed_principals).await;
 
                         Ok(JsonResponse::new(json!({
                             "data": (),
@@ -611,7 +732,7 @@ impl PrincipalManager for Server {
             .headers()
             .get(header::AUTHORIZATION)
             .and_then(|h| h.to_str().ok())
-            .map_or(true, |header| !header.to_lowercase().starts_with("basic "))
+            .is_none_or( |header| !header.to_lowercase().starts_with("basic "))
         {
             return Err(manage::error(
                 "Password changes only allowed using Basic auth",
@@ -626,14 +747,16 @@ impl PrincipalManager for Server {
                     self.core
                         .storage
                         .config
-                        .set([("authentication.fallback-admin.secret", password)])
+                        .set([("authentication.fallback-admin.secret", password)], true)
                         .await?;
 
-                    // Remove entries from cache
-                    self.inner
-                        .data
-                        .http_auth_cache
-                        .retain(|_, id| id.item != u32::MAX);
+                    // Increment revision
+                    self.increment_token_revision(ChangedPrincipals::from_change(
+                        access_token.primary_id(),
+                        Type::Individual,
+                        PrincipalField::Secrets,
+                    ))
+                    .await;
 
                     return Ok(JsonResponse::new(json!({
                         "data": (),
@@ -673,9 +796,10 @@ impl PrincipalManager for Server {
                 AccountAuthRequest::AddAppPassword { name, password } => {
                     (PrincipalAction::AddItem, format!("$app${name}${password}"))
                 }
-                AccountAuthRequest::RemoveAppPassword { name } => {
-                    (PrincipalAction::RemoveItem, format!("$app${name}"))
-                }
+                AccountAuthRequest::RemoveAppPassword { name } => (
+                    PrincipalAction::RemoveItem,
+                    format!("$app${}", name.unwrap_or_default()),
+                ),
             };
 
             actions.push(PrincipalUpdate {
@@ -686,7 +810,8 @@ impl PrincipalManager for Server {
         }
 
         // Update password
-        self.core
+        let changed_principals = self
+            .core
             .storage
             .data
             .update_principal(
@@ -696,11 +821,8 @@ impl PrincipalManager for Server {
             )
             .await?;
 
-        // Remove entries from cache
-        self.inner
-            .data
-            .http_auth_cache
-            .retain(|_, id| id.item != access_token.primary_id());
+        // Increment revision
+        self.increment_token_revision(changed_principals).await;
 
         Ok(JsonResponse::new(json!({
             "data": (),
@@ -716,7 +838,6 @@ impl PrincipalManager for Server {
             DirectoryInner::Imap(_) => "IMAP",
             DirectoryInner::Smtp(_) => "SMTP",
             DirectoryInner::Memory(_) => "In-Memory",
-            #[cfg(feature = "enterprise")]
             DirectoryInner::OpenId(_) => "OpenID",
         };
 
