@@ -97,142 +97,15 @@ impl ManageReports for Server {
                 access_token.assert_has_permission(Permission::IncomingReportList)?;
 
                 let params = UrlParams::new(req.uri().query());
-                let filter = params.get("text");
-                let page: usize = params.parse::<usize>("page").unwrap_or_default();
-                let limit: usize = params.parse::<usize>("limit").unwrap_or_default();
 
-                let range_start = params.parse::<u64>("range-start").unwrap_or_default();
-                let range_end = params.parse::<u64>("range-end").unwrap_or(u64::MAX);
-                let max_total = params.parse::<usize>("max-total").unwrap_or_default();
-
-                let (from_key, to_key, typ) = match class {
-                    "dmarc" => (
-                        ValueKey::from(ValueClass::Report(ReportClass::Dmarc {
-                            id: range_start,
-                            expires: 0,
-                        })),
-                        ValueKey::from(ValueClass::Report(ReportClass::Dmarc {
-                            id: range_end,
-                            expires: u64::MAX,
-                        })),
-                        ReportType::Dmarc,
-                    ),
-                    "tls" => (
-                        ValueKey::from(ValueClass::Report(ReportClass::Tls {
-                            id: range_start,
-                            expires: 0,
-                        })),
-                        ValueKey::from(ValueClass::Report(ReportClass::Tls {
-                            id: range_end,
-                            expires: u64::MAX,
-                        })),
-                        ReportType::Tls,
-                    ),
-                    "arf" => (
-                        ValueKey::from(ValueClass::Report(ReportClass::Arf {
-                            id: range_start,
-                            expires: 0,
-                        })),
-                        ValueKey::from(ValueClass::Report(ReportClass::Arf {
-                            id: range_end,
-                            expires: u64::MAX,
-                        })),
-                        ReportType::Arf,
-                    ),
-                    _ => unreachable!(),
-                };
-
-                let mut results = Vec::new();
-                let mut offset = page.saturating_sub(1) * limit;
-                let mut total = 0;
-                let mut last_id = 0;
-                let has_filters = filter.is_some() || tenant_domains.is_some();
-                self.core
-                    .storage
-                    .data
-                    .iterate(
-                        IterateParams::new(from_key, to_key)
-                            .set_values(has_filters)
-                            .descending(),
-                        |key, value| {
-                            // Skip chunked records
-                            let id = key.deserialize_be_u64(U64_LEN + 1)?;
-                            if id == last_id {
-                                return Ok(true);
-                            }
-                            last_id = id;
-
-                            // TODO: Support filtering chunked records (over 10MB) on FDB
-                            let matches = if has_filters {
-                                match typ {
-                                    ReportType::Dmarc => {
-                                        let report = Bincode::<
-                                            IncomingReport<mail_auth::report::Report>,
-                                        >::deserialize(
-                                            value
-                                        )
-                                        .caused_by(trc::location!())?
-                                        .inner;
-
-                                        filter.map_or(true, |f| report.contains(f))
-                                            && tenant_domains
-                                                .as_ref()
-                                                .map_or(true, |domains| report.has_domain(domains))
-                                    }
-                                    ReportType::Tls => {
-                                        let report =
-                                            Bincode::<IncomingReport<TlsReport>>::deserialize(
-                                                value,
-                                            )
-                                            .caused_by(trc::location!())?
-                                            .inner;
-
-                                        filter.map_or(true, |f| report.contains(f))
-                                            && tenant_domains
-                                                .as_ref()
-                                                .map_or(true, |domains| report.has_domain(domains))
-                                    }
-                                    ReportType::Arf => {
-                                        let report =
-                                            Bincode::<IncomingReport<Feedback>>::deserialize(value)
-                                                .caused_by(trc::location!())?
-                                                .inner;
-
-                                        filter.map_or(true, |f| report.contains(f))
-                                            && tenant_domains
-                                                .as_ref()
-                                                .map_or(true, |domains| report.has_domain(domains))
-                                    }
-                                }
-                            } else {
-                                true
-                            };
-
-                            if matches {
-                                if offset == 0 {
-                                    if limit == 0 || results.len() < limit {
-                                        results.push(format!(
-                                            "{}_{}",
-                                            id,
-                                            key.deserialize_be_u64(1)?
-                                        ));
-                                    }
-                                } else {
-                                    offset -= 1;
-                                }
-
-                                total += 1;
-                            }
-
-                            Ok(max_total == 0 || total < max_total)
-                        },
-                    )
-                    .await
-                    .caused_by(trc::location!())?;
+                let IncomingReports { ids, total } =
+                    fetch_incoming_reports(self, class, &params, &tenant_domains).await?;
 
                 Ok(JsonResponse::new(json!({
                         "data": {
-                            "items": results,
+                            "items": ids.into_iter().map(|(id, expires)| {
+                                format!("{id}_{expires}")
+                            }).collect::<Vec<_>>(),
                             "total": total,
                         },
                 }))
@@ -256,7 +129,7 @@ impl ManageReports for Server {
                             Some(report)
                                 if tenant_domains
                                     .as_ref()
-                                    .map_or(true, |domains| report.inner.has_domain(domains)) =>
+                                    .is_none_or( |domains| report.inner.has_domain(domains)) =>
                             {
                                 Ok(JsonResponse::new(json!({
                                         "data": report.inner,
@@ -277,7 +150,7 @@ impl ManageReports for Server {
                             Some(report)
                                 if tenant_domains
                                     .as_ref()
-                                    .map_or(true, |domains| report.inner.has_domain(domains)) =>
+                                    .is_none_or( |domains| report.inner.has_domain(domains)) =>
                             {
                                 Ok(JsonResponse::new(json!({
                                         "data": report.inner,
@@ -298,7 +171,7 @@ impl ManageReports for Server {
                             Some(report)
                                 if tenant_domains
                                     .as_ref()
-                                    .map_or(true, |domains| report.inner.has_domain(domains)) =>
+                                    .is_none_or( |domains| report.inner.has_domain(domains)) =>
                             {
                                 Ok(JsonResponse::new(json!({
                                         "data": report.inner,
@@ -312,6 +185,59 @@ impl ManageReports for Server {
                     Err(trc::ResourceEvent::NotFound.into_err())
                 }
             }
+            (class @ ("dmarc" | "tls" | "arf"), None, &Method::DELETE) => {
+                // Validate the access token
+                access_token.assert_has_permission(Permission::IncomingReportDelete)?;
+
+                let params = UrlParams::new(req.uri().query());
+
+                let IncomingReports { ids, .. } =
+                    fetch_incoming_reports(self, class, &params, &tenant_domains).await?;
+
+                let found = !ids.is_empty();
+                if found {
+                    let class = match class {
+                        "dmarc" => ReportClass::Dmarc { id: 0, expires: 0 },
+                        "tls" => ReportClass::Tls { id: 0, expires: 0 },
+                        "arf" => ReportClass::Arf { id: 0, expires: 0 },
+                        _ => unreachable!(),
+                    };
+                    let server = self.clone();
+                    tokio::spawn(async move {
+                        let mut batch = BatchBuilder::new();
+
+                        for (id, expires) in ids {
+                            let report_id = match &class {
+                                ReportClass::Dmarc { .. } => ReportClass::Dmarc { id, expires },
+                                ReportClass::Tls { .. } => ReportClass::Tls { id, expires },
+                                ReportClass::Arf { .. } => ReportClass::Arf { id, expires },
+                            };
+
+                            batch.clear(ValueClass::Report(report_id));
+
+                            if batch.ops.len() > 1000 {
+                                if let Err(err) =
+                                    server.core.storage.data.write(batch.build()).await
+                                {
+                                    trc::error!(err.caused_by(trc::location!()));
+                                }
+                                batch = BatchBuilder::new();
+                            }
+                        }
+
+                        if !batch.ops.is_empty() {
+                            if let Err(err) = server.core.storage.data.write(batch.build()).await {
+                                trc::error!(err.caused_by(trc::location!()));
+                            }
+                        }
+                    });
+                }
+
+                Ok(JsonResponse::new(json!({
+                        "data": found,
+                }))
+                .into_http_response())
+            }
             (class @ ("dmarc" | "tls" | "arf"), Some(report_id), &Method::DELETE) => {
                 // Validate the access token
                 access_token.assert_has_permission(Permission::IncomingReportDelete)?;
@@ -327,7 +253,7 @@ impl ManageReports for Server {
                                     ValueClass::Report(report_id.clone()),
                                 ))
                                 .await?
-                                .map_or(true, |report| report.inner.has_domain(domains)),
+                                .is_none_or( |report| report.inner.has_domain(domains)),
                             ReportClass::Dmarc { .. } => self
                                 .core
                                 .storage
@@ -336,7 +262,7 @@ impl ManageReports for Server {
                                     ValueKey::from(ValueClass::Report(report_id.clone())),
                                 )
                                 .await?
-                                .map_or(true, |report| report.inner.has_domain(domains)),
+                                .is_none_or( |report| report.inner.has_domain(domains)),
 
                             ReportClass::Arf { .. } => self
                                 .core
@@ -346,7 +272,7 @@ impl ManageReports for Server {
                                     ValueClass::Report(report_id.clone()),
                                 ))
                                 .await?
-                                .map_or(true, |report| report.inner.has_domain(domains)),
+                                .is_none_or( |report| report.inner.has_domain(domains)),
                         };
 
                         if !is_tenant_report {
@@ -371,6 +297,147 @@ impl ManageReports for Server {
     }
 }
 
+struct IncomingReports {
+    ids: Vec<(u64, u64)>,
+    total: usize,
+}
+
+async fn fetch_incoming_reports(
+    server: &Server,
+    class: &str,
+    params: &UrlParams<'_>,
+    tenant_domains: &Option<Vec<String>>,
+) -> trc::Result<IncomingReports> {
+    let filter = params.get("text");
+    let page: usize = params.parse::<usize>("page").unwrap_or_default();
+    let limit: usize = params.parse::<usize>("limit").unwrap_or_default();
+
+    let range_start = params.parse::<u64>("range-start").unwrap_or_default();
+    let range_end = params.parse::<u64>("range-end").unwrap_or(u64::MAX);
+    let max_total = params.parse::<usize>("max-total").unwrap_or_default();
+
+    let (from_key, to_key, typ) = match class {
+        "dmarc" => (
+            ValueKey::from(ValueClass::Report(ReportClass::Dmarc {
+                id: range_start,
+                expires: 0,
+            })),
+            ValueKey::from(ValueClass::Report(ReportClass::Dmarc {
+                id: range_end,
+                expires: u64::MAX,
+            })),
+            ReportType::Dmarc,
+        ),
+        "tls" => (
+            ValueKey::from(ValueClass::Report(ReportClass::Tls {
+                id: range_start,
+                expires: 0,
+            })),
+            ValueKey::from(ValueClass::Report(ReportClass::Tls {
+                id: range_end,
+                expires: u64::MAX,
+            })),
+            ReportType::Tls,
+        ),
+        "arf" => (
+            ValueKey::from(ValueClass::Report(ReportClass::Arf {
+                id: range_start,
+                expires: 0,
+            })),
+            ValueKey::from(ValueClass::Report(ReportClass::Arf {
+                id: range_end,
+                expires: u64::MAX,
+            })),
+            ReportType::Arf,
+        ),
+        _ => unreachable!(),
+    };
+
+    let mut results = IncomingReports {
+        ids: Vec::new(),
+        total: 0,
+    };
+    let mut offset = page.saturating_sub(1) * limit;
+    let mut last_id = 0;
+    let has_filters = filter.is_some() || tenant_domains.is_some();
+
+    server
+        .core
+        .storage
+        .data
+        .iterate(
+            IterateParams::new(from_key, to_key)
+                .set_values(has_filters)
+                .descending(),
+            |key, value| {
+                // Skip chunked records
+                let id = key.deserialize_be_u64(U64_LEN + 1)?;
+                if id == last_id {
+                    return Ok(true);
+                }
+                last_id = id;
+
+                // TODO: Support filtering chunked records (over 10MB) on FDB
+                let matches = if has_filters {
+                    match typ {
+                        ReportType::Dmarc => {
+                            let report =
+                                Bincode::<IncomingReport<mail_auth::report::Report>>::deserialize(
+                                    value,
+                                )
+                                .caused_by(trc::location!())?
+                                .inner;
+
+                            filter.is_none_or( |f| report.contains(f))
+                                && tenant_domains
+                                    .as_ref()
+                                    .is_none_or( |domains| report.has_domain(domains))
+                        }
+                        ReportType::Tls => {
+                            let report = Bincode::<IncomingReport<TlsReport>>::deserialize(value)
+                                .caused_by(trc::location!())?
+                                .inner;
+
+                            filter.is_none_or( |f| report.contains(f))
+                                && tenant_domains
+                                    .as_ref()
+                                    .is_none_or( |domains| report.has_domain(domains))
+                        }
+                        ReportType::Arf => {
+                            let report = Bincode::<IncomingReport<Feedback>>::deserialize(value)
+                                .caused_by(trc::location!())?
+                                .inner;
+
+                            filter.is_none_or( |f| report.contains(f))
+                                && tenant_domains
+                                    .as_ref()
+                                    .is_none_or( |domains| report.has_domain(domains))
+                        }
+                    }
+                } else {
+                    true
+                };
+
+                if matches {
+                    if offset == 0 {
+                        if limit == 0 || results.ids.len() < limit {
+                            results.ids.push((id, key.deserialize_be_u64(1)?));
+                        }
+                    } else {
+                        offset -= 1;
+                    }
+
+                    results.total += 1;
+                }
+
+                Ok(max_total == 0 || results.total < max_total)
+            },
+        )
+        .await
+        .caused_by(trc::location!())
+        .map(|_| results)
+}
+
 fn parse_incoming_report_id(class: &str, id: &str) -> Option<ReportClass> {
     let mut parts = id.split('_');
     let id = parts.next()?.parse().ok()?;
@@ -405,7 +472,7 @@ impl Contains for mail_auth::report::Report {
             || self.report_id().contains(text)
             || self
                 .extra_contact_info()
-                .map_or(false, |c| c.to_lowercase().contains(text))
+                .is_some_and(|c| c.to_lowercase().contains(text))
             || self.records().iter().any(|record| record.contains(text))
     }
 }
@@ -414,22 +481,22 @@ impl Contains for mail_auth::report::Record {
     fn contains(&self, filter: &str) -> bool {
         self.envelope_from().contains(filter)
             || self.header_from().contains(filter)
-            || self.envelope_to().map_or(false, |to| to.contains(filter))
+            || self.envelope_to().is_some_and(|to| to.contains(filter))
             || self.dkim_auth_result().iter().any(|dkim| {
                 dkim.domain().contains(filter)
                     || dkim.selector().contains(filter)
                     || dkim
                         .human_result()
                         .as_ref()
-                        .map_or(false, |r| r.contains(filter))
+                        .is_some_and(|r| r.contains(filter))
             })
             || self.spf_auth_result().iter().any(|spf| {
                 spf.domain().contains(filter)
-                    || spf.human_result().map_or(false, |r| r.contains(filter))
+                    || spf.human_result().is_some_and(|r| r.contains(filter))
             })
             || self
                 .source_ip()
-                .map_or(false, |ip| ip.to_string().contains(filter))
+                .is_some_and(|ip| ip.to_string().contains(filter))
     }
 }
 
@@ -437,11 +504,11 @@ impl Contains for TlsReport {
     fn contains(&self, text: &str) -> bool {
         self.organization_name
             .as_ref()
-            .map_or(false, |o| o.to_lowercase().contains(text))
+            .is_some_and(|o| o.to_lowercase().contains(text))
             || self
                 .contact_info
                 .as_ref()
-                .map_or(false, |c| c.to_lowercase().contains(text))
+                .is_some_and(|c| c.to_lowercase().contains(text))
             || self.report_id.contains(text)
             || self.policies.iter().any(|p| p.contains(text))
     }
@@ -467,26 +534,26 @@ impl Contains for Policy {
 impl Contains for FailureDetails {
     fn contains(&self, filter: &str) -> bool {
         self.sending_mta_ip
-            .map_or(false, |s| s.to_string().contains(filter))
+            .is_some_and(|s| s.to_string().contains(filter))
             || self
                 .receiving_ip
-                .map_or(false, |s| s.to_string().contains(filter))
+                .is_some_and(|s| s.to_string().contains(filter))
             || self
                 .receiving_mx_hostname
                 .as_ref()
-                .map_or(false, |s| s.contains(filter))
+                .is_some_and(|s| s.contains(filter))
             || self
                 .receiving_mx_helo
                 .as_ref()
-                .map_or(false, |s| s.contains(filter))
+                .is_some_and(|s| s.contains(filter))
             || self
                 .additional_information
                 .as_ref()
-                .map_or(false, |s| s.contains(filter))
+                .is_some_and(|s| s.contains(filter))
             || self
                 .failure_reason_code
                 .as_ref()
-                .map_or(false, |s| s.contains(filter))
+                .is_some_and(|s| s.contains(filter))
     }
 }
 
@@ -498,29 +565,27 @@ impl Contains for Feedback<'_> {
             .any(|s| s.contains(text))
             || self
                 .original_envelope_id()
-                .map_or(false, |s| s.contains(text))
-            || self
-                .original_mail_from()
-                .map_or(false, |s| s.contains(text))
-            || self.original_rcpt_to().map_or(false, |s| s.contains(text))
+                .is_some_and(|s| s.contains(text))
+            || self.original_mail_from().is_some_and(|s| s.contains(text))
+            || self.original_rcpt_to().is_some_and(|s| s.contains(text))
             || self.reported_domain().iter().any(|s| s.contains(text))
             || self.reported_uri().iter().any(|s| s.contains(text))
-            || self.reporting_mta().map_or(false, |s| s.contains(text))
-            || self.user_agent().map_or(false, |s| s.contains(text))
-            || self.dkim_adsp_dns().map_or(false, |s| s.contains(text))
+            || self.reporting_mta().is_some_and(|s| s.contains(text))
+            || self.user_agent().is_some_and(|s| s.contains(text))
+            || self.dkim_adsp_dns().is_some_and(|s| s.contains(text))
             || self
                 .dkim_canonicalized_body()
-                .map_or(false, |s| s.contains(text))
+                .is_some_and(|s| s.contains(text))
             || self
                 .dkim_canonicalized_header()
-                .map_or(false, |s| s.contains(text))
-            || self.dkim_domain().map_or(false, |s| s.contains(text))
-            || self.dkim_identity().map_or(false, |s| s.contains(text))
-            || self.dkim_selector().map_or(false, |s| s.contains(text))
-            || self.dkim_selector_dns().map_or(false, |s| s.contains(text))
-            || self.spf_dns().map_or(false, |s| s.contains(text))
-            || self.message().map_or(false, |s| s.contains(text))
-            || self.headers().map_or(false, |s| s.contains(text))
+                .is_some_and(|s| s.contains(text))
+            || self.dkim_domain().is_some_and(|s| s.contains(text))
+            || self.dkim_identity().is_some_and(|s| s.contains(text))
+            || self.dkim_selector().is_some_and(|s| s.contains(text))
+            || self.dkim_selector_dns().is_some_and(|s| s.contains(text))
+            || self.spf_dns().is_some_and(|s| s.contains(text))
+            || self.message().is_some_and(|s| s.contains(text))
+            || self.headers().is_some_and(|s| s.contains(text))
     }
 }
 
