@@ -22,8 +22,9 @@ use hyper::{
     Method, StatusCode,
 };
 use mail_auth::{
-    dmarc::{self},
+    dmarc::{self, verify::DmarcParameters},
     mta_sts::TlsRpt,
+    spf::verify::SpfParameters,
     AuthenticatedMessage, DkimResult, DmarcResult, IpLookupStrategy, IprevOutput, IprevResult,
     SpfOutput, SpfResult,
 };
@@ -317,14 +318,7 @@ async fn delivery_troubleshoot(
         (domain_or_email, None)
     };
 
-    let local_host = server
-        .core
-        .storage
-        .config
-        .get("lookup.default.hostname")
-        .await
-        .unwrap_or_default()
-        .unwrap_or_else(|| "local.host".to_string());
+    let local_host = &server.core.network.server_name;
 
     tx.send(DeliveryStage::MxLookupStart {
         domain: domain.to_string(),
@@ -333,7 +327,14 @@ async fn delivery_troubleshoot(
 
     // Lookup MX
     let now = Instant::now();
-    let mxs = match server.core.smtp.resolvers.dns.mx_lookup(&domain).await {
+    let mxs = match server
+        .core
+        .smtp
+        .resolvers
+        .dns
+        .mx_lookup(&domain, Some(&server.inner.cache.dns_mx))
+        .await
+    {
         Ok(mxs) => mxs,
         Err(err) => {
             tx.send(DeliveryStage::MxLookupError {
@@ -411,7 +412,10 @@ async fn delivery_troubleshoot(
         .smtp
         .resolvers
         .dns
-        .txt_lookup::<TlsRpt>(format!("_smtp._tls.{domain}."))
+        .txt_lookup::<TlsRpt>(
+            format!("_smtp._tls.{domain}."),
+            Some(&server.inner.cache.dns_txt),
+        )
         .await
     {
         Ok(record) => {
@@ -670,6 +674,46 @@ async fn delivery_troubleshoot(
                                 }
                             }
 
+                            // Say EHLO again (some SMTP servers require this)
+                            tx.send(DeliveryStage::EhloStart).await?;
+
+                            let now = Instant::now();
+                            match tokio::time::timeout(timeout, async {
+                                client
+                                    .stream
+                                    .write_all(format!("EHLO {local_host}\r\n",).as_bytes())
+                                    .await?;
+                                client.stream.flush().await?;
+                                client.read_ehlo().await
+                            })
+                            .await
+                            {
+                                Ok(Ok(_)) => {
+                                    tx.send(DeliveryStage::EhloSuccess {
+                                        elapsed: now.elapsed_ms(),
+                                    })
+                                    .await?;
+                                }
+                                Ok(Err(err)) => {
+                                    tx.send(DeliveryStage::EhloError {
+                                        elapsed: now.elapsed_ms(),
+                                        reason: err.to_string(),
+                                    })
+                                    .await?;
+
+                                    continue;
+                                }
+                                Err(_) => {
+                                    tx.send(DeliveryStage::EhloError {
+                                        elapsed: now.elapsed_ms(),
+                                        reason: "Timed out reading response".to_string(),
+                                    })
+                                    .await?;
+
+                                    continue;
+                                }
+                            }
+
                             // Verify recipient
                             let mut is_success = email.is_none();
                             if let Some(email) = &email {
@@ -846,14 +890,7 @@ async fn dmarc_troubleshoot(
     let mail_from = request.mail_from.to_lowercase();
     let mail_from_domain = mail_from.rsplit_once('@').map(|(_, domain)| domain);
 
-    let local_host = server
-        .core
-        .storage
-        .config
-        .get("lookup.default.hostname")
-        .await
-        .unwrap_or_default()
-        .unwrap_or_else(|| "local.host".to_string());
+    let local_host = &server.core.network.server_name;
 
     let now = Instant::now();
     let ehlo_spf_output = server
@@ -861,23 +898,38 @@ async fn dmarc_troubleshoot(
         .smtp
         .resolvers
         .dns
-        .verify_spf_helo(remote_ip, &ehlo_domain, &local_host)
+        .verify_spf(
+            server
+                .inner
+                .cache
+                .build_auth_parameters(SpfParameters::verify_ehlo(
+                    remote_ip,
+                    &ehlo_domain,
+                    local_host,
+                )),
+        )
         .await;
 
-    let iprev = server.core.smtp.resolvers.dns.verify_iprev(remote_ip).await;
+    let iprev = server
+        .core
+        .smtp
+        .resolvers
+        .dns
+        .verify_iprev(server.inner.cache.build_auth_parameters(remote_ip))
+        .await;
     let mail_spf_output = if let Some(mail_from_domain) = mail_from_domain {
         server
             .core
             .smtp
             .resolvers
             .dns
-            .check_host(
+            .check_host(server.inner.cache.build_auth_parameters(SpfParameters::new(
                 remote_ip,
                 mail_from_domain,
                 &ehlo_domain,
-                &local_host,
+                local_host,
                 &mail_from,
-            )
+            )))
             .await
     } else {
         server
@@ -885,13 +937,13 @@ async fn dmarc_troubleshoot(
             .smtp
             .resolvers
             .dns
-            .check_host(
+            .check_host(server.inner.cache.build_auth_parameters(SpfParameters::new(
                 remote_ip,
                 &ehlo_domain,
                 &ehlo_domain,
-                &local_host,
+                local_host,
                 &format!("postmaster@{ehlo_domain}"),
-            )
+            )))
             .await
     };
 
@@ -905,7 +957,7 @@ async fn dmarc_troubleshoot(
         .smtp
         .resolvers
         .dns
-        .verify_dkim(&auth_message)
+        .verify_dkim(server.inner.cache.build_auth_parameters(&auth_message))
         .await;
     let dkim_pass = dkim_output
         .iter()
@@ -916,7 +968,7 @@ async fn dmarc_troubleshoot(
         .smtp
         .resolvers
         .dns
-        .verify_arc(&auth_message)
+        .verify_arc(server.inner.cache.build_auth_parameters(&auth_message))
         .await;
 
     let dmarc_output = server
@@ -924,13 +976,13 @@ async fn dmarc_troubleshoot(
         .smtp
         .resolvers
         .dns
-        .verify_dmarc(
-            &auth_message,
-            &dkim_output,
-            mail_from_domain.unwrap_or(ehlo_domain.as_str()),
-            &mail_spf_output,
-            |domain| psl::domain_str(domain).unwrap_or(domain),
-        )
+        .verify_dmarc(server.inner.cache.build_auth_parameters(DmarcParameters {
+            message: &auth_message,
+            dkim_output: &dkim_output,
+            rfc5321_mail_from_domain: mail_from_domain.unwrap_or(ehlo_domain.as_str()),
+            spf_output: &mail_spf_output,
+            domain_suffix_fn: |domain| psl::domain_str(domain).unwrap_or(domain),
+        }))
         .await;
     let dmarc_pass = matches!(dmarc_output.spf_result(), DmarcResult::Pass)
         || matches!(dmarc_output.dkim_result(), DmarcResult::Pass);
@@ -992,6 +1044,28 @@ impl From<&SpfOutput> for AuthResult {
     }
 }
 
+impl From<AuthResult> for SpfOutput {
+    fn from(value: AuthResult) -> Self {
+        match value {
+            AuthResult::Pass => SpfOutput::new(String::new()).with_result(SpfResult::Pass),
+            AuthResult::Fail { .. } => SpfOutput::new(String::new()).with_result(SpfResult::Fail),
+            AuthResult::SoftFail { .. } => {
+                SpfOutput::new(String::new()).with_result(SpfResult::SoftFail)
+            }
+            AuthResult::Neutral { .. } => {
+                SpfOutput::new(String::new()).with_result(SpfResult::Neutral)
+            }
+            AuthResult::TempError { .. } => {
+                SpfOutput::new(String::new()).with_result(SpfResult::TempError)
+            }
+            AuthResult::PermError { .. } => {
+                SpfOutput::new(String::new()).with_result(SpfResult::PermError)
+            }
+            AuthResult::None => SpfOutput::new(String::new()).with_result(SpfResult::None),
+        }
+    }
+}
+
 impl From<&IprevOutput> for AuthResult {
     fn from(value: &IprevOutput) -> Self {
         match &value.result {
@@ -1010,6 +1084,25 @@ impl From<&IprevOutput> for AuthResult {
     }
 }
 
+impl From<AuthResult> for IprevResult {
+    fn from(value: AuthResult) -> Self {
+        match value {
+            AuthResult::Pass => IprevResult::Pass,
+            AuthResult::Fail { details } => {
+                IprevResult::Fail(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::TempError { details } => {
+                IprevResult::TempError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::PermError { details } => {
+                IprevResult::PermError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::None => IprevResult::None,
+            _ => IprevResult::None,
+        }
+    }
+}
+
 impl From<&DkimResult> for AuthResult {
     fn from(value: &DkimResult) -> Self {
         match value {
@@ -1031,6 +1124,27 @@ impl From<&DkimResult> for AuthResult {
     }
 }
 
+impl From<AuthResult> for DkimResult {
+    fn from(value: AuthResult) -> Self {
+        match value {
+            AuthResult::Pass => DkimResult::Pass,
+            AuthResult::Neutral { details } => {
+                DkimResult::Neutral(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::Fail { details } => {
+                DkimResult::Fail(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::PermError { details } => {
+                DkimResult::PermError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::TempError { details } => {
+                DkimResult::TempError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            _ => DkimResult::None,
+        }
+    }
+}
+
 impl From<&DmarcResult> for AuthResult {
     fn from(value: &DmarcResult) -> Self {
         match value {
@@ -1049,6 +1163,25 @@ impl From<&DmarcResult> for AuthResult {
     }
 }
 
+impl From<AuthResult> for DmarcResult {
+    fn from(value: AuthResult) -> Self {
+        match value {
+            AuthResult::Pass => DmarcResult::Pass,
+            AuthResult::Fail { details } => {
+                DmarcResult::Fail(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::TempError { details } => {
+                DmarcResult::TempError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::PermError { details } => {
+                DmarcResult::PermError(mail_auth::Error::Io(details.unwrap_or_default()))
+            }
+            AuthResult::None => DmarcResult::None,
+            _ => DmarcResult::None,
+        }
+    }
+}
+
 impl From<&dmarc::Policy> for DmarcPolicy {
     fn from(value: &dmarc::Policy) -> Self {
         match value {
@@ -1059,3 +1192,14 @@ impl From<&dmarc::Policy> for DmarcPolicy {
         }
     }
 }
+
+impl From<DmarcPolicy> for dmarc::Policy {
+    fn from(value: DmarcPolicy) -> Self {
+        match value {
+            DmarcPolicy::None => dmarc::Policy::None,
+            DmarcPolicy::Quarantine => dmarc::Policy::Quarantine,
+            DmarcPolicy::Reject => dmarc::Policy::Reject,
+            DmarcPolicy::Unspecified => dmarc::Policy::Unspecified,
+        }
+    }
+}
