@@ -8,6 +8,12 @@ use common::{
     auth::{AccessToken, ResourceToken},
     Server,
 };
+use email::{
+    index::{EmailIndexBuilder, TrimTextValue, VisitValues, MAX_ID_LENGTH, MAX_SORT_FIELD_LENGTH},
+    ingest::{EmailIngest, IngestedEmail, LogEmailInsert},
+    mailbox::{MailboxFnc, UidMailbox},
+    metadata::MessageMetadata,
+};
 use jmap_proto::{
     error::set::SetError,
     method::{
@@ -37,30 +43,17 @@ use mail_parser::{parsers::fields::thread::thread_name, HeaderName, HeaderValue}
 use store::{
     write::{
         log::{Changes, LogInsert},
-        BatchBuilder, Bincode, FtsQueueClass, MaybeDynamicId, TagValue, ValueClass, F_BITMAP,
+        BatchBuilder, Bincode, MaybeDynamicId, TagValue, TaskQueueClass, ValueClass, F_BITMAP,
         F_VALUE,
     },
-    BlobClass, Serialize,
+    BlobClass,
 };
 use trc::AddContext;
 use utils::map::vec_map::VecMap;
 
-use crate::{
-    api::http::HttpSessionData,
-    auth::acl::AclMethods,
-    changes::{state::StateManager, write::ChangeLog},
-    mailbox::{set::MailboxSet, UidMailbox},
-    services::index::Indexer,
-    JmapMethods,
-};
+use crate::{api::http::HttpSessionData, auth::acl::AclMethods, changes::state::StateManager};
 use std::future::Future;
 
-use super::{
-    index::{EmailIndexBuilder, TrimTextValue, VisitValues, MAX_ID_LENGTH, MAX_SORT_FIELD_LENGTH},
-    ingest::{EmailIngest, IngestedEmail, LogEmailInsert},
-    metadata::MessageMetadata,
-};
-
 pub trait EmailCopy: Sync + Send {
     fn email_copy(
         &self,
@@ -413,7 +406,7 @@ impl EmailCopy for Server {
         }
 
         // Prepare batch
-        let change_id = self.assign_change_id(account_id).await?;
+        let change_id = self.assign_change_id(account_id)?;
         let mut batch = BatchBuilder::new();
         batch
             .with_account_id(account_id)
@@ -441,11 +434,11 @@ impl EmailCopy for Server {
             .value(Property::Keywords, keywords, F_VALUE | F_BITMAP)
             .value(Property::Cid, change_id, F_VALUE)
             .set(
-                ValueClass::FtsQueue(FtsQueueClass {
+                ValueClass::TaskQueue(TaskQueueClass::IndexEmail {
                     seq: self.generate_snowflake_id()?,
                     hash: metadata.blob_hash.clone(),
                 }),
-                0u64.serialize(),
+                vec![],
             );
         EmailIndexBuilder::set(metadata).build(
             &mut batch,
@@ -468,7 +461,7 @@ impl EmailCopy for Server {
         let document_id = ids.last_document_id().caused_by(trc::location!())?;
 
         // Request FTS index
-        self.request_fts_index();
+        self.notify_task_queue();
 
         // Update response
         email.id = Id::from_parts(thread_id, document_id);
