@@ -4,17 +4,14 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
+pub mod bayes;
 pub mod body;
-pub mod cache;
 pub mod copy;
 pub mod crypto;
 pub mod delete;
 pub mod get;
 pub mod headers;
 pub mod import;
-pub mod index;
-pub mod ingest;
-pub mod metadata;
 pub mod parse;
 pub mod query;
 pub mod set;
