@@ -7,6 +7,10 @@
 use std::{borrow::Cow, collections::HashMap, slice::IterMut};
 
 use common::{auth::AccessToken, Server};
+use email::{
+    ingest::{EmailIngest, IngestEmail, IngestSource},
+    mailbox::{MailboxFnc, UidMailbox},
+};
 use jmap_proto::{
     error::set::{SetError, SetErrorType},
     method::set::{RequestArguments, SetRequest, SetResponse},
@@ -42,19 +46,14 @@ use store::{
 use trc::AddContext;
 
 use crate::{
-    api::http::HttpSessionData,
-    auth::acl::AclMethods,
-    blob::download::BlobDownload,
-    changes::{state::StateManager, write::ChangeLog},
-    mailbox::{set::MailboxSet, UidMailbox},
-    JmapMethods,
+    api::http::HttpSessionData, auth::acl::AclMethods, blob::download::BlobDownload,
+    changes::state::StateManager, JmapMethods,
 };
 use std::future::Future;
 
 use super::{
     delete::EmailDeletion,
     headers::{BuildHeader, ValueToHeader},
-    ingest::{EmailIngest, IngestEmail, IngestSource},
 };
 
 pub trait EmailSet: Sync + Send {
@@ -78,6 +77,7 @@ impl EmailSet for Server {
         let mut response = self
             .prepare_set_response(&request, Collection::Email)
             .await?;
+        let can_train_spam = self.email_bayes_can_train(access_token);
 
         // Obtain mailboxIds
         let mailbox_ids = self.mailbox_get_or_create(account_id).await?;
@@ -433,7 +433,7 @@ impl EmailSet for Server {
                                     }
                                 } else if expected_content_type
                                     .as_ref()
-                                    .map_or(false, |v| v != &content_type)
+                                    .is_some_and(|v| v != &content_type)
                                 {
                                     response.not_created.append(
                                         id,
@@ -738,7 +738,8 @@ impl EmailSet for Server {
                     keywords,
                     received_at,
                     source: IngestSource::Jmap,
-                    encrypt: self.core.jmap.encrypt && self.core.jmap.encrypt_append,
+                    spam_classify: false,
+                    spam_train: can_train_spam,
                     session_id: session.session_id,
                 })
                 .await
@@ -889,7 +890,7 @@ impl EmailSet for Server {
 
                 // Update last change id
                 if changes.change_id == u64::MAX {
-                    changes.change_id = self.assign_change_id(account_id).await?;
+                    changes.change_id = self.assign_change_id(account_id)?;
                 }
                 batch.value(Property::Cid, changes.change_id, F_VALUE);
             }
