@@ -6,7 +6,8 @@
 
 use std::{
     collections::BinaryHeap,
-    sync::{atomic::Ordering, Arc},
+    future::Future,
+    sync::Arc,
     time::{Duration, Instant, SystemTime},
 };
 
@@ -14,7 +15,7 @@ use common::{
     config::telemetry::OtelMetrics,
     core::BuildServer,
     ipc::{HousekeeperEvent, PurgeType},
-    Inner,
+    Inner, Server, KV_LOCK_HOUSEKEEPER,
 };
 
 #[cfg(feature = "enterprise")]
@@ -24,10 +25,9 @@ use common::telemetry::{
 };
 
 use smtp::reporting::SmtpReporting;
-use store::write::{now, purge::PurgeStore};
+use store::{write::now, PurgeStore};
 use tokio::sync::mpsc;
-use trc::{Collector, MetricType};
-use utils::map::ttl_dashmap::TtlMap;
+use trc::{Collector, MetricType, PurgeEvent};
 
 use crate::{email::delete::EmailDeletion, JmapMethods, LONG_SLUMBER};
 
@@ -39,7 +39,6 @@ struct Action {
 
 #[derive(PartialEq, Eq, Debug)]
 enum ActionClass {
-    Session,
     Account,
     Store(usize),
     Acme(String),
@@ -71,48 +70,52 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
         {
             let server = inner.build_server();
 
-            // Session purge
-            queue.schedule(
-                Instant::now() + server.core.jmap.session_purge_frequency.time_to_next(),
-                ActionClass::Session,
-            );
-
             // Account purge
-            queue.schedule(
-                Instant::now() + server.core.jmap.account_purge_frequency.time_to_next(),
-                ActionClass::Account,
-            );
-
-            // Store purges
-            for (idx, schedule) in server.core.storage.purge_schedules.iter().enumerate() {
+            if server.core.network.roles.purge_accounts {
                 queue.schedule(
-                    Instant::now() + schedule.cron.time_to_next(),
-                    ActionClass::Store(idx),
+                    Instant::now() + server.core.jmap.account_purge_frequency.time_to_next(),
+                    ActionClass::Account,
                 );
             }
 
+            // Store purges
+            if server.core.network.roles.purge_stores {
+                for (idx, schedule) in server.core.storage.purge_schedules.iter().enumerate() {
+                    queue.schedule(
+                        Instant::now() + schedule.cron.time_to_next(),
+                        ActionClass::Store(idx),
+                    );
+                }
+            }
+
             // OTEL Push Metrics
-            if let Some(otel) = &server.core.metrics.otel {
-                OtelMetrics::enable_errors();
-                queue.schedule(Instant::now() + otel.interval, ActionClass::OtelMetrics);
+            if server.core.network.roles.push_metrics {
+                if let Some(otel) = &server.core.metrics.otel {
+                    OtelMetrics::enable_errors();
+                    queue.schedule(Instant::now() + otel.interval, ActionClass::OtelMetrics);
+                }
             }
 
             // Calculate expensive metrics
             queue.schedule(Instant::now(), ActionClass::CalculateMetrics);
 
             // Add all ACME renewals to heap
-            for provider in server.core.acme.providers.values() {
-                match server.init_acme(provider).await {
-                    Ok(renew_at) => {
-                        queue.schedule(
-                            Instant::now() + renew_at,
-                            ActionClass::Acme(provider.id.clone()),
-                        );
-                    }
-                    Err(err) => {
-                        trc::error!(err.details("Failed to initialize ACME certificate manager."));
-                    }
-                };
+            if server.core.network.roles.renew_acme {
+                for provider in server.core.acme.providers.values() {
+                    match server.init_acme(provider).await {
+                        Ok(renew_at) => {
+                            queue.schedule(
+                                Instant::now() + renew_at,
+                                ActionClass::Acme(provider.id.clone()),
+                            );
+                        }
+                        Err(err) => {
+                            trc::error!(
+                                err.details("Failed to initialize ACME certificate manager.")
+                            );
+                        }
+                    };
+                }
             }
 
             // SPDX-SnippetBegin
@@ -229,94 +232,12 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                         queue.remove_action(&action);
                         queue.schedule(renew_at, action);
                     }
-                    HousekeeperEvent::Purge(purge) => match purge {
-                        PurgeType::Data(store) => {
-                            // SPDX-SnippetBegin
-                            // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
-                            // SPDX-License-Identifier: LicenseRef-SEL
-                            #[cfg(feature = "enterprise")]
-                            let trace_retention = inner
-                                .shared_core
-                                .load()
-                                .enterprise
-                                .as_ref()
-                                .and_then(|e| e.trace_store.as_ref())
-                                .and_then(|t| t.retention);
-                            #[cfg(feature = "enterprise")]
-                            let metrics_retention = inner
-                                .shared_core
-                                .load()
-                                .enterprise
-                                .as_ref()
-                                .and_then(|e| e.metrics_store.as_ref())
-                                .and_then(|m| m.retention);
-                            // SPDX-SnippetEnd
-
-                            tokio::spawn(async move {
-                                trc::event!(
-                                    Housekeeper(trc::HousekeeperEvent::PurgeStore),
-                                    Type = "data"
-                                );
-                                if let Err(err) = store.purge_store().await {
-                                    trc::error!(err.details("Failed to purge data store"));
-                                }
-
-                                // SPDX-SnippetBegin
-                                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
-                                // SPDX-License-Identifier: LicenseRef-SEL
-                                #[cfg(feature = "enterprise")]
-                                if let Some(trace_retention) = trace_retention {
-                                    if let Err(err) = store.purge_spans(trace_retention).await {
-                                        trc::error!(err.details("Failed to purge tracing spans"));
-                                    }
-                                }
-
-                                #[cfg(feature = "enterprise")]
-                                if let Some(metrics_retention) = metrics_retention {
-                                    if let Err(err) = store.purge_metrics(metrics_retention).await {
-                                        trc::error!(err.details("Failed to purge metrics"));
-                                    }
-                                }
-                                // SPDX-SnippetEnd
-                            });
-                        }
-                        PurgeType::Blobs { store, blob_store } => {
-                            trc::event!(
-                                Housekeeper(trc::HousekeeperEvent::PurgeStore),
-                                Type = "blob"
-                            );
-
-                            tokio::spawn(async move {
-                                if let Err(err) = store.purge_blobs(blob_store).await {
-                                    trc::error!(err.details("Failed to purge blob store"));
-                                }
-                            });
-                        }
-                        PurgeType::Lookup(store) => {
-                            trc::event!(
-                                Housekeeper(trc::HousekeeperEvent::PurgeStore),
-                                Type = "lookup"
-                            );
-
-                            tokio::spawn(async move {
-                                if let Err(err) = store.purge_lookup_store().await {
-                                    trc::error!(err.details("Failed to purge lookup store"));
-                                }
-                            });
-                        }
-                        PurgeType::Account(account_id) => {
-                            let server = inner.build_server();
-                            tokio::spawn(async move {
-                                trc::event!(Housekeeper(trc::HousekeeperEvent::PurgeAccounts));
-
-                                if let Some(account_id) = account_id {
-                                    server.purge_account(account_id).await;
-                                } else {
-                                    server.purge_accounts().await;
-                                }
-                            });
-                        }
-                    },
+                    HousekeeperEvent::Purge(purge) => {
+                        let server = inner.build_server();
+                        tokio::spawn(async move {
+                            server.purge(purge, 0).await;
+                        });
+                    }
                     HousekeeperEvent::Exit => {
                         trc::event!(Housekeeper(trc::HousekeeperEvent::Stop));
 
@@ -332,6 +253,8 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                     while let Some(event) = queue.pop() {
                         match event.event {
                             ActionClass::Acme(provider_id) => {
+                                trc::event!(Housekeeper(trc::HousekeeperEvent::Run), Type = "acme");
+
                                 let server = server.clone();
                                 tokio::spawn(async move {
                                     if let Some(provider) =
@@ -379,6 +302,11 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                                 });
                             }
                             ActionClass::Account => {
+                                trc::event!(
+                                    Housekeeper(trc::HousekeeperEvent::Run),
+                                    Type = "purge_account"
+                                );
+
                                 let server = server.clone();
                                 queue.schedule(
                                     Instant::now()
@@ -386,79 +314,55 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                                     ActionClass::Account,
                                 );
                                 tokio::spawn(async move {
-                                    trc::event!(Housekeeper(trc::HousekeeperEvent::PurgeAccounts));
-                                    server.purge_accounts().await;
-                                });
-                            }
-                            ActionClass::Session => {
-                                let server = server.clone();
-                                queue.schedule(
-                                    Instant::now()
-                                        + server.core.jmap.session_purge_frequency.time_to_next(),
-                                    ActionClass::Session,
-                                );
-
-                                tokio::spawn(async move {
-                                    trc::event!(Housekeeper(trc::HousekeeperEvent::PurgeSessions));
-                                    server.inner.data.http_auth_cache.cleanup();
-                                    server
-                                        .inner
-                                        .data
-                                        .jmap_limiter
-                                        .retain(|_, limiter| limiter.is_active());
-                                    server.inner.data.access_tokens.cleanup();
-
-                                    for throttle in [
-                                        &server.inner.data.smtp_session_throttle,
-                                        &server.inner.data.smtp_queue_throttle,
-                                    ] {
-                                        throttle.retain(|_, v| {
-                                            v.concurrent.load(Ordering::Relaxed) > 0
-                                        });
-                                    }
+                                    server.purge(PurgeType::Account(None), 0).await;
                                 });
                             }
                             ActionClass::Store(idx) => {
                                 if let Some(schedule) =
                                     server.core.storage.purge_schedules.get(idx).cloned()
                                 {
+                                    trc::event!(
+                                        Housekeeper(trc::HousekeeperEvent::Run),
+                                        Type = "purge_store",
+                                        Id = idx
+                                    );
+
                                     queue.schedule(
                                         Instant::now() + schedule.cron.time_to_next(),
                                         ActionClass::Store(idx),
                                     );
-                                    tokio::spawn(async move {
-                                        let (class, result) = match schedule.store {
-                                            PurgeStore::Data(store) => {
-                                                ("data", store.purge_store().await)
-                                            }
-                                            PurgeStore::Blobs { store, blob_store } => {
-                                                ("blob", store.purge_blobs(blob_store).await)
-                                            }
-                                            PurgeStore::Lookup(lookup_store) => {
-                                                ("lookup", lookup_store.purge_lookup_store().await)
-                                            }
-                                        };
 
-                                        match result {
-                                            Ok(_) => {
-                                                trc::event!(
-                                                    Housekeeper(trc::HousekeeperEvent::PurgeStore),
-                                                    Id = schedule.store_id
-                                                );
-                                            }
-                                            Err(err) => {
-                                                trc::error!(err
-                                                    .details(format!(
-                                                        "Failed to purge {class} store."
-                                                    ))
-                                                    .id(schedule.store_id));
-                                            }
-                                        }
+                                    let server = server.clone();
+                                    tokio::spawn(async move {
+                                        server
+                                            .purge(
+                                                match schedule.store {
+                                                    PurgeStore::Data(store) => {
+                                                        PurgeType::Data(store)
+                                                    }
+                                                    PurgeStore::Blobs { store, blob_store } => {
+                                                        PurgeType::Blobs { store, blob_store }
+                                                    }
+                                                    PurgeStore::Lookup(in_memory_store) => {
+                                                        PurgeType::Lookup {
+                                                            store: in_memory_store,
+                                                            prefix: None,
+                                                        }
+                                                    }
+                                                },
+                                                idx as u32,
+                                            )
+                                            .await;
                                     });
                                 }
                             }
                             ActionClass::OtelMetrics => {
                                 if let Some(otel) = &server.core.metrics.otel {
+                                    trc::event!(
+                                        Housekeeper(trc::HousekeeperEvent::Run),
+                                        Type = "metrics_report"
+                                    );
+
                                     queue.schedule(
                                         Instant::now() + otel.interval,
                                         ActionClass::OtelMetrics,
@@ -478,6 +382,11 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                                 }
                             }
                             ActionClass::CalculateMetrics => {
+                                trc::event!(
+                                    Housekeeper(trc::HousekeeperEvent::Run),
+                                    Type = "metrics_calculate"
+                                );
+
                                 // Calculate expensive metrics every 5 minutes
                                 queue.schedule(
                                     Instant::now() + Duration::from_secs(5 * 60),
@@ -494,50 +403,50 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
 
                                 let server = server.clone();
                                 tokio::spawn(async move {
-                                    #[cfg(feature = "enterprise")]
-                                    if server.is_enterprise_edition() {
-                                        // Obtain queue size
-                                        match server.total_queued_messages().await {
-                                            Ok(total) => {
-                                                Collector::update_gauge(
-                                                    MetricType::QueueCount,
-                                                    total,
-                                                );
-                                            }
-                                            Err(err) => {
-                                                trc::error!(
-                                                    err.details("Failed to obtain queue size")
-                                                );
-                                            }
-                                        }
-                                    }
-
-                                    if update_other_metrics {
-                                        match server.total_accounts().await {
-                                            Ok(total) => {
-                                                Collector::update_gauge(
-                                                    MetricType::UserCount,
-                                                    total,
-                                                );
-                                            }
-                                            Err(err) => {
-                                                trc::error!(
-                                                    err.details("Failed to obtain account count")
-                                                );
+                                    if server.core.network.roles.calculate_metrics {
+                                        #[cfg(feature = "enterprise")]
+                                        if server.is_enterprise_edition() {
+                                            // Obtain queue size
+                                            match server.total_queued_messages().await {
+                                                Ok(total) => {
+                                                    Collector::update_gauge(
+                                                        MetricType::QueueCount,
+                                                        total,
+                                                    );
+                                                }
+                                                Err(err) => {
+                                                    trc::error!(
+                                                        err.details("Failed to obtain queue size")
+                                                    );
+                                                }
                                             }
                                         }
 
-                                        match server.total_domains().await {
-                                            Ok(total) => {
-                                                Collector::update_gauge(
-                                                    MetricType::DomainCount,
-                                                    total,
-                                                );
+                                        if update_other_metrics {
+                                            match server.total_accounts().await {
+                                                Ok(total) => {
+                                                    Collector::update_gauge(
+                                                        MetricType::UserCount,
+                                                        total,
+                                                    );
+                                                }
+                                                Err(err) => {
+                                                    trc::error!(err
+                                                        .details("Failed to obtain account count"));
+                                                }
                                             }
-                                            Err(err) => {
-                                                trc::error!(
-                                                    err.details("Failed to obtain domain count")
-                                                );
+
+                                            match server.total_domains().await {
+                                                Ok(total) => {
+                                                    Collector::update_gauge(
+                                                        MetricType::DomainCount,
+                                                        total,
+                                                    );
+                                                }
+                                                Err(err) => {
+                                                    trc::error!(err
+                                                        .details("Failed to obtain domain count"));
+                                                }
                                             }
                                         }
                                     }
@@ -575,6 +484,11 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
                                     .as_ref()
                                     .and_then(|e| e.metrics_store.as_ref())
                                 {
+                                    trc::event!(
+                                        Housekeeper(trc::HousekeeperEvent::Run),
+                                        Type = "metrics_internal"
+                                    );
+
                                     queue.schedule(
                                         Instant::now() + metrics_store.interval.time_to_next(),
                                         ActionClass::InternalMetrics,
@@ -596,6 +510,11 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
 
                             #[cfg(feature = "enterprise")]
                             ActionClass::AlertMetrics => {
+                                trc::event!(
+                                    Housekeeper(trc::HousekeeperEvent::Run),
+                                    Type = "metrics_alert"
+                                );
+
                                 let server = server.clone();
 
                                 tokio::spawn(async move {
@@ -617,6 +536,11 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
 
                             #[cfg(feature = "enterprise")]
                             ActionClass::RenewLicense => {
+                                trc::event!(
+                                    Housekeeper(trc::HousekeeperEvent::Run),
+                                    Type = "renew_license"
+                                );
+
                                 match server.reload().await {
                                     Ok(result) => {
                                         if let Some(new_core) = result.new_core {
@@ -662,6 +586,154 @@ pub fn spawn_housekeeper(inner: Arc<Inner>, mut rx: mpsc::Receiver<HousekeeperEv
     });
 }
 
+pub trait Purge: Sync + Send {
+    fn purge(&self, purge: PurgeType, store_idx: u32) -> impl Future<Output = ()> + Send;
+}
+
+impl Purge for Server {
+    async fn purge(&self, purge: PurgeType, store_idx: u32) {
+        // Lock task
+        let (lock_type, lock_name) = match &purge {
+            PurgeType::Data(_) => (
+                "data",
+                [0u8]
+                    .into_iter()
+                    .chain(store_idx.to_be_bytes().into_iter())
+                    .collect::<Vec<_>>()
+                    .into(),
+            ),
+            PurgeType::Blobs { .. } => (
+                "blob",
+                [1u8]
+                    .into_iter()
+                    .chain(store_idx.to_be_bytes().into_iter())
+                    .collect::<Vec<_>>()
+                    .into(),
+            ),
+            PurgeType::Lookup { prefix: None, .. } => (
+                "in-memory",
+                [2u8]
+                    .into_iter()
+                    .chain(store_idx.to_be_bytes().into_iter())
+                    .collect::<Vec<_>>()
+                    .into(),
+            ),
+            PurgeType::Lookup { .. } => ("in-memory-prefix", None),
+            PurgeType::Account(_) => ("account", None),
+        };
+        if let Some(lock_name) = &lock_name {
+            match self
+                .core
+                .storage
+                .lookup
+                .try_lock(KV_LOCK_HOUSEKEEPER, lock_name, 3600)
+                .await
+            {
+                Ok(true) => (),
+                Ok(false) => {
+                    trc::event!(Purge(PurgeEvent::InProgress), Details = lock_type);
+                    return;
+                }
+                Err(err) => {
+                    trc::error!(err.details("Failed to lock task.").details(lock_type));
+                    return;
+                }
+            }
+        }
+
+        trc::event!(Purge(PurgeEvent::Started), Type = lock_type, Id = store_idx);
+        let time = Instant::now();
+
+        match purge {
+            PurgeType::Data(store) => {
+                // SPDX-SnippetBegin
+                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                // SPDX-License-Identifier: LicenseRef-SEL
+                #[cfg(feature = "enterprise")]
+                let trace_retention = self
+                    .core
+                    .enterprise
+                    .as_ref()
+                    .and_then(|e| e.trace_store.as_ref())
+                    .and_then(|t| t.retention);
+                #[cfg(feature = "enterprise")]
+                let metrics_retention = self
+                    .core
+                    .enterprise
+                    .as_ref()
+                    .and_then(|e| e.metrics_store.as_ref())
+                    .and_then(|m| m.retention);
+                // SPDX-SnippetEnd
+
+                if let Err(err) = store.purge_store().await {
+                    trc::error!(err.details("Failed to purge data store"));
+                }
+
+                // SPDX-SnippetBegin
+                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                // SPDX-License-Identifier: LicenseRef-SEL
+                #[cfg(feature = "enterprise")]
+                if let Some(trace_retention) = trace_retention {
+                    if let Err(err) = store.purge_spans(trace_retention).await {
+                        trc::error!(err.details("Failed to purge tracing spans"));
+                    }
+                }
+
+                #[cfg(feature = "enterprise")]
+                if let Some(metrics_retention) = metrics_retention {
+                    if let Err(err) = store.purge_metrics(metrics_retention).await {
+                        trc::error!(err.details("Failed to purge metrics"));
+                    }
+                }
+                // SPDX-SnippetEnd
+            }
+            PurgeType::Blobs { store, blob_store } => {
+                if let Err(err) = store.purge_blobs(blob_store).await {
+                    trc::error!(err.details("Failed to purge blob store"));
+                }
+            }
+            PurgeType::Lookup { store, prefix } => {
+                if let Some(prefix) = prefix {
+                    if let Err(err) = store.key_delete_prefix(&prefix).await {
+                        trc::error!(err
+                            .details("Failed to delete key prefix")
+                            .ctx(trc::Key::Key, prefix));
+                    }
+                } else if let Err(err) = store.purge_in_memory_store().await {
+                    trc::error!(err.details("Failed to purge in-memory store"));
+                }
+            }
+            PurgeType::Account(account_id) => {
+                if let Some(account_id) = account_id {
+                    self.purge_account(account_id).await;
+                } else {
+                    self.purge_accounts().await;
+                }
+            }
+        }
+
+        trc::event!(
+            Purge(PurgeEvent::Finished),
+            Type = lock_type,
+            Id = store_idx,
+            Elapsed = time.elapsed()
+        );
+
+        // Remove lock
+        if let Some(lock_name) = &lock_name {
+            if let Err(err) = self
+                .in_memory_store()
+                .remove_lock(KV_LOCK_HOUSEKEEPER, lock_name)
+                .await
+            {
+                trc::error!(err
+                    .details("Failed to delete task lock.")
+                    .details(lock_type));
+            }
+        }
+    }
+}
+
 impl Queue {
     pub fn schedule(&mut self, due: Instant, event: ActionClass) {
         trc::event!(
