@@ -10,6 +10,7 @@ use common::{
     listener::{stream::NullIo, ServerInstance},
     Server,
 };
+use email::metadata::MessageMetadata;
 use jmap_proto::{
     error::set::{SetError, SetErrorType},
     method::set::{self, SetRequest, SetResponse},
@@ -38,12 +39,10 @@ use smtp::{
 };
 use smtp_proto::{request::parser::Rfc5321Parser, MailFrom, RcptTo};
 use store::write::{assert::HashedValue, log::ChangeLogBuilder, now, BatchBuilder, Bincode};
+use trc::AddContext;
 use utils::{map::vec_map::VecMap, sanitize_email};
 
-use crate::{
-    blob::download::BlobDownload, changes::write::ChangeLog, email::metadata::MessageMetadata,
-    JmapMethods,
-};
+use crate::blob::download::BlobDownload;
 use std::future::Future;
 
 pub static SCHEMA: &[IndexProperty] = &[
@@ -107,7 +106,11 @@ impl EmailSubmissionSet for Server {
                         .with_collection(Collection::EmailSubmission)
                         .create_document()
                         .custom(ObjectIndexBuilder::new(SCHEMA).with_changes(submission));
-                    let document_id = self.write_batch_expect_id(batch).await?;
+                    let document_id = self
+                        .store()
+                        .write_expect_id(batch)
+                        .await
+                        .caused_by(trc::location!())?;
                     changes.log_insert(Collection::EmailSubmission, document_id);
                     response.created(id, document_id);
                 }
@@ -193,7 +196,10 @@ impl EmailSubmissionSet for Server {
                                             .with_property(Property::UndoStatus, undo_status),
                                     ),
                             );
-                        self.write_batch(batch).await?;
+                        self.store()
+                            .write(batch)
+                            .await
+                            .caused_by(trc::location!())?;
                         changes.log_update(Collection::EmailSubmission, document_id);
                         response.updated.append(id, None);
                     } else {
@@ -242,7 +248,10 @@ impl EmailSubmissionSet for Server {
                     .with_collection(Collection::EmailSubmission)
                     .delete_document(document_id)
                     .custom(ObjectIndexBuilder::new(SCHEMA).with_current(submission));
-                self.write_batch(batch).await?;
+                self.store()
+                    .write(batch)
+                    .await
+                    .caused_by(trc::location!())?;
                 changes.log_delete(Collection::EmailSubmission, document_id);
                 response.destroyed.push(id);
             } else {
@@ -260,12 +269,12 @@ impl EmailSubmissionSet for Server {
             .arguments
             .on_success_destroy_email
             .as_ref()
-            .map_or(false, |p| !p.is_empty())
+            .is_some_and(|p| !p.is_empty())
             || request
                 .arguments
                 .on_success_update_email
                 .as_ref()
-                .map_or(false, |p| !p.is_empty()))
+                .is_some_and(|p| !p.is_empty()))
             && response.has_changes()
         {
             *next_call = Call {
@@ -507,6 +516,7 @@ impl EmailSubmissionSet for Server {
         };
 
         // Add recipients to envelope if missing
+        let mut bcc_header = None;
         if rcpt_to.is_empty() {
             let mut envelope_values = Vec::new();
             for header in &metadata.contents.parts[0].headers {
@@ -514,6 +524,9 @@ impl EmailSubmissionSet for Server {
                     header.name,
                     HeaderName::To | HeaderName::Cc | HeaderName::Bcc
                 ) {
+                    if matches!(header.name, HeaderName::Bcc) {
+                        bcc_header = Some(header);
+                    }
                     if let HeaderValue::Address(addr) = &header.value {
                         for address in addr.iter() {
                             if let Some(address) = address.address().and_then(sanitize_email) {
@@ -546,6 +559,11 @@ impl EmailSubmissionSet for Server {
                 return Ok(Err(SetError::new(SetErrorType::NoRecipients)
                     .with_description("No recipients found in email.")));
             }
+        } else {
+            bcc_header = metadata.contents.parts[0]
+                .headers
+                .iter()
+                .find(|header| matches!(header.name, HeaderName::Bcc));
         }
 
         // Update sendAt
@@ -561,7 +579,7 @@ impl EmailSubmissionSet for Server {
         );
 
         // Obtain raw message
-        let message =
+        let mut message =
             if let Some(message) = self.get_blob(&metadata.blob_hash, 0..usize::MAX).await? {
                 if message.len() > self.core.jmap.mail_max_size {
                     return Ok(Err(SetError::new(SetErrorType::InvalidEmail)
@@ -578,6 +596,14 @@ impl EmailSubmissionSet for Server {
                     .with_description("Blob for email not found.")));
             };
 
+        // Remove BCC header if present
+        if let Some(bcc_header) = bcc_header {
+            let mut new_message = Vec::with_capacity(message.len());
+            new_message.extend_from_slice(&message[..bcc_header.offset_field]);
+            new_message.extend_from_slice(&message[bcc_header.offset_end..]);
+            message = new_message;
+        }
+
         // Begin local SMTP session
         let mut session =
             Session::<NullIo>::local(self.clone(), instance.clone(), SessionData::default());
