@@ -7,8 +7,8 @@
 use std::time::{Duration, Instant, SystemTime};
 
 use common::{config::smtp::session::Stage, listener::SessionStream, scripts::ScriptModification};
-use mail_auth::{IprevOutput, IprevResult, SpfOutput, SpfResult};
-use smtp_proto::{MailFrom, MtPriority, MAIL_BY_NOTIFY, MAIL_BY_RETURN, MAIL_REQUIRETLS};
+use mail_auth::{IprevOutput, IprevResult, SpfOutput, SpfResult, spf::verify::SpfParameters};
+use smtp_proto::{MAIL_BY_NOTIFY, MAIL_BY_RETURN, MAIL_REQUIRETLS, MailFrom, MtPriority};
 use trc::SmtpEvent;
 use utils::config::Rate;
 
@@ -59,7 +59,12 @@ impl<T: SessionStream> Session<T> {
                 .smtp
                 .resolvers
                 .dns
-                .verify_iprev(self.data.remote_ip)
+                .verify_iprev(
+                    self.server
+                        .inner
+                        .cache
+                        .build_auth_parameters(self.data.remote_ip),
+                )
                 .await;
 
             trc::event!(
@@ -70,7 +75,7 @@ impl<T: SessionStream> Session<T> {
                 }),
                 SpanId = self.data.session_id,
                 Domain = self.data.helo_domain.clone(),
-                Result = trc::Event::from(&iprev),
+                Result = trc::Error::from(&iprev),
                 Elapsed = time.elapsed(),
             );
 
@@ -226,7 +231,17 @@ impl<T: SessionStream> Session<T> {
 
         // Make sure that the authenticated user is allowed to send from this address
         match self.authenticated_as() {
-            Some(authenticated_as) if self.params.auth_match_sender => {
+            Some(authenticated_as)
+                if self
+                    .server
+                    .eval_if(
+                        &self.server.core.smtp.session.auth.must_match_sender,
+                        self,
+                        self.data.session_id,
+                    )
+                    .await
+                    .unwrap_or(true) =>
+            {
                 let address_lcase = self.data.mail_from.as_ref().unwrap().address_lcase.as_str();
                 if authenticated_as != address_lcase
                     && !self.authenticated_emails().iter().any(|e| {
@@ -435,13 +450,15 @@ impl<T: SessionStream> Session<T> {
                         .smtp
                         .resolvers
                         .dns
-                        .check_host(
-                            self.data.remote_ip,
-                            &mail_from.domain,
-                            &self.data.helo_domain,
-                            &self.hostname,
-                            &mail_from.address_lcase,
-                        )
+                        .check_host(self.server.inner.cache.build_auth_parameters(
+                            SpfParameters::new(
+                                self.data.remote_ip,
+                                &mail_from.domain,
+                                &self.data.helo_domain,
+                                &self.hostname,
+                                &mail_from.address_lcase,
+                            ),
+                        ))
                         .await
                 } else {
                     self.server
@@ -449,13 +466,15 @@ impl<T: SessionStream> Session<T> {
                         .smtp
                         .resolvers
                         .dns
-                        .check_host(
-                            self.data.remote_ip,
-                            &self.data.helo_domain,
-                            &self.data.helo_domain,
-                            &self.hostname,
-                            &format!("postmaster@{}", self.data.helo_domain),
-                        )
+                        .check_host(self.server.inner.cache.build_auth_parameters(
+                            SpfParameters::new(
+                                self.data.remote_ip,
+                                &self.data.helo_domain,
+                                &self.data.helo_domain,
+                                &self.hostname,
+                                &format!("postmaster@{}", self.data.helo_domain),
+                            ),
+                        ))
                         .await
                 };
 
@@ -473,7 +492,7 @@ impl<T: SessionStream> Session<T> {
                         "<>"
                     }
                     .to_string(),
-                    Result = trc::Event::from(&spf_output),
+                    Result = trc::Error::from(&spf_output),
                     Elapsed = time.elapsed(),
                 );
 
@@ -504,7 +523,7 @@ impl<T: SessionStream> Session<T> {
             );
 
             self.data.mail_from = None;
-            self.write(b"451 4.4.5 Rate limit exceeded, try again later.\r\n")
+            self.write(b"452 4.4.5 Rate limit exceeded, try again later.\r\n")
                 .await
         }
     }
@@ -554,10 +573,11 @@ impl<T: SessionStream> Session<T> {
                 Ok(true) => return Ok(result),
                 Ok(false) => (),
                 Err(err) => {
-                    trc::error!(err
-                        .caused_by(trc::location!())
-                        .span_id(self.data.session_id)
-                        .details("Failed to lookup local domain"));
+                    trc::error!(
+                        err.caused_by(trc::location!())
+                            .span_id(self.data.session_id)
+                            .details("Failed to lookup local domain")
+                    );
                 }
             }
 
