@@ -4,32 +4,34 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use crate::outbound::client::{from_error_status, from_mail_send_error, SmtpClient};
+use crate::outbound::client::{SmtpClient, from_error_status, from_mail_send_error};
 use crate::outbound::dane::dnssec::TlsaLookup;
 use crate::outbound::lookup::DnsLookup;
 use crate::outbound::mta_sts::lookup::MtaStsLookup;
 use crate::outbound::mta_sts::verify::VerifyPolicy;
 use crate::outbound::{client::StartTlsResult, dane::verify::TlsaVerify};
 use crate::queue::dsn::SendDsn;
-use crate::queue::spool::SmtpSpool;
+use crate::queue::spool::{LOCK_EXPIRY, SmtpSpool};
 use crate::queue::throttle::IsAllowed;
 use crate::reporting::SmtpReporting;
+use common::Server;
 use common::config::{
     server::ServerProtocol,
     smtp::{queue::RequireOptional, report::AggregateFrequency},
 };
-use common::ipc::{OnHold, PolicyType, QueueEvent, TlsEvent};
-use common::Server;
+use common::ipc::{PolicyType, QueueEvent, QueueEventStatus, TlsEvent};
 use mail_auth::{
     mta_sts::TlsRpt,
     report::tlsrpt::{FailureDetails, ResultType},
 };
+use rand::Rng;
 use smtp_proto::MAIL_REQUIRETLS;
+use std::sync::Arc;
 use std::{
     net::{IpAddr, Ipv4Addr, SocketAddr},
     time::{Duration, Instant},
 };
-use store::write::{now, BatchBuilder, QueueClass, ValueClass};
+use store::write::{BatchBuilder, QueueClass, ValueClass, now};
 use trc::{DaneEvent, DeliveryEvent, MtaStsEvent, ServerEvent, TlsRptEvent};
 
 use crate::{
@@ -37,18 +39,16 @@ use crate::{
     reporting::tls::TlsRptOptions,
 };
 
-use super::{lookup::ToNextHop, mta_sts, session::SessionParams, NextHop, TlsStrategy};
-use crate::queue::{throttle, DeliveryAttempt, Domain, Error, QueueEnvelope, Status};
+use super::{NextHop, TlsStrategy, lookup::ToNextHop, mta_sts, session::SessionParams};
+use crate::queue::{Domain, Error, QueueEnvelope, QueuedMessage, Status};
 
-impl DeliveryAttempt {
-    pub async fn try_deliver(mut self, server: Server) {
+impl QueuedMessage {
+    pub fn try_deliver(self, server: Server) {
         tokio::spawn(async move {
-            // Lock message
-            if let Some(event) = server.try_lock_event(self.event).await {
-                self.event = event;
-
-                // Fetch message
-                if let Some(mut message) = server.read_message(self.event.queue_id).await {
+            // Lock queue event
+            let queue_id = self.queue_id;
+            let status = if server.try_lock_event(queue_id).await {
+                if let Some(mut message) = server.read_message(queue_id).await {
                     // Generate span id
                     message.span_id = server.inner.data.span_id_gen.generate().unwrap_or_else(now);
                     let span_id = message.span_id;
@@ -82,34 +82,65 @@ impl DeliveryAttempt {
 
                     // Attempt delivery
                     let start_time = Instant::now();
-                    self.deliver_task(server, message).await;
+                    let queue_event = self.deliver_task(server.clone(), message).await;
 
                     trc::event!(
                         Delivery(DeliveryEvent::AttemptEnd),
                         SpanId = span_id,
                         Elapsed = start_time.elapsed(),
                     );
+
+                    // Unlock event
+                    server.unlock_event(queue_id).await;
+
+                    queue_event
                 } else {
                     // Message no longer exists, delete queue event.
                     let mut batch = BatchBuilder::new();
                     batch.clear(ValueClass::Queue(QueueClass::MessageEvent(
                         store::write::QueueEvent {
-                            due: self.event.due,
-                            queue_id: self.event.queue_id,
+                            due: self.due,
+                            queue_id: self.queue_id,
                         },
                     )));
 
                     if let Err(err) = server.store().write(batch.build()).await {
-                        trc::error!(err
-                            .details("Failed to delete queue event.")
-                            .caused_by(trc::location!()));
+                        trc::error!(
+                            err.details("Failed to delete queue event.")
+                                .caused_by(trc::location!())
+                        );
                     }
+
+                    // Unlock event
+                    server.unlock_event(queue_id).await;
+
+                    QueueEventStatus::Completed
                 }
+            } else {
+                QueueEventStatus::Locked {
+                    until: now() + LOCK_EXPIRY + rand::rng().random_range(5..10),
+                }
+            };
+
+            // Notify queue manager
+            if server
+                .inner
+                .ipc
+                .queue_tx
+                .send(QueueEvent::WorkerDone { queue_id, status })
+                .await
+                .is_err()
+            {
+                trc::event!(
+                    Server(ServerEvent::ThreadError),
+                    Reason = "Channel closed.",
+                    CausedBy = trc::location!(),
+                );
             }
         });
     }
 
-    async fn deliver_task(mut self, server: Server, mut message: Message) {
+    async fn deliver_task(self, server: Server, mut message: Message) -> QueueEventStatus {
         // Check that the message still has recipients to be delivered
         let has_pending_delivery = message.has_pending_delivery();
         let span_id = message.span_id;
@@ -123,24 +154,9 @@ impl DeliveryAttempt {
             if due > now() {
                 // Save changes
                 message
-                    .save_changes(&server, self.event.due.into(), due.into())
+                    .save_changes(&server, self.due.into(), due.into())
                     .await;
-                if server
-                    .inner
-                    .ipc
-                    .queue_tx
-                    .send(QueueEvent::Reload)
-                    .await
-                    .is_err()
-                {
-                    trc::event!(
-                        Server(ServerEvent::ThreadError),
-                        Reason = "Channel closed.",
-                        CausedBy = trc::location!(),
-                        SpanId = span_id
-                    );
-                }
-                return;
+                return QueueEventStatus::Deferred;
             }
         } else {
             trc::event!(
@@ -150,86 +166,36 @@ impl DeliveryAttempt {
             );
 
             // All message recipients expired, do not re-queue. (DSN has been already sent)
-            message.remove(&server, self.event.due).await;
-            if server
-                .inner
-                .ipc
-                .queue_tx
-                .send(QueueEvent::Reload)
-                .await
-                .is_err()
-            {
-                trc::event!(
-                    Server(ServerEvent::ThreadError),
-                    Reason = "Channel closed.",
-                    CausedBy = trc::location!(),
-                    SpanId = span_id
-                );
-            }
+            message.remove(&server, self.due).await;
 
-            return;
+            return QueueEventStatus::Completed;
         }
 
         // Throttle sender
-        for throttle in &server.core.smtp.queue.throttle.sender {
-            if let Err(err) = server
-                .is_allowed(throttle, &message, &mut self.in_flight, message.span_id)
-                .await
-            {
-                let event = match err {
-                    throttle::Error::Concurrency { limiter } => {
-                        // Save changes to disk
-                        let next_due = message.next_event_after(now());
-                        message.save_changes(&server, None, None).await;
+        for throttle in &server.core.smtp.queue.outbound_limiters.sender {
+            if let Err(retry_at) = server.is_allowed(throttle, &message, message.span_id).await {
+                // Save changes to disk
+                let next_event = std::cmp::min(
+                    retry_at,
+                    message.next_event_after(now()).unwrap_or(u64::MAX),
+                );
 
-                        trc::event!(
-                            Delivery(DeliveryEvent::ConcurrencyLimitExceeded),
-                            Id = throttle.id.clone(),
-                            SpanId = span_id,
-                        );
+                trc::event!(
+                    Delivery(DeliveryEvent::RateLimitExceeded),
+                    Id = throttle.id.clone(),
+                    SpanId = span_id,
+                    NextRetry = trc::Value::Timestamp(next_event)
+                );
 
-                        QueueEvent::OnHold(OnHold {
-                            next_due,
-                            limiters: vec![limiter],
-                            message: self.event,
-                        })
-                    }
-                    throttle::Error::Rate { retry_at } => {
-                        // Save changes to disk
-                        let next_event = std::cmp::min(
-                            retry_at,
-                            message.next_event_after(now()).unwrap_or(u64::MAX),
-                        );
+                message
+                    .save_changes(&server, self.due.into(), next_event.into())
+                    .await;
 
-                        trc::event!(
-                            Delivery(DeliveryEvent::RateLimitExceeded),
-                            Id = throttle.id.clone(),
-                            SpanId = span_id,
-                            NextRetry = trc::Value::Timestamp(next_event)
-                        );
-
-                        message
-                            .save_changes(&server, self.event.due.into(), next_event.into())
-                            .await;
-
-                        QueueEvent::Reload
-                    }
-                };
-
-                if server.inner.ipc.queue_tx.send(event).await.is_err() {
-                    trc::event!(
-                        Server(ServerEvent::ThreadError),
-                        Reason = "Channel closed.",
-                        CausedBy = trc::location!(),
-                        SpanId = span_id
-                    );
-                }
-                return;
+                return QueueEventStatus::Deferred;
             }
         }
 
         let queue_config = &server.core.smtp.queue;
-        let mut on_hold = Vec::new();
         let no_ip = IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0));
         let mut recipients = std::mem::take(&mut message.recipients);
         'next_domain: for domain_idx in 0..message.domains.len() {
@@ -252,10 +218,9 @@ impl DeliveryAttempt {
             let mut envelope = QueueEnvelope::new(&message, domain_idx);
 
             // Throttle recipient domain
-            let mut in_flight = Vec::new();
-            for throttle in &queue_config.throttle.rcpt {
-                if let Err(err) = server
-                    .is_allowed(throttle, &envelope, &mut in_flight, message.span_id)
+            for throttle in &queue_config.outbound_limiters.rcpt {
+                if let Err(retry_at) = server
+                    .is_allowed(throttle, &envelope, message.span_id)
                     .await
                 {
                     trc::event!(
@@ -265,7 +230,7 @@ impl DeliveryAttempt {
                         Domain = domain.domain.clone(),
                     );
 
-                    message.domains[domain_idx].set_throttle_error(err, &mut on_hold);
+                    message.domains[domain_idx].set_rate_limiter_error(retry_at);
                     continue 'next_domain;
                 }
             }
@@ -281,7 +246,7 @@ impl DeliveryAttempt {
                     let delivery_result = message
                         .deliver_local(
                             recipients.iter_mut().filter(|r| r.domain_idx == domain_idx),
-                            &server.inner.ipc.delivery_tx,
+                            &server,
                         )
                         .await;
 
@@ -338,7 +303,10 @@ impl DeliveryAttempt {
                         .smtp
                         .resolvers
                         .dns
-                        .txt_lookup::<TlsRpt>(format!("_smtp._tls.{}.", domain.domain))
+                        .txt_lookup::<TlsRpt>(
+                            format!("_smtp._tls.{}.", domain.domain),
+                            Some(&server.inner.cache.dns_txt),
+                        )
                         .await
                     {
                         Ok(record) => {
@@ -374,7 +342,7 @@ impl DeliveryAttempt {
                                 TlsRpt(TlsRptEvent::RecordFetchError),
                                 SpanId = message.span_id,
                                 Domain = domain.domain.clone(),
-                                CausedBy = trc::Event::from(err),
+                                CausedBy = trc::Error::from(err),
                                 Elapsed = time.elapsed(),
                             );
                             None
@@ -466,7 +434,7 @@ impl DeliveryAttempt {
                                     MtaSts(MtaStsEvent::PolicyFetchError),
                                     SpanId = message.span_id,
                                     Domain = domain.domain.clone(),
-                                    CausedBy = trc::Event::from(err.clone()),
+                                    CausedBy = trc::Error::from(err.clone()),
                                     Strict = strict,
                                     Elapsed = time.elapsed(),
                                 );
@@ -523,16 +491,27 @@ impl DeliveryAttempt {
                     .smtp
                     .resolvers
                     .dns
-                    .mx_lookup(&domain.domain)
+                    .mx_lookup(&domain.domain, Some(&server.inner.cache.dns_mx))
                     .await
                 {
                     Ok(mx) => mx,
+                    Err(mail_auth::Error::DnsRecordNotFound(_)) => {
+                        trc::event!(
+                            Delivery(DeliveryEvent::MxLookupFailed),
+                            SpanId = message.span_id,
+                            Domain = domain.domain.clone(),
+                            Details = "No MX records were found, attempting implicit MX.",
+                            Elapsed = time.elapsed(),
+                        );
+
+                        Arc::new(vec![])
+                    }
                     Err(err) => {
                         trc::event!(
                             Delivery(DeliveryEvent::MxLookupFailed),
                             SpanId = message.span_id,
                             Domain = domain.domain.clone(),
-                            CausedBy = trc::Event::from(err.clone()),
+                            CausedBy = trc::Error::from(err.clone()),
                             Elapsed = time.elapsed(),
                         );
 
@@ -820,7 +799,7 @@ impl DeliveryAttempt {
                                     SpanId = message.span_id,
                                     Domain = domain.domain.clone(),
                                     Hostname = envelope.mx.to_string(),
-                                    CausedBy = trc::Event::from(err.clone()),
+                                    CausedBy = trc::Error::from(err.clone()),
                                     Strict = strict,
                                     Elapsed = time.elapsed(),
                                 );
@@ -875,11 +854,10 @@ impl DeliveryAttempt {
                     envelope.local_ip = source_ip.unwrap_or(no_ip);
 
                     // Throttle remote host
-                    let mut in_flight_host = Vec::new();
                     envelope.remote_ip = remote_ip;
-                    for throttle in &queue_config.throttle.host {
-                        if let Err(err) = server
-                            .is_allowed(throttle, &envelope, &mut in_flight_host, message.span_id)
+                    for throttle in &queue_config.outbound_limiters.remote {
+                        if let Err(retry_at) = server
+                            .is_allowed(throttle, &envelope, message.span_id)
                             .await
                         {
                             trc::event!(
@@ -888,7 +866,7 @@ impl DeliveryAttempt {
                                 Id = throttle.id.clone(),
                                 RemoteIp = remote_ip,
                             );
-                            message.domains[domain_idx].set_throttle_error(err, &mut on_hold);
+                            message.domains[domain_idx].set_rate_limiter_error(retry_at);
                             continue 'next_domain;
                         }
                     }
@@ -989,7 +967,10 @@ impl DeliveryAttempt {
                         || (message.flags & MAIL_REQUIRETLS) != 0
                         || mta_sts_policy.is_some()
                         || dane_policy.is_some();
-                    let tls_connector = if allow_invalid_certs || remote_host.allow_invalid_certs()
+                    // As per RFC7671 Section 5.1, DANE-EE(3) allows name mismatch
+                    let tls_connector = if allow_invalid_certs
+                        || remote_host.allow_invalid_certs()
+                        || dane_policy.as_ref().is_some_and(|t| t.has_end_entities)
                     {
                         &server.inner.data.smtp_connectors.dummy_verify
                     } else {
@@ -1329,22 +1310,7 @@ impl DeliveryAttempt {
         server.send_dsn(&mut message).await;
 
         // Notify queue manager
-        let result = if !on_hold.is_empty() {
-            // Save changes to disk
-            let next_due = message.next_event_after(now());
-            message.save_changes(&server, None, None).await;
-
-            trc::event!(
-                Delivery(DeliveryEvent::ConcurrencyLimitExceeded),
-                SpanId = span_id,
-            );
-
-            QueueEvent::OnHold(OnHold {
-                next_due,
-                limiters: on_hold,
-                message: self.event,
-            })
-        } else if let Some(due) = message.next_event() {
+        if let Some(due) = message.next_event() {
             trc::event!(
                 Queue(trc::QueueEvent::Rescheduled),
                 SpanId = span_id,
@@ -1355,10 +1321,10 @@ impl DeliveryAttempt {
 
             // Save changes to disk
             message
-                .save_changes(&server, self.event.due.into(), due.into())
+                .save_changes(&server, self.due.into(), due.into())
                 .await;
 
-            QueueEvent::Reload
+            QueueEventStatus::Deferred
         } else {
             trc::event!(
                 Delivery(DeliveryEvent::Completed),
@@ -1367,17 +1333,9 @@ impl DeliveryAttempt {
             );
 
             // Delete message from queue
-            message.remove(&server, self.event.due).await;
+            message.remove(&server, self.due).await;
 
-            QueueEvent::Reload
-        };
-        if server.inner.ipc.queue_tx.send(result).await.is_err() {
-            trc::event!(
-                Server(ServerEvent::ThreadError),
-                Reason = "Channel closed.",
-                CausedBy = trc::location!(),
-                SpanId = span_id
-            );
+            QueueEventStatus::Completed
         }
     }
 }
