@@ -33,7 +33,7 @@ use crate::{
     reporting::SmtpReporting,
 };
 
-use super::{AggregateTimestamp, ReportLock, SerializedSize};
+use super::{AggregateTimestamp, SerializedSize};
 
 #[derive(Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
 pub struct DmarcFormat {
@@ -70,7 +70,11 @@ impl<T: SessionStream> Session<T> {
                 .smtp
                 .resolvers
                 .dns
-                .verify_dmarc_report_address(dmarc_output.domain(), dmarc_record.ruf())
+                .verify_dmarc_report_address(
+                    dmarc_output.domain(),
+                    dmarc_record.ruf(),
+                    Some(&self.server.inner.cache.dns_txt),
+                )
                 .await
             {
                 Some(rcpts) => {
@@ -364,7 +368,7 @@ impl DmarcReporting for Server {
             .smtp
             .resolvers
             .dns
-            .verify_dmarc_report_address(&event.domain, &rua)
+            .verify_dmarc_report_address(&event.domain, &rua, Some(&self.inner.cache.dns_txt))
             .await
         {
             Some(rcpts) => {
@@ -547,7 +551,7 @@ impl DmarcReporting for Server {
                     Entry::Vacant(e) => {
                         if serialized_size
                             .as_deref_mut()
-                            .map_or(true, |serialized_size| {
+                            .is_none_or( |serialized_size| {
                                 serde::Serialize::serialize(e.key(), serialized_size).is_ok()
                             })
                         {
@@ -646,12 +650,6 @@ impl DmarcReporting for Server {
                 ValueClass::Queue(QueueClass::DmarcReportHeader(report_event.clone())),
                 Bincode::new(entry).serialize(),
             );
-
-            // Add lock
-            builder.set(
-                ValueClass::Queue(QueueClass::dmarc_lock(&report_event)),
-                0u64.serialize(),
-            );
         }
 
         // Write entry
