@@ -8,6 +8,7 @@ use std::{borrow::Cow, future::Future, sync::Arc, time::Instant};
 
 use common::{scripts::plugins::PluginContext, Server};
 use mail_auth::common::headers::HeaderWriter;
+use mail_parser::{Encoding, Message, MessagePart, PartType};
 use sieve::{
     compiler::grammar::actions::action_redirect::{ByMode, ByTime, Notify, NotifyItem, Ret},
     Event, Input, MatchAs, Recipient, Sieve,
@@ -47,7 +48,19 @@ impl RunScript for Server {
             .core
             .sieve
             .trusted_runtime
-            .filter(params.message.unwrap_or_default())
+            .filter_parsed(params.message.unwrap_or_else(|| Message {
+                parts: vec![MessagePart {
+                    headers: vec![],
+                    is_encoding_problem: false,
+                    body: PartType::Text("".into()),
+                    encoding: Encoding::None,
+                    offset_header: 0,
+                    offset_body: 0,
+                    offset_end: 0,
+                }],
+                raw_message: b""[..].into(),
+                ..Default::default()
+            }))
             .with_vars_env(params.variables)
             .with_envelope_list(params.envelope)
             .with_user_address(&params.from_addr)
@@ -65,7 +78,8 @@ impl RunScript for Server {
             match result {
                 Ok(event) => match event {
                     Event::IncludeScript { name, optional } => {
-                        if let Some(script) = self.core.sieve.trusted_scripts.get(name.as_str()) {
+                        let name_ = name.as_str().to_lowercase();
+                        if let Some(script) = self.core.sieve.trusted_scripts.get(&name_) {
                             input = Input::script(name, script.clone());
                         } else if optional {
                             input = false.into();
@@ -74,7 +88,7 @@ impl RunScript for Server {
                                 Sieve(SieveEvent::ScriptNotFound),
                                 Id = script_id.clone(),
                                 SpanId = session_id,
-                                Details = name.as_str().to_string(),
+                                Details = name_,
                             );
                             break;
                         }
@@ -89,14 +103,11 @@ impl RunScript for Server {
                             if let Some(store) = self.core.storage.lookups.get(&list) {
                                 for value in &values {
                                     if let Ok(true) = store
-                                        .key_exists(
-                                            if !matches!(match_as, MatchAs::Lowercase) {
-                                                value.clone()
-                                            } else {
-                                                value.to_lowercase()
-                                            }
-                                            .into_bytes(),
-                                        )
+                                        .key_exists(if !matches!(match_as, MatchAs::Lowercase) {
+                                            value.clone()
+                                        } else {
+                                            value.to_lowercase()
+                                        })
                                         .await
                                     {
                                         input = true.into();
@@ -278,7 +289,7 @@ impl RunScript for Server {
                                                 signature.write_header(&mut headers);
                                             }
                                             Err(err) => {
-                                                trc::error!(trc::Event::from(err)
+                                                trc::error!(trc::Error::from(err)
                                                     .span_id(session_id)
                                                     .caused_by(trc::location!())
                                                     .details("DKIM sign failed"));
@@ -359,33 +370,6 @@ impl RunScript for Server {
             }
         }
 
-        // Assert global variables
-        #[cfg(feature = "test_mode")]
-        if let Some(expected_variables) = params.expected_variables {
-            for var_name in instance.global_variable_names() {
-                if instance.global_variable(var_name).unwrap().to_bool()
-                    && !expected_variables.contains_key(var_name)
-                {
-                    panic!(
-                        "Unexpected variable {var_name:?} with value {:?}\nExpected {:?}\nFound: {:?}",
-                        instance.global_variable(var_name).unwrap(),
-                        expected_variables.keys().collect::<Vec<_>>(),
-                        instance.global_variable_names().collect::<Vec<_>>()
-                    );
-                }
-            }
-
-            for (name, expected) in &expected_variables {
-                if let Some(value) = instance.global_variable(name.as_str()) {
-                    assert_eq!(value, expected, "Variable {name:?} has unexpected value");
-                } else {
-                    panic!("Missing variable {name:?} with value {expected:?}\nExpected {:?}\nFound: {:?}", 
-                    expected_variables.keys().collect::<Vec<_>>(),
-                    instance.global_variable_names().collect::<Vec<_>>());
-                }
-            }
-        }
-
         // Keep id
         // 0 = use original message
         // MAX = implicit keep
