@@ -8,6 +8,7 @@
  *
  */
 
-pub mod distributed_blob;
 #[cfg(any(feature = "postgres", feature = "mysql"))]
 pub mod read_replica;
+pub mod sharded_blob;
+pub mod sharded_lookup;
