@@ -13,13 +13,15 @@ use roaring::RoaringBitmap;
 use trc::{AddContext, StoreEvent};
 
 use crate::{
+    BitmapKey, Deserialize, IterateParams, Key, QueryResult, SUBSPACE_BITMAP_ID,
+    SUBSPACE_BITMAP_TAG, SUBSPACE_BITMAP_TEXT, SUBSPACE_INDEXES, SUBSPACE_LOGS, Store, U32_LEN,
+    Value, ValueKey,
     write::{
+        AnyClass, AnyKey, AssignedIds, Batch, BatchBuilder, BitmapClass, BitmapHash, Operation,
+        ReportClass, ValueClass, ValueOp,
         key::{DeserializeBigEndian, KeySerializer},
-        now, AnyClass, AnyKey, AssignedIds, Batch, BatchBuilder, BitmapClass, BitmapHash,
-        Operation, ReportClass, ValueClass, ValueOp,
+        now,
     },
-    BitmapKey, Deserialize, IterateParams, Key, Store, ValueKey, SUBSPACE_BITMAP_ID,
-    SUBSPACE_BITMAP_TAG, SUBSPACE_BITMAP_TEXT, SUBSPACE_INDEXES, SUBSPACE_LOGS, U32_LEN,
 };
 
 use super::DocumentSet;
@@ -154,9 +156,37 @@ impl Store {
         .caused_by(trc::location!())
     }
 
-    pub async fn write(&self, batch: Batch) -> trc::Result<AssignedIds> {
+    #[allow(unreachable_patterns)]
+    #[allow(unused_variables)]
+    pub async fn sql_query<T: QueryResult + std::fmt::Debug>(
+        &self,
+        query: &str,
+        params: Vec<Value<'_>>,
+    ) -> trc::Result<T> {
+        let result = match self {
+            #[cfg(feature = "sqlite")]
+            Self::SQLite(store) => store.query(query, &params).await,
+            #[cfg(feature = "postgres")]
+            Self::PostgreSQL(store) => store.query(query, &params).await,
+            #[cfg(feature = "mysql")]
+            Self::MySQL(store) => store.query(query, &params).await,
+            _ => Err(trc::StoreEvent::NotSupported.into_err()),
+        };
+
+        trc::event!(
+            Store(trc::StoreEvent::SqlQuery),
+            Details = query.to_string(),
+            Value = params.as_slice(),
+            Result = &result,
+        );
+
+        result.caused_by(trc::location!())
+    }
+
+    pub async fn write(&self, batch: impl Into<Batch>) -> trc::Result<AssignedIds> {
+        let batch = batch.into();
         #[cfg(feature = "test_mode")]
-        if std::env::var("PARANOID_WRITE").map_or(false, |v| v == "1") {
+        if std::env::var("PARANOID_WRITE").is_ok_and(|v| v == "1") {
             let mut account_id = u32::MAX;
             let mut collection = u8::MAX;
             let mut document_id = u32::MAX;
@@ -273,6 +303,13 @@ impl Store {
         result
     }
 
+    #[inline]
+    pub async fn write_expect_id(&self, batch: impl Into<Batch>) -> trc::Result<u32> {
+        self.write(batch)
+            .await
+            .and_then(|ids| ids.last_document_id())
+    }
+
     pub async fn purge_store(&self) -> trc::Result<()> {
         // Delete expired reports
         let now = now();
@@ -381,7 +418,7 @@ impl Store {
             )
             .no_values(),
             |key, _| {
-                if collection_offset.map_or(true, |offset| {
+                if collection_offset.is_none_or(|offset| {
                     key.get(key.len() - U32_LEN - offset).copied() == Some(collection)
                 }) {
                     let document_id = key.deserialize_be_u32(key.len() - U32_LEN)?;
@@ -562,12 +599,12 @@ impl Store {
             SUBSPACE_BITMAP_TAG,
             SUBSPACE_BITMAP_TEXT,
             SUBSPACE_DIRECTORY,
-            SUBSPACE_FTS_QUEUE,
+            SUBSPACE_TASK_QUEUE,
             SUBSPACE_INDEXES,
             SUBSPACE_BLOB_RESERVE,
             SUBSPACE_BLOB_LINK,
             SUBSPACE_LOGS,
-            SUBSPACE_LOOKUP_VALUE,
+            SUBSPACE_IN_MEMORY_VALUE,
             SUBSPACE_COUNTER,
             SUBSPACE_PROPERTY,
             SUBSPACE_SETTINGS,
@@ -609,9 +646,9 @@ impl Store {
 
     #[cfg(feature = "test_mode")]
     pub async fn blob_expire_all(&self) {
-        use utils::{BlobHash, BLOB_HASH_LEN};
+        use utils::{BLOB_HASH_LEN, BlobHash};
 
-        use crate::{write::BlobOp, U64_LEN};
+        use crate::{U64_LEN, write::BlobOp};
 
         // Delete all temporary hashes
         let from_key = ValueKey {
@@ -666,11 +703,11 @@ impl Store {
 
     #[cfg(feature = "test_mode")]
     pub async fn lookup_expire_all(&self) {
-        use crate::write::LookupClass;
+        use crate::write::InMemoryClass;
 
         // Delete all temporary counters
-        let from_key = ValueKey::from(ValueClass::Lookup(LookupClass::Key(vec![0u8])));
-        let to_key = ValueKey::from(ValueClass::Lookup(LookupClass::Key(vec![u8::MAX; 10])));
+        let from_key = ValueKey::from(ValueClass::InMemory(InMemoryClass::Key(vec![0u8])));
+        let to_key = ValueKey::from(ValueClass::InMemory(InMemoryClass::Key(vec![u8::MAX; 10])));
 
         let mut expired_keys = Vec::new();
         let mut expired_counters = Vec::new();
@@ -691,7 +728,7 @@ impl Store {
             let mut batch = BatchBuilder::new();
             for key in expired_keys {
                 batch.ops.push(Operation::Value {
-                    class: ValueClass::Lookup(LookupClass::Key(key)),
+                    class: ValueClass::InMemory(InMemoryClass::Key(key)),
                     op: ValueOp::Clear,
                 });
                 if batch.ops.len() >= 1000 {
@@ -708,11 +745,11 @@ impl Store {
             let mut batch = BatchBuilder::new();
             for key in expired_counters {
                 batch.ops.push(Operation::Value {
-                    class: ValueClass::Lookup(LookupClass::Counter(key.clone())),
+                    class: ValueClass::InMemory(InMemoryClass::Counter(key.clone())),
                     op: ValueOp::Clear,
                 });
                 batch.ops.push(Operation::Value {
-                    class: ValueClass::Lookup(LookupClass::Key(key)),
+                    class: ValueClass::InMemory(InMemoryClass::Key(key)),
                     op: ValueOp::Clear,
                 });
                 if batch.ops.len() >= 1000 {
@@ -744,8 +781,9 @@ impl Store {
         for (subspace, with_values) in [
             (SUBSPACE_ACL, true),
             //(SUBSPACE_DIRECTORY, true),
-            (SUBSPACE_FTS_QUEUE, true),
-            (SUBSPACE_LOOKUP_VALUE, true),
+            (SUBSPACE_TASK_QUEUE, true),
+            (SUBSPACE_IN_MEMORY_VALUE, true),
+            (SUBSPACE_IN_MEMORY_COUNTER, false),
             (SUBSPACE_PROPERTY, true),
             (SUBSPACE_SETTINGS, true),
             (SUBSPACE_QUEUE_MESSAGE, true),
@@ -851,10 +889,12 @@ impl Store {
                         }
                         _ => {
                             println!(
-                                "Found key in {:?}: {:?} {:?}",
+                                "Found key in {:?}: {:?} ({:?}) = {:?} ({:?})",
                                 char::from(subspace),
                                 key,
-                                value
+                                String::from_utf8_lossy(key),
+                                value,
+                                String::from_utf8_lossy(value)
                             );
                         }
                     }
@@ -894,3 +934,9 @@ impl Store {
         }
     }
 }
+
+impl From<BatchBuilder> for Batch {
+    fn from(builder: BatchBuilder) -> Self {
+        builder.build()
+    }
+}
