@@ -15,12 +15,14 @@ pub mod write;
 
 pub use ahash;
 use ahash::AHashMap;
-use backend::{fs::FsStore, memory::MemoryStore};
+use backend::{fs::FsStore, http::HttpStore, memory::StaticMemoryStore};
 pub use blake3;
 pub use parking_lot;
 pub use rand;
 pub use roaring;
-use write::{purge::PurgeSchedule, BitmapClass, ValueClass};
+use utils::config::cron::SimpleCron;
+use write::{BitmapClass, ValueClass};
+pub use xxhash_rust;
 
 #[cfg(feature = "s3")]
 use backend::s3::S3Store;
@@ -134,14 +136,15 @@ pub const SUBSPACE_BITMAP_ID: u8 = b'b';
 pub const SUBSPACE_BITMAP_TAG: u8 = b'c';
 pub const SUBSPACE_BITMAP_TEXT: u8 = b'v';
 pub const SUBSPACE_DIRECTORY: u8 = b'd';
-pub const SUBSPACE_FTS_QUEUE: u8 = b'f';
+pub const SUBSPACE_TASK_QUEUE: u8 = b'f';
 pub const SUBSPACE_INDEXES: u8 = b'i';
 pub const SUBSPACE_BLOB_RESERVE: u8 = b'j';
 pub const SUBSPACE_BLOB_LINK: u8 = b'k';
 pub const SUBSPACE_BLOBS: u8 = b't';
 pub const SUBSPACE_LOGS: u8 = b'l';
 pub const SUBSPACE_COUNTER: u8 = b'n';
-pub const SUBSPACE_LOOKUP_VALUE: u8 = b'm';
+pub const SUBSPACE_IN_MEMORY_VALUE: u8 = b'm';
+pub const SUBSPACE_IN_MEMORY_COUNTER: u8 = b'y';
 pub const SUBSPACE_PROPERTY: u8 = b'p';
 pub const SUBSPACE_SETTINGS: u8 = b's';
 pub const SUBSPACE_QUEUE_MESSAGE: u8 = b'e';
@@ -154,7 +157,6 @@ pub const SUBSPACE_TELEMETRY_SPAN: u8 = b'o';
 pub const SUBSPACE_TELEMETRY_INDEX: u8 = b'w';
 pub const SUBSPACE_TELEMETRY_METRIC: u8 = b'x';
 
-pub const SUBSPACE_RESERVED_1: u8 = b'y';
 pub const SUBSPACE_RESERVED_2: u8 = b'z';
 
 #[derive(Clone)]
@@ -171,7 +173,7 @@ pub struct Stores {
     pub stores: AHashMap<String, Store>,
     pub blob_stores: AHashMap<String, BlobStore>,
     pub fts_stores: AHashMap<String, FtsStore>,
-    pub lookup_stores: AHashMap<String, LookupStore>,
+    pub in_memory_stores: AHashMap<String, InMemoryStore>,
     pub purge_schedules: Vec<PurgeSchedule>,
 }
 
@@ -214,7 +216,7 @@ pub enum BlobBackend {
     #[cfg(feature = "azure")]
     Azure(Arc<AzureStore>),
     #[cfg(feature = "enterprise")]
-    Composite(Arc<backend::composite::distributed_blob::DistributedBlob>),
+    Sharded(Arc<backend::composite::sharded_blob::ShardedBlob>),
 }
 
 #[derive(Clone)]
@@ -225,18 +227,14 @@ pub enum FtsStore {
 }
 
 #[derive(Clone, Debug)]
-pub enum LookupStore {
+pub enum InMemoryStore {
     Store(Store),
-    Query(Arc<QueryStore>),
     #[cfg(feature = "redis")]
     Redis(Arc<RedisStore>),
-    Memory(Arc<MemoryStore>),
-}
-
-#[derive(Debug)]
-pub struct QueryStore {
-    pub store: LookupStore,
-    pub query: String,
+    Http(Arc<HttpStore>),
+    Static(Arc<StaticMemoryStore>),
+    #[cfg(feature = "enterprise")]
+    Sharded(Arc<backend::composite::sharded_lookup::ShardedInMemory>),
 }
 
 #[cfg(feature = "sqlite")]
@@ -311,7 +309,7 @@ impl From<ElasticSearchStore> for FtsStore {
 }
 
 #[cfg(feature = "redis")]
-impl From<RedisStore> for LookupStore {
+impl From<RedisStore> for InMemoryStore {
     fn from(store: RedisStore) -> Self {
         Self::Redis(Arc::new(store))
     }
@@ -332,7 +330,7 @@ impl From<Store> for BlobStore {
     }
 }
 
-impl From<Store> for LookupStore {
+impl From<Store> for InMemoryStore {
     fn from(store: Store) -> Self {
         Self::Store(store)
     }
@@ -347,7 +345,7 @@ impl Default for BlobStore {
     }
 }
 
-impl Default for LookupStore {
+impl Default for InMemoryStore {
     fn default() -> Self {
         Self::Store(Store::None)
     }
@@ -359,6 +357,20 @@ impl Default for FtsStore {
     }
 }
 
+#[derive(Clone)]
+pub enum PurgeStore {
+    Data(Store),
+    Blobs { store: Store, blob_store: BlobStore },
+    Lookup(InMemoryStore),
+}
+
+#[derive(Clone)]
+pub struct PurgeSchedule {
+    pub cron: SimpleCron,
+    pub store_id: String,
+    pub store: PurgeStore,
+}
+
 #[derive(Clone, Debug, PartialEq)]
 pub enum Value<'x> {
     Integer(i64),
@@ -757,7 +769,7 @@ impl Stores {
             self.stores
                 .retain(|_, store| !matches!(store, Store::SQLReadReplica(_)));
             self.blob_stores
-                .retain(|_, store| !matches!(store.backend, BlobBackend::Composite(_)));
+                .retain(|_, store| !matches!(store.backend, BlobBackend::Sharded(_)));
         }
     }
 }
