@@ -17,11 +17,11 @@ use nlp::tokenizers::word::WordTokenizer;
 use rand::Rng;
 use roaring::RoaringBitmap;
 use utils::{
-    codec::leb128::{Leb128Iterator, Leb128Vec},
     BlobHash,
+    codec::leb128::{Leb128Iterator, Leb128Vec},
 };
 
-use crate::{backend::MAX_TOKEN_LENGTH, BlobClass, Deserialize, Serialize, Value};
+use crate::{BlobClass, Deserialize, Serialize, Value, backend::MAX_TOKEN_LENGTH};
 
 use self::assert::AssertValue;
 
@@ -31,7 +31,6 @@ pub mod blob;
 pub mod hash;
 pub mod key;
 pub mod log;
-pub mod purge;
 
 pub trait SerializeWithId: Send + Sync {
     fn serialize_with_id(&self, ids: &AssignedIds) -> trc::Result<Vec<u8>>;
@@ -145,9 +144,9 @@ pub enum TagValue<T> {
 pub enum ValueClass<T> {
     Property(u8),
     Acl(u32),
-    Lookup(LookupClass),
+    InMemory(InMemoryClass),
     FtsIndex(BitmapHash),
-    FtsQueue(FtsQueueClass),
+    TaskQueue(TaskQueueClass),
     Directory(DirectoryClass<T>),
     Blob(BlobOp),
     Config(Vec<u8>),
@@ -158,9 +157,16 @@ pub enum ValueClass<T> {
 }
 
 #[derive(Debug, PartialEq, Clone, Eq, Hash)]
-pub struct FtsQueueClass {
-    pub seq: u64,
-    pub hash: BlobHash,
+pub enum TaskQueueClass {
+    IndexEmail {
+        seq: u64,
+        hash: BlobHash,
+    },
+    BayesTrain {
+        seq: u64,
+        hash: BlobHash,
+        learn_spam: bool,
+    },
 }
 
 #[derive(Debug, PartialEq, Clone, Eq, Hash)]
@@ -170,7 +176,7 @@ pub struct AnyClass {
 }
 
 #[derive(Debug, PartialEq, Clone, Eq, Hash)]
-pub enum LookupClass {
+pub enum InMemoryClass {
     Key(Vec<u8>),
     Counter(Vec<u8>),
 }
@@ -890,6 +896,16 @@ impl RandomAvailableId for RoaringBitmap {
             last_id += 1;
         }
 
-        available_ids[rand::thread_rng().gen_range(0..available_ids.len())]
+        available_ids[rand::rng().random_range(0..available_ids.len())]
+    }
+}
+
+impl QueueClass {
+    pub fn due(&self) -> Option<u64> {
+        match self {
+            QueueClass::DmarcReportHeader(report_event) => report_event.due.into(),
+            QueueClass::TlsReportHeader(report_event) => report_event.due.into(),
+            _ => None,
+        }
     }
 }
