@@ -39,7 +39,7 @@ impl EventType {
             EventType::PushSubscription(event) => event.description(),
             EventType::Cluster(event) => event.description(),
             EventType::Housekeeper(event) => event.description(),
-            EventType::FtsIndex(event) => event.description(),
+            EventType::TaskQueue(event) => event.description(),
             EventType::Milter(event) => event.description(),
             EventType::MtaHook(event) => event.description(),
             EventType::Delivery(event) => event.description(),
@@ -87,7 +87,7 @@ impl EventType {
             EventType::PushSubscription(event) => event.explain(),
             EventType::Cluster(event) => event.explain(),
             EventType::Housekeeper(event) => event.explain(),
-            EventType::FtsIndex(event) => event.explain(),
+            EventType::TaskQueue(event) => event.explain(),
             EventType::Milter(event) => event.explain(),
             EventType::MtaHook(event) => event.explain(),
             EventType::Delivery(event) => event.explain(),
@@ -174,9 +174,7 @@ impl HousekeeperEvent {
             HousekeeperEvent::Start => "Housekeeper process started",
             HousekeeperEvent::Stop => "Housekeeper process stopped",
             HousekeeperEvent::Schedule => "Housekeeper task scheduled",
-            HousekeeperEvent::PurgeAccounts => "Purging accounts",
-            HousekeeperEvent::PurgeSessions => "Purging sessions",
-            HousekeeperEvent::PurgeStore => "Purging store",
+            HousekeeperEvent::Run => "Housekeeper task run",
         }
     }
 
@@ -185,31 +183,29 @@ impl HousekeeperEvent {
             HousekeeperEvent::Start => "The housekeeper process has started",
             HousekeeperEvent::Stop => "The housekeeper process has stopped",
             HousekeeperEvent::Schedule => "A housekeeper task has been scheduled",
-            HousekeeperEvent::PurgeAccounts => "Purging accounts",
-            HousekeeperEvent::PurgeSessions => "Purging sessions",
-            HousekeeperEvent::PurgeStore => "Purging store",
+            HousekeeperEvent::Run => "A housekeeper task is running",
         }
     }
 }
 
-impl FtsIndexEvent {
+impl TaskQueueEvent {
     pub fn description(&self) -> &'static str {
         match self {
-            FtsIndexEvent::Index => "Full-text search index done",
-            FtsIndexEvent::Locked => "Full-text search index is locked",
-            FtsIndexEvent::LockBusy => "Full-text search index lock is busy",
-            FtsIndexEvent::BlobNotFound => "Blob not found for full-text indexing",
-            FtsIndexEvent::MetadataNotFound => "Metadata not found for full-text indexing",
+            TaskQueueEvent::Index => "Full-text search indexing completed",
+            TaskQueueEvent::Locked => "Task is locked by another process",
+            TaskQueueEvent::BlobNotFound => "Blob not found for task",
+            TaskQueueEvent::MetadataNotFound => "Metadata not found for task",
+            TaskQueueEvent::BayesTrain => "Bayesian training completed",
         }
     }
 
     pub fn explain(&self) -> &'static str {
         match self {
-            FtsIndexEvent::Index => "The full-text search index has been updated",
-            FtsIndexEvent::Locked => "The full-text search index is locked",
-            FtsIndexEvent::LockBusy => "The full-text search index lock is busy",
-            FtsIndexEvent::BlobNotFound => "The blob was not found for full-text indexing",
-            FtsIndexEvent::MetadataNotFound => "The metadata was not found for full-text indexing",
+            TaskQueueEvent::Index => "The full-text search index has been updated",
+            TaskQueueEvent::Locked => "The task id is locked by another process",
+            TaskQueueEvent::BlobNotFound => "The requested blob was not found for task",
+            TaskQueueEvent::MetadataNotFound => "The metadata was not found for task",
+            TaskQueueEvent::BayesTrain => "Bayesian training has been completed",
         }
     }
 }
@@ -253,6 +249,7 @@ impl ImapEvent {
             ImapEvent::RawOutput => "Raw IMAP output sent",
             ImapEvent::ConnectionStart => "IMAP connection started",
             ImapEvent::ConnectionEnd => "IMAP connection ended",
+            ImapEvent::GetQuota => "IMAP GETQUOTA command",
         }
     }
 
@@ -294,6 +291,7 @@ impl ImapEvent {
             ImapEvent::RawOutput => "Raw IMAP output sent",
             ImapEvent::ConnectionStart => "IMAP connection started",
             ImapEvent::ConnectionEnd => "IMAP connection ended",
+            ImapEvent::GetQuota => "Client requested mailbox quota",
         }
     }
 }
@@ -409,8 +407,6 @@ impl SmtpEvent {
             SmtpEvent::MessageParseFailed => "Message parsing failed",
             SmtpEvent::MessageTooLarge => "Message too large",
             SmtpEvent::LoopDetected => "Mail loop detected",
-            SmtpEvent::PipeSuccess => "Pipe command succeeded",
-            SmtpEvent::PipeError => "Pipe command failed",
             SmtpEvent::DkimPass => "DKIM verification passed",
             SmtpEvent::DkimFail => "DKIM verification failed",
             SmtpEvent::ArcPass => "ARC verification passed",
@@ -442,6 +438,7 @@ impl SmtpEvent {
             SmtpEvent::RcptToDuplicate => "Duplicate RCPT TO",
             SmtpEvent::RcptToRewritten => "RCPT TO address rewritten",
             SmtpEvent::RcptToMissing => "RCPT TO address missing",
+            SmtpEvent::RcptToGreylisted => "RCPT TO greylisted",
             SmtpEvent::TooManyRecipients => "Too many recipients",
             SmtpEvent::TooManyInvalidRcpt => "Too many invalid recipients",
             SmtpEvent::RawInput => "Raw SMTP input received",
@@ -503,8 +500,6 @@ impl SmtpEvent {
             SmtpEvent::LoopDetected => {
                 "A mail loop was detected, the message contains too many Received headers"
             }
-            SmtpEvent::PipeSuccess => "The pipe command succeeded",
-            SmtpEvent::PipeError => "The pipe command failed",
             SmtpEvent::DkimPass => "Successful DKIM verification",
             SmtpEvent::DkimFail => "Failed to verify DKIM signature",
             SmtpEvent::ArcPass => "Successful ARC verification",
@@ -552,6 +547,7 @@ impl SmtpEvent {
             }
             SmtpEvent::RcptToRewritten => "The envelope recipient address was rewritten",
             SmtpEvent::RcptToMissing => "The remote client issued a DATA command before RCPT TO",
+            SmtpEvent::RcptToGreylisted => "The recipient was greylisted",
             SmtpEvent::TooManyRecipients => {
                 "The remote client exceeded the number of recipients allowed"
             }
@@ -716,8 +712,7 @@ impl QueueEvent {
     pub fn description(&self) -> &'static str {
         match self {
             QueueEvent::Rescheduled => "Message rescheduled for delivery",
-            QueueEvent::LockBusy => "Queue lock is busy",
-            QueueEvent::Locked => "Queue is locked",
+            QueueEvent::Locked => "Queue event is locked by another process",
             QueueEvent::BlobNotFound => "Message blob not found",
             QueueEvent::RateLimitExceeded => "Rate limit exceeded",
             QueueEvent::ConcurrencyLimitExceeded => "Concurrency limit exceeded",
@@ -727,14 +722,14 @@ impl QueueEvent {
             QueueEvent::QueueReport => "Queued report for delivery",
             QueueEvent::QueueDsn => "Queued DSN for delivery",
             QueueEvent::QueueAutogenerated => "Queued autogenerated message for delivery",
+            QueueEvent::BackPressure => "Queue backpressure detected",
         }
     }
 
     pub fn explain(&self) -> &'static str {
         match self {
             QueueEvent::Rescheduled => "The message was rescheduled for delivery",
-            QueueEvent::LockBusy => "The queue lock is busy",
-            QueueEvent::Locked => "The queue is locked",
+            QueueEvent::Locked => "The queue event is locked by another process",
             QueueEvent::BlobNotFound => "The message blob was not found",
             QueueEvent::RateLimitExceeded => "The queue rate limit was exceeded",
             QueueEvent::ConcurrencyLimitExceeded => "The queue concurrency limit was exceeded",
@@ -746,6 +741,9 @@ impl QueueEvent {
             QueueEvent::QueueReport => "A new report was queued for delivery",
             QueueEvent::QueueDsn => "A delivery status notification was queued for delivery",
             QueueEvent::QueueAutogenerated => "A system generated message was queued for delivery",
+            QueueEvent::BackPressure => {
+                "Queue congested, processing can't keep up with incoming message rate"
+            }
         }
     }
 }
@@ -809,8 +807,8 @@ impl OutgoingReportEvent {
             OutgoingReportEvent::DkimRateLimited => "DKIM report rate limited",
             OutgoingReportEvent::DmarcReport => "DMARC report sent",
             OutgoingReportEvent::DmarcRateLimited => "DMARC report rate limited",
-            OutgoingReportEvent::DmarcAggregateReport => "DMARC aggregate report sent",
-            OutgoingReportEvent::TlsAggregate => "TLS aggregate report sent",
+            OutgoingReportEvent::DmarcAggregateReport => "DMARC aggregate is being prepared",
+            OutgoingReportEvent::TlsAggregate => "TLS aggregate report is being prepared",
             OutgoingReportEvent::HttpSubmission => "Report submitted via HTTP",
             OutgoingReportEvent::UnauthorizedReportingAddress => "Unauthorized reporting address",
             OutgoingReportEvent::ReportingAddressValidationError => {
@@ -819,9 +817,7 @@ impl OutgoingReportEvent {
             OutgoingReportEvent::NotFound => "Report not found",
             OutgoingReportEvent::SubmissionError => "Error submitting report",
             OutgoingReportEvent::NoRecipientsFound => "No recipients found for report",
-            OutgoingReportEvent::LockBusy => "Report lock is busy",
-            OutgoingReportEvent::LockDeleted => "Report lock was deleted",
-            OutgoingReportEvent::Locked => "Report is locked",
+            OutgoingReportEvent::Locked => "Report is locked by another process",
         }
     }
 
@@ -833,8 +829,8 @@ impl OutgoingReportEvent {
             OutgoingReportEvent::DkimRateLimited => "The DKIM report was rate limited",
             OutgoingReportEvent::DmarcReport => "A DMARC report has been sent",
             OutgoingReportEvent::DmarcRateLimited => "The DMARC report was rate limited",
-            OutgoingReportEvent::DmarcAggregateReport => "A DMARC aggregate report has been sent",
-            OutgoingReportEvent::TlsAggregate => "A TLS aggregate report has been sent",
+            OutgoingReportEvent::DmarcAggregateReport => "A DMARC aggregate report will be sent",
+            OutgoingReportEvent::TlsAggregate => "A TLS aggregate report will be sent",
             OutgoingReportEvent::HttpSubmission => "The report was submitted via HTTP",
             OutgoingReportEvent::UnauthorizedReportingAddress => {
                 "The reporting address is not authorized to send reports"
@@ -845,9 +841,7 @@ impl OutgoingReportEvent {
             OutgoingReportEvent::NotFound => "The report was not found",
             OutgoingReportEvent::SubmissionError => "Error submitting the report",
             OutgoingReportEvent::NoRecipientsFound => "No recipients found for the report",
-            OutgoingReportEvent::LockBusy => "The report lock is busy",
-            OutgoingReportEvent::LockDeleted => "The report lock was deleted",
-            OutgoingReportEvent::Locked => "The report is locked",
+            OutgoingReportEvent::Locked => "The report is locked by another process",
         }
     }
 }
@@ -1017,29 +1011,29 @@ impl PushSubscriptionEvent {
 impl SpamEvent {
     pub fn description(&self) -> &'static str {
         match self {
+            SpamEvent::Pyzor => "Pyzor success",
             SpamEvent::PyzorError => "Pyzor error",
-            SpamEvent::ListUpdated => "Spam list updated",
             SpamEvent::Train => "Training spam filter",
-            SpamEvent::TrainBalance => "Balancing spam filter training data",
+            SpamEvent::TrainBalance => "Spam filter model balance verify",
             SpamEvent::TrainError => "Error training spam filter",
             SpamEvent::Classify => "Classifying message for spam",
-            SpamEvent::ClassifyError => "Error classifying message for spam",
-            SpamEvent::NotEnoughTrainingData => "Not enough training data for spam filter",
+            SpamEvent::ClassifyError => "Not enough training data for spam filter",
+            SpamEvent::Dnsbl => "DNSBL query",
+            SpamEvent::DnsblError => "Error querying DNSBL",
         }
     }
 
     pub fn explain(&self) -> &'static str {
         match self {
             SpamEvent::PyzorError => "An error occurred with Pyzor",
-            SpamEvent::ListUpdated => "The spam list has been updated",
             SpamEvent::Train => "The spam filter is being trained with the message",
-            SpamEvent::TrainBalance => "The spam filter training data is being balanced",
+            SpamEvent::TrainBalance => "The spam filter training data is verified for balance",
             SpamEvent::TrainError => "An error occurred while training the spam filter",
             SpamEvent::Classify => "The message is being classified for spam",
-            SpamEvent::ClassifyError => "An error occurred while classifying the message for spam",
-            SpamEvent::NotEnoughTrainingData => {
-                "There is not enough training data for the spam filter"
-            }
+            SpamEvent::ClassifyError => "There is not enough training data for the spam filter",
+            SpamEvent::Pyzor => "Pyzor query successful",
+            SpamEvent::Dnsbl => "The DNSBL query was successful",
+            SpamEvent::DnsblError => "An error occurred while querying the DNSBL",
         }
     }
 }
@@ -1276,7 +1270,7 @@ impl PurgeEvent {
             PurgeEvent::Finished => "Purge finished",
             PurgeEvent::Running => "Purge running",
             PurgeEvent::Error => "Purge error",
-            PurgeEvent::PurgeActive => "Active purge in progress",
+            PurgeEvent::InProgress => "Active purge in progress",
             PurgeEvent::AutoExpunge => "Auto-expunge executed",
             PurgeEvent::TombstoneCleanup => "Tombstone cleanup executed",
         }
@@ -1288,7 +1282,7 @@ impl PurgeEvent {
             PurgeEvent::Finished => "The purge has finished",
             PurgeEvent::Running => "The purge is running",
             PurgeEvent::Error => "An error occurred with the purge",
-            PurgeEvent::PurgeActive => "An active purge is in progress",
+            PurgeEvent::InProgress => "An active purge is in progress",
             PurgeEvent::AutoExpunge => "Auto-expunge has been executed",
             PurgeEvent::TombstoneCleanup => "Tombstone cleanup has been executed",
         }
@@ -1331,7 +1325,6 @@ impl ConfigEvent {
             ConfigEvent::ParseWarning => "Configuration parse warning",
             ConfigEvent::BuildWarning => "Configuration build warning",
             ConfigEvent::ImportExternal => "Importing external configuration",
-            ConfigEvent::ExternalKeyIgnored => "External configuration key ignored",
             ConfigEvent::AlreadyUpToDate => "Configuration already up to date",
         }
     }
@@ -1349,7 +1342,6 @@ impl ConfigEvent {
             ConfigEvent::ParseWarning => "A warning occurred while parsing the configuration",
             ConfigEvent::BuildWarning => "A warning occurred while building the configuration",
             ConfigEvent::ImportExternal => "An external configuration is being imported",
-            ConfigEvent::ExternalKeyIgnored => "An external configuration key is ignored",
             ConfigEvent::AlreadyUpToDate => "The configuration is already up to date",
         }
     }
@@ -1562,6 +1554,8 @@ impl StoreEvent {
             StoreEvent::BlobWrite => "Blob write operation",
             StoreEvent::BlobDelete => "Blob delete operation",
             StoreEvent::DataIterate => "Data store iteration operation",
+            StoreEvent::HttpStoreFetch => "HTTP store updated",
+            StoreEvent::HttpStoreError => "Error updating HTTP store",
         }
     }
 
@@ -1597,6 +1591,8 @@ impl StoreEvent {
             StoreEvent::BlobWrite => "A blob write operation was executed",
             StoreEvent::BlobDelete => "A blob delete operation was executed",
             StoreEvent::DataIterate => "A data store iteration operation was executed",
+            StoreEvent::HttpStoreFetch => "The HTTP store was updated",
+            StoreEvent::HttpStoreError => "An error occurred while updating the HTTP store",
         }
     }
 }
