@@ -20,7 +20,7 @@ impl EventType {
                 | StoreEvent::SqlQuery
                 | StoreEvent::LdapQuery
                 | StoreEvent::LdapBind => Level::Trace,
-                StoreEvent::NotFound => Level::Debug,
+                StoreEvent::NotFound | StoreEvent::HttpStoreFetch => Level::Debug,
                 StoreEvent::AssertValueFailed
                 | StoreEvent::FoundationdbError
                 | StoreEvent::MysqlError
@@ -41,7 +41,7 @@ impl EventType {
                 | StoreEvent::NotSupported
                 | StoreEvent::UnexpectedError
                 | StoreEvent::CryptoError => Level::Error,
-                StoreEvent::BlobMissingMarker => Level::Warn,
+                StoreEvent::BlobMissingMarker | StoreEvent::HttpStoreError => Level::Warn,
             },
             EventType::Jmap(_) => Level::Debug,
             EventType::Imap(event) => match event {
@@ -77,7 +77,8 @@ impl EventType {
                 | ImapEvent::Thread
                 | ImapEvent::Error
                 | ImapEvent::IdleStart
-                | ImapEvent::IdleStop => Level::Debug,
+                | ImapEvent::IdleStop
+                | ImapEvent::GetQuota => Level::Debug,
                 ImapEvent::RawInput | ImapEvent::RawOutput => Level::Trace,
             },
             EventType::ManageSieve(event) => match event {
@@ -155,8 +156,6 @@ impl EventType {
                 | SmtpEvent::InvalidParameter
                 | SmtpEvent::UnsupportedParameter
                 | SmtpEvent::SyntaxError
-                | SmtpEvent::PipeSuccess
-                | SmtpEvent::PipeError
                 | SmtpEvent::Error => Level::Debug,
                 SmtpEvent::MissingLocalHostname | SmtpEvent::RemoteIdNotFound => Level::Warn,
                 SmtpEvent::ConcurrencyLimitExceeded
@@ -186,6 +185,7 @@ impl EventType {
                 | SmtpEvent::MailboxDoesNotExist
                 | SmtpEvent::RelayNotAllowed
                 | SmtpEvent::RcptTo
+                | SmtpEvent::RcptToGreylisted
                 | SmtpEvent::TooManyInvalidRcpt
                 | SmtpEvent::Vrfy
                 | SmtpEvent::VrfyNotFound
@@ -241,8 +241,7 @@ impl EventType {
                 ConfigEvent::DefaultApplied
                 | ConfigEvent::MissingSetting
                 | ConfigEvent::UnusedSetting
-                | ConfigEvent::AlreadyUpToDate
-                | ConfigEvent::ExternalKeyIgnored => Level::Debug,
+                | ConfigEvent::AlreadyUpToDate => Level::Debug,
                 ConfigEvent::ParseWarning | ConfigEvent::BuildWarning => Level::Warn,
                 ConfigEvent::ImportExternal => Level::Info,
             },
@@ -269,9 +268,9 @@ impl EventType {
                 PurgeEvent::Finished => Level::Debug,
                 PurgeEvent::Running => Level::Info,
                 PurgeEvent::Error => Level::Error,
-                PurgeEvent::PurgeActive
-                | PurgeEvent::AutoExpunge
-                | PurgeEvent::TombstoneCleanup => Level::Debug,
+                PurgeEvent::InProgress | PurgeEvent::AutoExpunge | PurgeEvent::TombstoneCleanup => {
+                    Level::Debug
+                }
             },
             EventType::Eval(event) => match event {
                 EvalEvent::Error | EvalEvent::StoreNotFound => Level::Debug,
@@ -336,14 +335,15 @@ impl EventType {
                 | SieveEvent::ActionReject => Level::Debug,
             },
             EventType::Spam(event) => match event {
-                SpamEvent::PyzorError | SpamEvent::TrainError | SpamEvent::ClassifyError => {
-                    Level::Warn
-                }
-                SpamEvent::Train
+                SpamEvent::PyzorError
+                | SpamEvent::TrainError
+                | SpamEvent::DnsblError
+                | SpamEvent::Pyzor
+                | SpamEvent::Train
                 | SpamEvent::Classify
-                | SpamEvent::NotEnoughTrainingData
-                | SpamEvent::TrainBalance => Level::Debug,
-                SpamEvent::ListUpdated => Level::Info,
+                | SpamEvent::ClassifyError
+                | SpamEvent::TrainBalance
+                | SpamEvent::Dnsbl => Level::Debug,
             },
             EventType::Http(event) => match event {
                 HttpEvent::ConnectionStart | HttpEvent::ConnectionEnd => Level::Debug,
@@ -370,19 +370,15 @@ impl EventType {
                 | ClusterEvent::InvalidPacket => Level::Warn,
             },
             EventType::Housekeeper(event) => match event {
-                HousekeeperEvent::Start
-                | HousekeeperEvent::PurgeAccounts
-                | HousekeeperEvent::PurgeSessions
-                | HousekeeperEvent::PurgeStore
-                | HousekeeperEvent::Stop => Level::Info,
-                HousekeeperEvent::Schedule => Level::Debug,
+                HousekeeperEvent::Start | HousekeeperEvent::Stop => Level::Info,
+                HousekeeperEvent::Run | HousekeeperEvent::Schedule => Level::Debug,
             },
-            EventType::FtsIndex(event) => match event {
-                FtsIndexEvent::Index => Level::Info,
-                FtsIndexEvent::LockBusy => Level::Warn,
-                FtsIndexEvent::BlobNotFound
-                | FtsIndexEvent::Locked
-                | FtsIndexEvent::MetadataNotFound => Level::Debug,
+            EventType::TaskQueue(event) => match event {
+                TaskQueueEvent::Index => Level::Info,
+                TaskQueueEvent::BlobNotFound
+                | TaskQueueEvent::Locked
+                | TaskQueueEvent::BayesTrain
+                | TaskQueueEvent::MetadataNotFound => Level::Debug,
             },
             EventType::Dmarc(_) => Level::Debug,
             EventType::Spf(_) => Level::Debug,
@@ -464,6 +460,7 @@ impl EventType {
                 DeliveryEvent::RawInput | DeliveryEvent::RawOutput => Level::Trace,
             },
             EventType::Queue(event) => match event {
+                QueueEvent::BackPressure => Level::Warn,
                 QueueEvent::QueueMessage
                 | QueueEvent::QueueMessageAuthenticated
                 | QueueEvent::QueueReport
@@ -473,9 +470,7 @@ impl EventType {
                 | QueueEvent::ConcurrencyLimitExceeded
                 | QueueEvent::Rescheduled
                 | QueueEvent::QuotaExceeded => Level::Info,
-                QueueEvent::LockBusy | QueueEvent::Locked | QueueEvent::BlobNotFound => {
-                    Level::Debug
-                }
+                QueueEvent::Locked | QueueEvent::BlobNotFound => Level::Debug,
             },
             EventType::TlsRpt(event) => match event {
                 TlsRptEvent::RecordFetch
@@ -508,10 +503,7 @@ impl EventType {
                 | IncomingReportEvent::DecompressError => Level::Info,
             },
             EventType::OutgoingReport(event) => match event {
-                OutgoingReportEvent::LockBusy
-                | OutgoingReportEvent::LockDeleted
-                | OutgoingReportEvent::Locked
-                | OutgoingReportEvent::NotFound => Level::Info,
+                OutgoingReportEvent::Locked | OutgoingReportEvent::NotFound => Level::Info,
                 OutgoingReportEvent::SpfReport
                 | OutgoingReportEvent::SpfRateLimited
                 | OutgoingReportEvent::DkimReport
