@@ -178,7 +178,7 @@ impl Collector {
             EventType::Queue(QueueEvent::QueueAutogenerated | QueueEvent::QueueDsn) => {
                 QUEUE_COUNT.increment();
             }
-            EventType::FtsIndex(FtsIndexEvent::Index) => {
+            EventType::TaskQueue(TaskQueueEvent::Index) => {
                 MESSAGE_INDEX_TIME.observe(elapsed);
             }
             EventType::Store(StoreEvent::BlobWrite) => {
@@ -456,7 +456,8 @@ impl EventType {
                 | StoreEvent::DataIterate
                 | StoreEvent::BlobRead
                 | StoreEvent::BlobWrite
-                | StoreEvent::BlobDelete,
+                | StoreEvent::BlobDelete
+                | StoreEvent::HttpStoreError,
             ) => true,
             EventType::MessageIngest(_) => true,
             EventType::Jmap(
@@ -577,12 +578,11 @@ impl EventType {
             ) => true,
             EventType::Spam(
                 SpamEvent::PyzorError
-                | SpamEvent::ListUpdated
                 | SpamEvent::Train
                 | SpamEvent::TrainError
                 | SpamEvent::Classify
                 | SpamEvent::ClassifyError
-                | SpamEvent::NotEnoughTrainingData,
+                | SpamEvent::DnsblError,
             ) => true,
             EventType::PushSubscription(_) => true,
             EventType::Cluster(
@@ -595,10 +595,10 @@ impl EventType {
                 | ClusterEvent::Error,
             ) => true,
             EventType::Housekeeper(_) => false,
-            EventType::FtsIndex(
-                FtsIndexEvent::Index
-                | FtsIndexEvent::BlobNotFound
-                | FtsIndexEvent::MetadataNotFound,
+            EventType::TaskQueue(
+                TaskQueueEvent::Index
+                | TaskQueueEvent::BlobNotFound
+                | TaskQueueEvent::MetadataNotFound,
             ) => true,
             EventType::Milter(
                 MilterEvent::ActionAccept
