@@ -174,9 +174,9 @@ impl Value {
             }
             Value::Event(v) => {
                 buf.push(11u8);
-                leb128_write(buf, v.inner.code());
-                leb128_write(buf, v.keys.len() as u64);
-                for (k, v) in &v.keys {
+                leb128_write(buf, v.0.inner.code());
+                leb128_write(buf, v.0.keys.len() as u64);
+                for (k, v) in &v.0.keys {
                     leb128_write(buf, k.code());
                     v.serialize(buf);
                 }
@@ -258,7 +258,9 @@ impl Value {
                     let value = Value::deserialize(iter)?;
                     keys.push((key, value));
                 }
-                Some(Value::Event(Event::with_keys(code, keys)))
+                Some(Value::Event(Error(
+                    Event::with_keys(code, keys).into_boxed(),
+                )))
             }
             12 => {
                 let len = leb128_read(iter)?;
@@ -362,7 +364,6 @@ impl EventType {
             EventType::Config(ConfigEvent::BuildError) => 54,
             EventType::Config(ConfigEvent::BuildWarning) => 55,
             EventType::Config(ConfigEvent::DefaultApplied) => 56,
-            EventType::Config(ConfigEvent::ExternalKeyIgnored) => 57,
             EventType::Config(ConfigEvent::FetchError) => 58,
             EventType::Config(ConfigEvent::ImportExternal) => 59,
             EventType::Config(ConfigEvent::MacroError) => 60,
@@ -446,14 +447,12 @@ impl EventType {
             EventType::Eval(EvalEvent::Error) => 138,
             EventType::Eval(EvalEvent::Result) => 139,
             EventType::Eval(EvalEvent::StoreNotFound) => 140,
-            EventType::FtsIndex(FtsIndexEvent::BlobNotFound) => 141,
-            EventType::FtsIndex(FtsIndexEvent::Index) => 142,
-            EventType::FtsIndex(FtsIndexEvent::LockBusy) => 143,
-            EventType::FtsIndex(FtsIndexEvent::Locked) => 144,
-            EventType::FtsIndex(FtsIndexEvent::MetadataNotFound) => 145,
-            EventType::Housekeeper(HousekeeperEvent::PurgeAccounts) => 146,
-            EventType::Housekeeper(HousekeeperEvent::PurgeSessions) => 147,
-            EventType::Housekeeper(HousekeeperEvent::PurgeStore) => 148,
+            EventType::TaskQueue(TaskQueueEvent::BlobNotFound) => 141,
+            EventType::TaskQueue(TaskQueueEvent::Index) => 142,
+            EventType::TaskQueue(TaskQueueEvent::BayesTrain) => 143,
+            EventType::TaskQueue(TaskQueueEvent::Locked) => 144,
+            EventType::TaskQueue(TaskQueueEvent::MetadataNotFound) => 145,
+            EventType::Housekeeper(HousekeeperEvent::Run) => 146,
             EventType::Housekeeper(HousekeeperEvent::Schedule) => 149,
             EventType::Housekeeper(HousekeeperEvent::Start) => 150,
             EventType::Housekeeper(HousekeeperEvent::Stop) => 151,
@@ -640,8 +639,6 @@ impl EventType {
             EventType::OutgoingReport(OutgoingReportEvent::DmarcRateLimited) => 332,
             EventType::OutgoingReport(OutgoingReportEvent::DmarcReport) => 333,
             EventType::OutgoingReport(OutgoingReportEvent::HttpSubmission) => 334,
-            EventType::OutgoingReport(OutgoingReportEvent::LockBusy) => 335,
-            EventType::OutgoingReport(OutgoingReportEvent::LockDeleted) => 336,
             EventType::OutgoingReport(OutgoingReportEvent::Locked) => 337,
             EventType::OutgoingReport(OutgoingReportEvent::NoRecipientsFound) => 338,
             EventType::OutgoingReport(OutgoingReportEvent::NotFound) => 339,
@@ -672,7 +669,7 @@ impl EventType {
             EventType::Purge(PurgeEvent::AutoExpunge) => 364,
             EventType::Purge(PurgeEvent::Error) => 365,
             EventType::Purge(PurgeEvent::Finished) => 366,
-            EventType::Purge(PurgeEvent::PurgeActive) => 367,
+            EventType::Purge(PurgeEvent::InProgress) => 367,
             EventType::Purge(PurgeEvent::Running) => 368,
             EventType::Purge(PurgeEvent::Started) => 369,
             EventType::Purge(PurgeEvent::TombstoneCleanup) => 370,
@@ -681,7 +678,6 @@ impl EventType {
             EventType::PushSubscription(PushSubscriptionEvent::Success) => 373,
             EventType::Queue(QueueEvent::BlobNotFound) => 374,
             EventType::Queue(QueueEvent::ConcurrencyLimitExceeded) => 375,
-            EventType::Queue(QueueEvent::LockBusy) => 376,
             EventType::Queue(QueueEvent::Locked) => 377,
             EventType::Queue(QueueEvent::QueueAutogenerated) => 378,
             EventType::Queue(QueueEvent::QueueDsn) => 379,
@@ -763,8 +759,6 @@ impl EventType {
             EventType::Smtp(SmtpEvent::MtPriorityInvalid) => 455,
             EventType::Smtp(SmtpEvent::MultipleMailFrom) => 456,
             EventType::Smtp(SmtpEvent::Noop) => 457,
-            EventType::Smtp(SmtpEvent::PipeError) => 458,
-            EventType::Smtp(SmtpEvent::PipeSuccess) => 459,
             EventType::Smtp(SmtpEvent::Quit) => 460,
             EventType::Smtp(SmtpEvent::RateLimitExceeded) => 461,
             EventType::Smtp(SmtpEvent::RawInput) => 462,
@@ -797,8 +791,8 @@ impl EventType {
             EventType::Smtp(SmtpEvent::VrfyNotFound) => 489,
             EventType::Spam(SpamEvent::Classify) => 490,
             EventType::Spam(SpamEvent::ClassifyError) => 491,
-            EventType::Spam(SpamEvent::ListUpdated) => 492,
-            EventType::Spam(SpamEvent::NotEnoughTrainingData) => 493,
+            EventType::Store(StoreEvent::HttpStoreFetch) => 492,
+            EventType::Store(StoreEvent::HttpStoreError) => 493,
             EventType::Spam(SpamEvent::PyzorError) => 494,
             EventType::Spam(SpamEvent::Train) => 495,
             EventType::Spam(SpamEvent::TrainBalance) => 496,
@@ -866,6 +860,12 @@ impl EventType {
             EventType::Security(SecurityEvent::ScanBan) => 558,
             EventType::Store(StoreEvent::AzureError) => 559,
             EventType::TlsRpt(TlsRptEvent::RecordNotFound) => 560,
+            EventType::Smtp(SmtpEvent::RcptToGreylisted) => 561,
+            EventType::Spam(SpamEvent::Dnsbl) => 562,
+            EventType::Spam(SpamEvent::DnsblError) => 563,
+            EventType::Spam(SpamEvent::Pyzor) => 564,
+            EventType::Queue(QueueEvent::BackPressure) => 48,
+            EventType::Imap(ImapEvent::GetQuota) => 57,
         }
     }
 
@@ -919,7 +919,6 @@ impl EventType {
             45 => Some(EventType::Cluster(ClusterEvent::PeerBackOnline)),
             46 => Some(EventType::Cluster(ClusterEvent::PeerDiscovered)),
             47 => Some(EventType::Cluster(ClusterEvent::PeerHasChanges)),
-            48 => Some(EventType::Cluster(ClusterEvent::PeerHasChanges)), // TODO: recycle
             49 => Some(EventType::Cluster(ClusterEvent::PeerLeaving)),
             50 => Some(EventType::Cluster(ClusterEvent::PeerOffline)),
             51 => Some(EventType::Cluster(ClusterEvent::PeerSuspected)),
@@ -928,7 +927,6 @@ impl EventType {
             54 => Some(EventType::Config(ConfigEvent::BuildError)),
             55 => Some(EventType::Config(ConfigEvent::BuildWarning)),
             56 => Some(EventType::Config(ConfigEvent::DefaultApplied)),
-            57 => Some(EventType::Config(ConfigEvent::ExternalKeyIgnored)),
             58 => Some(EventType::Config(ConfigEvent::FetchError)),
             59 => Some(EventType::Config(ConfigEvent::ImportExternal)),
             60 => Some(EventType::Config(ConfigEvent::MacroError)),
@@ -1012,14 +1010,12 @@ impl EventType {
             138 => Some(EventType::Eval(EvalEvent::Error)),
             139 => Some(EventType::Eval(EvalEvent::Result)),
             140 => Some(EventType::Eval(EvalEvent::StoreNotFound)),
-            141 => Some(EventType::FtsIndex(FtsIndexEvent::BlobNotFound)),
-            142 => Some(EventType::FtsIndex(FtsIndexEvent::Index)),
-            143 => Some(EventType::FtsIndex(FtsIndexEvent::LockBusy)),
-            144 => Some(EventType::FtsIndex(FtsIndexEvent::Locked)),
-            145 => Some(EventType::FtsIndex(FtsIndexEvent::MetadataNotFound)),
-            146 => Some(EventType::Housekeeper(HousekeeperEvent::PurgeAccounts)),
-            147 => Some(EventType::Housekeeper(HousekeeperEvent::PurgeSessions)),
-            148 => Some(EventType::Housekeeper(HousekeeperEvent::PurgeStore)),
+            141 => Some(EventType::TaskQueue(TaskQueueEvent::BlobNotFound)),
+            142 => Some(EventType::TaskQueue(TaskQueueEvent::Index)),
+            143 => Some(EventType::TaskQueue(TaskQueueEvent::BayesTrain)),
+            144 => Some(EventType::TaskQueue(TaskQueueEvent::Locked)),
+            145 => Some(EventType::TaskQueue(TaskQueueEvent::MetadataNotFound)),
+            146 => Some(EventType::Housekeeper(HousekeeperEvent::Run)),
             149 => Some(EventType::Housekeeper(HousekeeperEvent::Schedule)),
             150 => Some(EventType::Housekeeper(HousekeeperEvent::Start)),
             151 => Some(EventType::Housekeeper(HousekeeperEvent::Stop)),
@@ -1232,8 +1228,6 @@ impl EventType {
             334 => Some(EventType::OutgoingReport(
                 OutgoingReportEvent::HttpSubmission,
             )),
-            335 => Some(EventType::OutgoingReport(OutgoingReportEvent::LockBusy)),
-            336 => Some(EventType::OutgoingReport(OutgoingReportEvent::LockDeleted)),
             337 => Some(EventType::OutgoingReport(OutgoingReportEvent::Locked)),
             338 => Some(EventType::OutgoingReport(
                 OutgoingReportEvent::NoRecipientsFound,
@@ -1274,7 +1268,7 @@ impl EventType {
             364 => Some(EventType::Purge(PurgeEvent::AutoExpunge)),
             365 => Some(EventType::Purge(PurgeEvent::Error)),
             366 => Some(EventType::Purge(PurgeEvent::Finished)),
-            367 => Some(EventType::Purge(PurgeEvent::PurgeActive)),
+            367 => Some(EventType::Purge(PurgeEvent::InProgress)),
             368 => Some(EventType::Purge(PurgeEvent::Running)),
             369 => Some(EventType::Purge(PurgeEvent::Started)),
             370 => Some(EventType::Purge(PurgeEvent::TombstoneCleanup)),
@@ -1283,7 +1277,6 @@ impl EventType {
             373 => Some(EventType::PushSubscription(PushSubscriptionEvent::Success)),
             374 => Some(EventType::Queue(QueueEvent::BlobNotFound)),
             375 => Some(EventType::Queue(QueueEvent::ConcurrencyLimitExceeded)),
-            376 => Some(EventType::Queue(QueueEvent::LockBusy)),
             377 => Some(EventType::Queue(QueueEvent::Locked)),
             378 => Some(EventType::Queue(QueueEvent::QueueAutogenerated)),
             379 => Some(EventType::Queue(QueueEvent::QueueDsn)),
@@ -1365,8 +1358,6 @@ impl EventType {
             455 => Some(EventType::Smtp(SmtpEvent::MtPriorityInvalid)),
             456 => Some(EventType::Smtp(SmtpEvent::MultipleMailFrom)),
             457 => Some(EventType::Smtp(SmtpEvent::Noop)),
-            458 => Some(EventType::Smtp(SmtpEvent::PipeError)),
-            459 => Some(EventType::Smtp(SmtpEvent::PipeSuccess)),
             460 => Some(EventType::Smtp(SmtpEvent::Quit)),
             461 => Some(EventType::Smtp(SmtpEvent::RateLimitExceeded)),
             462 => Some(EventType::Smtp(SmtpEvent::RawInput)),
@@ -1399,8 +1390,8 @@ impl EventType {
             489 => Some(EventType::Smtp(SmtpEvent::VrfyNotFound)),
             490 => Some(EventType::Spam(SpamEvent::Classify)),
             491 => Some(EventType::Spam(SpamEvent::ClassifyError)),
-            492 => Some(EventType::Spam(SpamEvent::ListUpdated)),
-            493 => Some(EventType::Spam(SpamEvent::NotEnoughTrainingData)),
+            492 => Some(EventType::Store(StoreEvent::HttpStoreFetch)),
+            493 => Some(EventType::Store(StoreEvent::HttpStoreError)),
             494 => Some(EventType::Spam(SpamEvent::PyzorError)),
             495 => Some(EventType::Spam(SpamEvent::Train)),
             496 => Some(EventType::Spam(SpamEvent::TrainBalance)),
@@ -1472,11 +1463,19 @@ impl EventType {
             558 => Some(EventType::Security(SecurityEvent::ScanBan)),
             559 => Some(EventType::Store(StoreEvent::AzureError)),
             560 => Some(EventType::TlsRpt(TlsRptEvent::RecordNotFound)),
+            561 => Some(EventType::Smtp(SmtpEvent::RcptToGreylisted)),
+            562 => Some(EventType::Spam(SpamEvent::Dnsbl)),
+            563 => Some(EventType::Spam(SpamEvent::DnsblError)),
+            564 => Some(EventType::Spam(SpamEvent::Pyzor)),
+            48 => Some(EventType::Queue(QueueEvent::BackPressure)),
+            57 => Some(EventType::Imap(ImapEvent::GetQuota)),
             _ => None,
         }
     }
 }
 
+// 147 148 335 336 376 458 459
+
 impl Key {
     fn code(&self) -> u64 {
         match self {
