@@ -75,6 +75,9 @@ main() {
     ensure mkdir -p "$_dir"
     ensure downloader "$_url" "$_file" "$_arch"
     ensure tar zxvf "$_file" -C "$_dir/bin"
+    if [ "$_component" = "stalwart-mail-foundationdb" ]; then
+        ignore mv "$_dir/bin/stalwart-mail-foundationdb" "$_dir/bin/stalwart-mail"
+    fi
     ignore chmod +x "$_dir/bin/stalwart-mail"
     ignore rm "$_file"
 
@@ -127,7 +130,7 @@ main() {
     fi
 
     # Installation complete
-    local _host=$(hostname)
+    local _host=$(hostname -f)
     say "🎉 Installation complete! Continue the setup at http://$_host:8080/login"
 
     return 0
@@ -138,7 +141,7 @@ create_service_linux() {
     local _dir="$1"
     cat <<EOF | sed "s|__PATH__|$_dir|g" > /etc/systemd/system/stalwart-mail.service
 [Unit]
-Description=Stalwart Mail Server Server
+Description=Stalwart Mail Server
 Conflicts=postfix.service sendmail.service exim4.service
 ConditionPathExists=__PATH__/etc/config.toml
 After=network-online.target
