@@ -21,7 +21,7 @@ use rustls::ServerConfig;
 use rustls_pemfile::{certs, pkcs8_private_keys};
 use rustls_pki_types::PrivateKeyDer;
 use std::{borrow::Cow, io::BufReader, sync::Arc};
-use store::{LookupStore, Store, Stores};
+use store::{Store, Stores};
 use tokio_rustls::TlsAcceptor;
 
 use crate::{store::TempDir, AssertConfig};
@@ -307,7 +307,7 @@ fields.full-name = "name"
 "#;
 
 pub struct DirectoryStore {
-    pub store: LookupStore,
+    pub store: Store,
 }
 
 pub struct DirectoryTest {
@@ -352,7 +352,7 @@ impl DirectoryTest {
                 config_file.replace("type = \"memory\"", "type = \"memory\"\ndisable = true")
         }
         let mut config = utils::config::Config::new(&config_file).unwrap();
-        let stores = Stores::parse_all(&mut config).await;
+        let stores = Stores::parse_all(&mut config, false).await;
         let directories = Directories::parse(
             &mut config,
             &stores,
