@@ -7,6 +7,7 @@
 pub mod acl;
 pub mod append;
 pub mod basic;
+pub mod bayes;
 pub mod body_structure;
 pub mod condstore;
 pub mod copy_move;
@@ -33,7 +34,7 @@ use common::{
     },
     core::BuildServer,
     manager::boot::build_ipc,
-    Core, Data, Inner, Server,
+    Caches, Core, Data, Inner, Server,
 };
 
 use ::store::Stores;
@@ -55,7 +56,7 @@ use crate::{
 };
 
 const SERVER: &str = r#"
-[lookup.default]
+[server]
 hostname = "imap.example.org"
 
 [server.listener.imap]
@@ -107,6 +108,16 @@ directory = "'{STORE}'"
 total = 5
 wait = "1ms"
 
+[spam-filter]
+enable = true
+
+[spam-filter.bayes.account]
+enable = true
+
+[spam-filter.bayes.classify]
+balance = "0.0"
+learns = 10
+
 [queue]
 path = "{TMP}"
 hash = 64
@@ -132,6 +143,12 @@ protocol = "smtp"
 enable = false
 allow-invalid-certs = true
 
+[session.data]
+spam-filter = "recipients[0] != 'popper@example.com'"
+
+[session.data.add-headers]
+delivered-to = false
+
 [session.extensions]
 future-release = [ { if = "!is_empty(authenticated_as)", then = "99999999d"},
                    { else = false } ]
@@ -308,18 +325,21 @@ async fn init_imap_tests(store_id: &str, delete_if_exists: bool) -> IMAPTest {
     servers.bind_and_drop_priv(&mut config);
 
     // Build stores
-    let stores = Stores::parse_all(&mut config).await;
+    let stores = Stores::parse_all(&mut config, false).await;
 
     // Parse core
     let tracers = Telemetry::parse(&mut config, &stores);
     let core = Core::parse(&mut config, stores, Default::default()).await;
     let data = Data::parse(&mut config);
+    let cache = Caches::parse(&mut config);
+
     let store = core.storage.data.clone();
-    let (ipc, mut ipc_rxs) = build_ipc();
+    let (ipc, mut ipc_rxs) = build_ipc(&mut config);
     let inner = Arc::new(Inner {
         shared_core: core.into_shared(),
         data,
         ipc,
+        cache,
     });
 
     // Parse acceptors
@@ -409,6 +429,14 @@ async fn init_imap_tests(store_id: &str, delete_if_exists: bool) -> IMAPTest {
             &["popper@example.com"],
         )
         .await;
+    store
+        .create_test_user(
+            "bayes@example.com",
+            "secret",
+            "Thomas Bayes",
+            &["bayes@example.com"],
+        )
+        .await;
     store
         .create_test_group(
             "support@example.com",
@@ -482,6 +510,9 @@ pub async fn imap_tests() {
         imap.assert_read(Type::Untagged, ResponseType::Bye).await;
     }
 
+    // Bayes training
+    bayes::test(&handle).await;
+
     // Run ManageSieve tests
     managesieve::test().await;
 
