@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use jmap::mailbox::{INBOX_ID, TRASH_ID};
+use ::email::mailbox::{INBOX_ID, TRASH_ID};
 use jmap_client::{
     core::{
         error::{MethodError, MethodErrorType},
@@ -664,13 +664,16 @@ pub async fn test(params: &mut JMAPTest) {
     // Add John and Jane to the Sales group
     for name in ["jdoe@example.com", "jane.smith@example.com"] {
         server
-            .core
-            .storage
-            .data
-            .add_to_group(name, "sales@example.com")
+            .increment_token_revision(
+                server
+                    .core
+                    .storage
+                    .data
+                    .add_to_group(name, "sales@example.com")
+                    .await,
+            )
             .await;
     }
-    server.inner.data.access_tokens.clear();
     john_client.refresh_session().await.unwrap();
     jane_client.refresh_session().await.unwrap();
     bill_client.refresh_session().await.unwrap();
@@ -765,13 +768,15 @@ pub async fn test(params: &mut JMAPTest) {
 
     // Remove John from the sales group
     server
-        .core
-        .storage
-        .data
-        .remove_from_group("jdoe@example.com", "sales@example.com")
+        .increment_token_revision(
+            server
+                .core
+                .storage
+                .data
+                .remove_from_group("jdoe@example.com", "sales@example.com")
+                .await,
+        )
         .await;
-    server.inner.data.http_auth_cache.clear();
-    server.inner.data.access_tokens.clear();
     assert_forbidden(
         john_client
             .set_default_account_id(sales_id.to_string())
