@@ -28,6 +28,7 @@ use tokio::{
 use crate::{
     directory::internal::TestInternalDirectory,
     jmap::{assert_is_empty, email_set::assert_email_properties, mailbox::destroy_all_mailboxes},
+    smtp::DnsCache,
 };
 
 use super::JMAPTest;
@@ -68,7 +69,7 @@ pub async fn test(params: &mut JMAPTest) {
     let server = params.server.clone();
     let client = &mut params.client;
     let (mut smtp_rx, smtp_settings) = spawn_mock_smtp_server();
-    server.core.smtp.resolvers.dns.ipv4_add(
+    server.ipv4_add(
         "localhost",
         vec!["127.0.0.1".parse().unwrap()],
         Instant::now() + std::time::Duration::from_secs(10),
@@ -92,6 +93,7 @@ pub async fn test(params: &mut JMAPTest) {
     .to_string();
 
     // Test automatic identity creation
+    client.set_default_account_id(&account_id);
     for (identity_id, email) in [(0u64, "jdoe@example.com"), (1u64, "john.doe@example.com")] {
         let identity = client
             .identity_get(&Id::from(identity_id).to_string(), None)
@@ -104,7 +106,6 @@ pub async fn test(params: &mut JMAPTest) {
 
     // Create an identity without using a valid address should fail
     match client
-        .set_default_account_id(&account_id)
         .identity_create("John Doe", "someaddress@domain.com")
         .await
         .unwrap_err()
@@ -193,8 +194,13 @@ pub async fn test(params: &mut JMAPTest) {
     ));
 
     // Submit a valid message submission
-    let email_body =
-        "From: jdoe@example.com\r\nTo: jane_smith@remote.org\r\nSubject: hey\r\n\r\ntest";
+    let email_body = concat!(
+        "From: jdoe@example.com\r\n",
+        "To: jane_smith@remote.org\r\n",
+        "Bcc: bill@remote.org\r\n",
+        "Subject: hey\r\n\r\n",
+        "test"
+    );
     let email_id = client
         .email_import(
             email_body.as_bytes().to_vec(),
@@ -211,12 +217,13 @@ pub async fn test(params: &mut JMAPTest) {
         .unwrap();
 
     // Confirm that the message has been delivered
+    let email_body = email_body.replace("Bcc: bill@remote.org\r\n", "");
     assert_message_delivery(
         &mut smtp_rx,
         MockMessage::new(
             "<jdoe@example.com>",
-            ["<jane_smith@remote.org>"],
-            email_body,
+            ["<bill@remote.org>", "<jane_smith@remote.org>"],
+            &email_body,
         ),
     )
     .await;
@@ -252,7 +259,7 @@ pub async fn test(params: &mut JMAPTest) {
         .contains(&rcpt_to.as_str()));
 
         assert!(
-            message.message.contains(email_body),
+            message.message.contains(&email_body),
             "Got [{}], Expected[{}]",
             message.message,
             email_body
@@ -303,7 +310,7 @@ pub async fn test(params: &mut JMAPTest) {
         .take_id();
     assert_message_delivery(
         &mut smtp_rx,
-        MockMessage::new("<jdoe@example.com>", ["<tim@foobar.com>"], email_body),
+        MockMessage::new("<jdoe@example.com>", ["<tim@foobar.com>"], &email_body),
     )
     .await;
     expect_nothing(&mut smtp_rx).await;
