@@ -4,18 +4,14 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{
-    fmt::Debug,
-    path::PathBuf,
-    sync::Arc,
-    time::{Duration, Instant},
-};
+use std::{fmt::Debug, path::PathBuf, sync::Arc, time::Duration};
 
 use base64::{
-    engine::general_purpose::{self, STANDARD},
     Engine,
+    engine::general_purpose::{self, STANDARD},
 };
 use common::{
+    Caches, Core, Data, Inner, KV_BAYES_MODEL_GLOBAL, Server,
     auth::AccessToken,
     config::{
         server::{Listeners, ServerProtocol},
@@ -26,31 +22,30 @@ use common::{
         boot::build_ipc,
         config::{ConfigManager, Patterns},
     },
-    Core, Data, Inner, Server,
 };
-use enterprise::{insert_test_metrics, EnterpriseCore};
-use hyper::{header::AUTHORIZATION, Method};
+use enterprise::{EnterpriseCore, insert_test_metrics};
+use hyper::{Method, header::AUTHORIZATION};
 use imap::core::ImapSessionManager;
-use jmap::{api::JmapSessionManager, email::delete::EmailDeletion, SpawnServices};
+use jmap::{SpawnServices, api::JmapSessionManager, email::delete::EmailDeletion};
 use jmap_client::client::{Client, Credentials};
 use jmap_proto::{error::request::RequestError, types::id::Id};
 use managesieve::core::ManageSieveSessionManager;
 use pop3::Pop3SessionManager;
 use reqwest::header;
-use serde::{de::DeserializeOwned, Deserialize, Serialize};
-use smtp::{core::SmtpSessionManager, SpawnQueueManager};
+use serde::{Deserialize, Serialize, de::DeserializeOwned};
+use smtp::{SpawnQueueManager, core::SmtpSessionManager};
 
 use store::{
+    IterateParams, SUBSPACE_PROPERTY, Stores, ValueKey,
     roaring::RoaringBitmap,
-    write::{key::DeserializeBigEndian, AnyKey, FtsQueueClass, ValueClass},
-    IterateParams, Stores, ValueKey, SUBSPACE_PROPERTY,
+    write::{AnyKey, TaskQueueClass, ValueClass, key::DeserializeBigEndian},
 };
 use tokio::sync::watch;
-use utils::{config::Config, map::ttl_dashmap::TtlMap, BlobHash};
-use webhooks::{spawn_mock_webhook_endpoint, MockWebhookEndpoint};
+use utils::{BlobHash, config::Config};
+use webhooks::{MockWebhookEndpoint, spawn_mock_webhook_endpoint};
 
 use crate::{
-    add_test_certs, directory::internal::TestInternalDirectory, store::TempDir, AssertConfig,
+    AssertConfig, add_test_certs, directory::internal::TestInternalDirectory, store::TempDir,
 };
 
 pub mod auth_acl;
@@ -120,7 +115,7 @@ implicit = false
 certificate = "default"
 
 [server.fail2ban]
-authentication = "101/5s"
+authentication = "100/5s"
 
 [authentication]
 rate-limit = "100/2s"
@@ -141,6 +136,12 @@ wait = "1ms"
 mechanisms = "[plain, login, oauthbearer]"
 directory = "'{STORE}'"
 
+[session.data]
+spam-filter = "recipients[0] != 'robert@example.com'"
+
+[session.data.add-headers]
+delivered-to = false
+
 [queue]
 path = "{TMP}"
 hash = 64
@@ -216,9 +217,6 @@ blob = "{STORE}"
 lookup = "{STORE}"
 directory = "{STORE}"
 
-[spam.header]
-is-spam  = "X-Spam-Status: Yes"
-
 [jmap.protocol.get]
 max-objects = 100000
 
@@ -338,6 +336,9 @@ signature-algorithm = "ES256"
 expn = true
 vrfy = true
 
+[spam-filter]
+enable = true
+
 [tracer.console]
 type = "console"
 level = "{LEVEL}"
@@ -371,7 +372,7 @@ pub async fn jmap_tests() {
     .await;
 
     webhooks::test(&mut params).await;
-    email_query::test(&mut params, delete).await;
+    /*email_query::test(&mut params, delete).await;
     email_get::test(&mut params).await;
     email_set::test(&mut params).await;
     email_parse::test(&mut params).await;
@@ -383,7 +384,7 @@ pub async fn jmap_tests() {
     thread_merge::test(&mut params).await;
     mailbox::test(&mut params).await;
     delivery::test(&mut params).await;
-    auth_acl::test(&mut params).await;
+    auth_acl::test(&mut params).await;*/
     auth_limits::test(&mut params).await;
     auth_oauth::test(&mut params).await;
     event_source::test(&mut params).await;
@@ -452,7 +453,7 @@ pub async fn wait_for_index(server: &Server) {
                         account_id: 0,
                         collection: 0,
                         document_id: 0,
-                        class: ValueClass::FtsQueue(FtsQueueClass {
+                        class: ValueClass::TaskQueue(TaskQueueClass::IndexEmail {
                             seq: 0,
                             hash: BlobHash::default(),
                         }),
@@ -461,7 +462,7 @@ pub async fn wait_for_index(server: &Server) {
                         account_id: u32::MAX,
                         collection: u8::MAX,
                         document_id: u32::MAX,
-                        class: ValueClass::FtsQueue(FtsQueueClass {
+                        class: ValueClass::TaskQueue(TaskQueueClass::IndexEmail {
                             seq: u64::MAX,
                             hash: BlobHash::default(),
                         }),
@@ -489,6 +490,13 @@ pub async fn assert_is_empty(server: Server) {
     // Wait for pending FTS index tasks
     wait_for_index(&server).await;
 
+    // Delete bayes model
+    server
+        .in_memory_store()
+        .key_delete_prefix(&[KV_BAYES_MODEL_GLOBAL])
+        .await
+        .unwrap();
+
     // Purge accounts
     emails_purge_tombstoned(&server).await;
 
@@ -529,23 +537,18 @@ pub async fn emails_purge_tombstoned(server: &Server) {
         .unwrap();
 
     for account_id in account_ids {
-        let do_add = server
-            .inner
-            .data
-            .access_tokens
-            .get_with_ttl(&account_id)
-            .is_none();
+        let do_add = server.inner.cache.access_tokens.get(&account_id).is_none();
 
         if do_add {
-            server.inner.data.access_tokens.insert_with_ttl(
-                account_id,
-                Arc::new(AccessToken::from_id(account_id)),
-                Instant::now() + Duration::from_secs(3600),
-            );
+            server
+                .inner
+                .cache
+                .access_tokens
+                .insert(account_id, Arc::new(AccessToken::from_id(account_id)));
         }
         server.emails_purge_tombstoned(account_id).await.unwrap();
         if do_add {
-            server.inner.data.access_tokens.remove(&account_id);
+            server.inner.cache.access_tokens.remove(&account_id);
         }
     }
 }
@@ -572,7 +575,7 @@ async fn init_jmap_tests(store_id: &str, delete_if_exists: bool) -> JMAPTest {
     servers.bind_and_drop_priv(&mut config);
 
     // Build stores
-    let stores = Stores::parse_all(&mut config).await;
+    let stores = Stores::parse_all(&mut config, false).await;
 
     // Parse core
     let config_manager = ConfigManager {
@@ -590,12 +593,14 @@ async fn init_jmap_tests(store_id: &str, delete_if_exists: bool) -> JMAPTest {
         .await
         .enable_enterprise();
     let data = Data::parse(&mut config);
+    let cache = Caches::parse(&mut config);
     let store = core.storage.data.clone();
-    let (ipc, mut ipc_rxs) = build_ipc();
+    let (ipc, mut ipc_rxs) = build_ipc(&mut config);
     let inner = Arc::new(Inner {
         shared_core: core.into_shared(),
         data,
         ipc,
+        cache,
     });
 
     // Parse acceptors
@@ -993,7 +998,7 @@ impl<T: Debug> Response<T> {
 
     pub fn expect_request_error(self, value: &str) {
         let err = self.unwrap_request_error();
-        if !err.detail.contains(value) && !err.title.as_ref().map_or(false, |t| t.contains(value)) {
+        if !err.detail.contains(value) && !err.title.as_ref().is_some_and(|t| t.contains(value)) {
             panic!("Expected request error containing {value:?}, found {err:?}")
         }
     }
@@ -1001,8 +1006,8 @@ impl<T: Debug> Response<T> {
     pub fn expect_error(self, value: &str) {
         let (error, details, reason) = self.unwrap_error();
         if !error.contains(value)
-            && !details.as_ref().map_or(false, |d| d.contains(value))
-            && !reason.as_ref().map_or(false, |r| r.contains(value))
+            && !details.as_ref().is_some_and(|d| d.contains(value))
+            && !reason.as_ref().is_some_and(|r| r.contains(value))
         {
             panic!("Expected error containing {value:?}, found {error:?}: {details:?} {reason:?}")
         }
