@@ -4,16 +4,15 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
+use std::sync::Arc;
+
 use ahash::AHashSet;
-use common::{
-    auth::{AccessToken, TenantInfo},
-    ipc::{DeliveryResult, IngestMessage},
-};
+use common::auth::{AccessToken, TenantInfo};
 use directory::{
     backend::internal::{PrincipalField, PrincipalUpdate, PrincipalValue},
     Permission, Principal, Type,
 };
-use jmap::{services::ingest::MailDelivery, JmapMethods};
+use email::delivery::{IngestMessage, LocalDeliveryStatus, MailDelivery};
 use utils::BlobHash;
 
 use crate::jmap::assert_is_empty;
@@ -48,7 +47,8 @@ pub async fn test(params: &JMAPTest) {
         .unwrap()
         .validate_permissions(
             Permission::all().filter(|p| p.is_user_permission() && *p != Permission::Pop3Dele),
-        );
+        )
+        .validate_revision(0);
 
     // Create multiple roles
     for (role, permissions, parent_role) in &[
@@ -137,7 +137,8 @@ pub async fn test(params: &JMAPTest) {
             Permission::ImapList,
             Permission::Pop3Authenticate,
             Permission::Pop3List,
-        ]);
+        ])
+        .validate_revision(1);
 
     // Query all principals
     api.get::<List<Principal>>("/api/principal")
@@ -611,8 +612,9 @@ pub async fn test(params: &JMAPTest) {
                 message_size: TEST_MESSAGE.len(),
                 session_id: 0,
             })
-            .await,
-        vec![DeliveryResult::PermanentFailure {
+            .await
+            .status,
+        vec![LocalDeliveryStatus::PermanentFailure {
             code: [5, 5, 0],
             reason: "This account is not authorized to receive email.".into()
         }]
@@ -648,8 +650,9 @@ pub async fn test(params: &JMAPTest) {
                 message_size: TEST_MESSAGE.len(),
                 session_id: 0,
             })
-            .await,
-        vec![DeliveryResult::Success]
+            .await
+            .status,
+        vec![LocalDeliveryStatus::Success]
     );
 
     // Quota for the tenant and user should be updated
@@ -672,8 +675,9 @@ pub async fn test(params: &JMAPTest) {
                 message_size: TEST_MESSAGE.len(),
                 session_id: 0,
             })
-            .await,
-        vec![DeliveryResult::TemporaryFailure {
+            .await
+            .status,
+        vec![LocalDeliveryStatus::TemporaryFailure {
             reason: "Organization over quota.".into()
         }]
     );
@@ -733,6 +737,14 @@ pub async fn test(params: &JMAPTest) {
         .unwrap()
         .unwrap_data();
 
+    server
+        .core
+        .storage
+        .config
+        .clear("report.domain")
+        .await
+        .unwrap();
+
     assert_is_empty(server).await;
 }
 
@@ -823,9 +835,10 @@ trait ValidatePermissions {
         expected_permissions: impl IntoIterator<Item = Permission>,
     ) -> Self;
     fn validate_tenant(self, tenant_id: u32, tenant_quota: u64) -> Self;
+    fn validate_revision(self, revision: u64) -> Self;
 }
 
-impl ValidatePermissions for AccessToken {
+impl ValidatePermissions for Arc<AccessToken> {
     fn validate_permissions(
         self,
         expected_permissions: impl IntoIterator<Item = Permission>,
@@ -867,4 +880,9 @@ impl ValidatePermissions for AccessToken {
         );
         self
     }
+
+    fn validate_revision(self, revision: u64) -> Self {
+        assert_eq!(self.revision, revision);
+        self
+    }
 }
