@@ -11,10 +11,8 @@ use crate::{
     store::deflate_test_resource,
 };
 use common::auth::AccessToken;
-use jmap::{
-    email::ingest::{EmailIngest, IngestEmail, IngestSource},
-    JmapMethods,
-};
+
+use ::email::ingest::{EmailIngest, IngestEmail, IngestSource};
 use jmap_client::{email, mailbox::Role};
 use jmap_proto::types::{collection::Collection, id::Id};
 use mail_parser::{mailbox::mbox::MessageIterator, MessageParser};
@@ -250,8 +248,11 @@ async fn test_multi_thread(params: &mut JMAPTest) {
                         mailbox_ids: vec![mailbox_id],
                         keywords: vec![],
                         received_at: None,
-                        source: IngestSource::Smtp,
-                        encrypt: false,
+                        source: IngestSource::Smtp {
+                            deliver_to: "test@domain.org",
+                        },
+                        spam_classify: false,
+                        spam_train: false,
                         session_id: 0,
                     })
                     .await
@@ -260,7 +261,7 @@ async fn test_multi_thread(params: &mut JMAPTest) {
                     Err(err) => {
                         if err.is_assertion_failure() && retry_count < 10 {
                             //println!("Retrying ingest for {}...", message.from());
-                            let backoff = rand::thread_rng().gen_range(50..=300);
+                            let backoff = rand::rng().random_range(50..=300);
                             tokio::time::sleep(Duration::from_millis(backoff)).await;
                             retry_count += 1;
                             continue;
