@@ -9,11 +9,12 @@ use std::{fs, net::IpAddr, path::PathBuf, sync::Arc, time::Duration};
 use common::{
     config::{
         server::{Listener, Listeners, ServerProtocol, TcpListener},
-        smtp::{throttle::parse_throttle, *},
+        smtp::*,
     },
     expr::{functions::ResolveVariable, if_block::*, tokenizer::TokenMap, *},
     Server,
 };
+use throttle::parse_queue_rate_limiter;
 use tokio::net::TcpSocket;
 
 use utils::config::{Config, Rate};
@@ -241,7 +242,7 @@ fn parse_throttles() {
     file.push("throttle.toml");
 
     let mut config = Config::new(fs::read_to_string(file).unwrap()).unwrap();
-    let throttle = parse_throttle(
+    let throttle = parse_queue_rate_limiter(
         &mut config,
         "throttle",
         &TokenMap::default().with_variables(&[
@@ -261,7 +262,7 @@ fn parse_throttles() {
     assert_eq!(
         throttle,
         vec![
-            Throttle {
+            QueueRateLimiter {
                 id: "0000".to_string(),
                 expr: Expression {
                     items: vec![
@@ -271,19 +272,19 @@ fn parse_throttles() {
                     ]
                 },
                 keys: THROTTLE_REMOTE_IP | THROTTLE_AUTH_AS,
-                concurrency: 100.into(),
                 rate: Rate {
                     requests: 50,
                     period: Duration::from_secs(30)
                 }
-                .into()
             },
-            Throttle {
+            QueueRateLimiter {
                 id: "0001".to_string(),
                 expr: Expression::default(),
                 keys: THROTTLE_SENDER_DOMAIN,
-                concurrency: 10000.into(),
-                rate: None
+                rate: Rate {
+                    requests: 50,
+                    period: Duration::from_secs(30)
+                }
             }
         ]
     );
@@ -426,15 +427,18 @@ async fn eval_if() {
         //println!("============= Testing {:?} ==================", key);
         let (_, expected_result) = key.rsplit_once('-').unwrap();
         assert_eq!(
-            IfBlock {
-                key: key.to_string(),
-                if_then: vec![IfThen {
-                    expr: Expression::try_parse(&mut config, key.as_str(), &token_map).unwrap(),
-                    then: Expression::from(true),
-                }],
-                default: Expression::from(false),
-            }
-            .eval(&envelope, &core, 0)
+            core.eval_if::<Variable, _>(
+                &IfBlock {
+                    key: key.to_string(),
+                    if_then: vec![IfThen {
+                        expr: Expression::try_parse(&mut config, key.as_str(), &token_map).unwrap(),
+                        then: Expression::from(true),
+                    }],
+                    default: Expression::from(false),
+                },
+                &envelope,
+                0
+            )
             .await
             .unwrap()
             .to_bool(),
@@ -485,7 +489,7 @@ async fn eval_dynvalue() {
             .unwrap_or_else(|| panic!("Missing expect for test {test_name:?}"));
 
         assert_eq!(
-            String::try_from(if_block.eval(&envelope, &core, 0).await.unwrap()).ok(),
+            core.eval_if::<String, _>(&if_block, &envelope, 0).await,
             expected,
             "failed for test {test_name:?}"
         );
@@ -509,6 +513,10 @@ impl ResolveVariable for TestEnvelope {
             _ => Default::default(),
         }
     }
+
+    fn resolve_global(&self, _: &str) -> Variable<'_> {
+        Variable::Integer(0)
+    }
 }
 
 impl TestEnvelope {
